@@ -243,7 +243,6 @@ int largestComponent(const cvf::Vec3d v)
 
     if (v.z() > maxLength)
     {
-        maxLength = v.z();
         idx = 2;
     }
 
@@ -787,8 +786,11 @@ void AABBTree::deleteInternalNodesBottomUp(AABBTreeNode* node)
     auto internalNode = dynamic_cast<AABBTreeNodeInternal*>(node);
     CVF_ASSERT(internalNode);
 
-    AABBTree::deleteInternalNodesBottomUp(internalNode->left());
-    AABBTree::deleteInternalNodesBottomUp(internalNode->right());
+    if (internalNode)
+    {
+        AABBTree::deleteInternalNodesBottomUp(internalNode->left());
+        AABBTree::deleteInternalNodesBottomUp(internalNode->right());
+    }
 
     delete internalNode;
 }
