@@ -33,7 +33,6 @@
 #include <opm/input/eclipse/Schedule/Action/SimulatorUpdate.hpp>
 #include <opm/input/eclipse/Schedule/UDQ/UDQState.hpp>
 #include <opm/input/eclipse/Schedule/UDQ/UDQConfig.hpp>
-#include <opm/input/eclipse/Schedule/Well/PAvgCalculatorCollection.hpp>
 #include <opm/input/eclipse/Schedule/Well/Well.hpp>
 #include <opm/input/eclipse/Schedule/Well/WellMatcher.hpp>
 #include <opm/input/eclipse/Schedule/Well/WellTestState.hpp>
@@ -122,13 +121,19 @@ void msim::run_step(WellTestState& wtest_state, UDQState& udq_state, data::Solut
                           report_step,
                           seconds_elapsed,
                           well_data,
+                          /* wbp = */ {},
                           group_nwrk_data,
-                          {},
-                          {},
-                          {},
-                          {});
+                          /* sing_values = */ {},
+                          /* initial_inplace = */ {},
+                          /* inplace = */ {});
 
-        this->schedule.getUDQConfig( report_step ).eval(report_step, schedule.wellMatcher(report_step), this->st, udq_state);
+        this->schedule.getUDQConfig(report_step - 1)
+            .eval(report_step,
+                  this->schedule,
+                  this->schedule.wellMatcher(report_step),
+                  this->schedule.segmentMatcherFactory(report_step),
+                  this->st,
+                  udq_state);
 
         this->output(wtest_state,
                      udq_state,
@@ -144,7 +149,10 @@ void msim::run_step(WellTestState& wtest_state, UDQState& udq_state, data::Solut
 
 
 
-void msim::output(WellTestState& wtest_state, const UDQState& udq_state, size_t report_step, bool substep, double seconds_elapsed, const data::Solution& sol, const data::Wells& well_data, const data::GroupAndNetworkValues& group_nwrk_data, EclipseIO& io) {
+void msim::output(const WellTestState& wtest_state, const UDQState& udq_state,
+                  size_t report_step, bool substep, double seconds_elapsed,
+                  const data::Solution& sol, const data::Wells& well_data,
+                  const data::GroupAndNetworkValues& group_nwrk_data, EclipseIO& io) {
     RestartValue value(sol, well_data, group_nwrk_data, {});
     io.writeTimeStep(this->action_state,
                      wtest_state,
