@@ -28,11 +28,13 @@
 #include <opm/input/eclipse/EclipseState/EclipseConfig.hpp>
 #include <opm/input/eclipse/EclipseState/TracerConfig.hpp>
 #include <opm/input/eclipse/EclipseState/MICPpara.hpp>
+#include <opm/input/eclipse/EclipseState/WagHysteresisConfig.hpp>
 #include <opm/input/eclipse/EclipseState/Grid/FieldPropsManager.hpp>
 #include <opm/input/eclipse/EclipseState/Grid/EclipseGrid.hpp>
 #include <opm/input/eclipse/EclipseState/Grid/FaultCollection.hpp>
 #include <opm/input/eclipse/EclipseState/Grid/NNC.hpp>
 #include <opm/input/eclipse/EclipseState/Grid/TransMult.hpp>
+#include <opm/input/eclipse/EclipseState/Grid/LgrCollection.hpp>
 #include <opm/input/eclipse/EclipseState/Runspec.hpp>
 #include <opm/input/eclipse/EclipseState/Tables/TableManager.hpp>
 #include <opm/input/eclipse/EclipseState/SimulationConfig/SimulationConfig.hpp>
@@ -94,6 +96,9 @@ namespace Opm {
         const EclipseConfig& cfg() const;
         const GridDims& gridDims() const;
 
+        const LgrCollection& getLgrs() const;
+        bool hasInputLGR() const;
+
         // the unit system used by the deck. note that it is rarely needed
         // to convert units because internally to opm-parser everything is
         // represented by SI units.
@@ -108,6 +113,7 @@ namespace Opm {
         const AquiferConfig& aquifer() const;
         const TracerConfig& tracer() const;
         const MICPpara& getMICPpara() const;
+        const WagHysteresisConfig& getWagHysteresis() const;
 
         void reset_actnum(const std::vector<int>& new_actnum);
         void pruneDeactivatedAquiferConnections(const std::vector<std::size_t>& deactivated_cells);
@@ -129,6 +135,7 @@ namespace Opm {
             serializer(m_deckUnitSystem);
             serializer(m_inputNnc);
             serializer(m_gridDims);
+            serializer(m_lgrs);
             serializer(m_simulationConfig);
             serializer(aquifer_config);
             serializer(m_transMult);
@@ -136,6 +143,7 @@ namespace Opm {
             serializer(m_title);
             serializer(tracer_config);
             serializer(m_micppara);
+            serializer(wag_hyst_config);
         }
 
         static bool rst_cmp(const EclipseState& full_state, const EclipseState& rst_state);
@@ -145,12 +153,13 @@ namespace Opm {
         void initIOConfigPostSchedule(const Deck& deck);
         void assignRunTitle(const Deck& deck);
         void reportNumberOfActivePhases() const;
+        void initLgrs(const Deck& deck);
         void conveyNumericalAquiferEffects();
         void applyMULTXYZ();
         void initFaults(const Deck& deck);
         void initPara(const Deck& deck);
 
-        void setMULTFLT(const Opm::DeckSection& section);
+        void setMULTFLT(const Opm::DeckSection& section, bool edit = false);
 
         void complainAboutAmbiguousKeyword(const Deck& deck,
                                            const std::string& keywordName);
@@ -164,11 +173,13 @@ namespace Opm {
         NNC m_inputNnc;
         GridDims m_gridDims;
         FieldPropsManager field_props;
+        LgrCollection m_lgrs;
         SimulationConfig m_simulationConfig;
         AquiferConfig aquifer_config;
         TransMult m_transMult;
         TracerConfig tracer_config;
         MICPpara m_micppara;
+        WagHysteresisConfig wag_hyst_config;
 
         std::string m_title{};
         FaultCollection m_faults{};
