@@ -17,23 +17,31 @@
   along with OPM.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-
 #ifndef OPM_PARSER_MULTREGTSCANNER_HPP
 #define OPM_PARSER_MULTREGTSCANNER_HPP
 
 #include <opm/input/eclipse/EclipseState/Grid/FaceDir.hpp>
+#include <opm/input/eclipse/EclipseState/Grid/GridDims.hpp>
+
+#include <cstddef>
+#include <map>
+#include <string>
+#include <utility>
+#include <vector>
 
 namespace Opm {
 
     class DeckRecord;
     class DeckKeyword;
     class FieldPropsManager;
-    class GridDims;
+
+} // namespace Opm
+
+namespace Opm {
 
     namespace MULTREGT {
-
-
-        enum NNCBehaviourEnum {
+        enum class NNCBehaviourEnum
+        {
             NNC = 1,
             NONNC = 2,
             ALL = 3,
@@ -42,12 +50,10 @@ namespace Opm {
 
         std::string RegionNameFromDeckValue(const std::string& stringValue);
         NNCBehaviourEnum NNCBehaviourFromString(const std::string& stringValue);
-    }
+    } // namespace MULTREGT
 
-
-
-
-    struct MULTREGTRecord {
+    struct MULTREGTRecord
+    {
         int src_value;
         int target_value;
         double trans_mult;
@@ -55,13 +61,15 @@ namespace Opm {
         MULTREGT::NNCBehaviourEnum nnc_behaviour;
         std::string region_name;
 
-        bool operator==(const MULTREGTRecord& data) const {
-            return src_value == data.src_value &&
-                   target_value == data.target_value &&
-                   trans_mult == data.trans_mult &&
-                   directions == data.directions &&
-                   nnc_behaviour == data.nnc_behaviour &&
-                   region_name == data.region_name;
+        bool operator==(const MULTREGTRecord& data) const
+        {
+            return (src_value == data.src_value)
+                && (target_value == data.target_value)
+                && (trans_mult == data.trans_mult)
+                && (directions == data.directions)
+                && (nnc_behaviour == data.nnc_behaviour)
+                && (region_name == data.region_name)
+                ;
         }
 
         template<class Serializer>
@@ -76,58 +84,62 @@ namespace Opm {
         }
     };
 
-    typedef std::map< std::pair<int , int> , const MULTREGTRecord * >  MULTREGTSearchMap;
-    typedef std::tuple<size_t , FaceDir::DirEnum , double> MULTREGTConnection;
-
-
-
-    class MULTREGTScanner {
-
+    class MULTREGTScanner
+    {
     public:
-        using ExternalSearchMap = std::map<std::string, std::map<std::pair<int,int>, int>>;
-
         MULTREGTScanner() = default;
         MULTREGTScanner(const MULTREGTScanner& data);
         MULTREGTScanner(const GridDims& grid,
                         const FieldPropsManager* fp_arg,
-                        const std::vector< const DeckKeyword* >& keywords);
+                        const std::vector<const DeckKeyword*>& keywords);
 
         static MULTREGTScanner serializationTestObject();
 
-        double getRegionMultiplier(size_t globalCellIdx1, size_t globalCellIdx2, FaceDir::DirEnum faceDir) const;
-
         bool operator==(const MULTREGTScanner& data) const;
         MULTREGTScanner& operator=(const MULTREGTScanner& data);
 
-        template<class Serializer>
+        void applyNumericalAquifer(const std::vector<std::size_t>& aquifer_cells);
+
+        double getRegionMultiplier(std::size_t globalCellIdx1,
+                                   std::size_t globalCellIdx2,
+                                   FaceDir::DirEnum faceDir) const;
+
+        double getRegionMultiplierNNC(std::size_t globalCellIdx1,
+                                      std::size_t globalCellIdx2) const;
+
+        template <class Serializer>
         void serializeOp(Serializer& serializer)
         {
-            serializer(nx);
-            serializer(ny);
-            serializer(nz);
+            serializer(gridDims);
+
             serializer(m_records);
-            ExternalSearchMap searchMap = getSearchMap();
-            serializer(searchMap);
-            if (m_searchMap.empty())
-                constructSearchMap(searchMap);
+            serializer(m_searchMap);
+
             serializer(regions);
-            serializer(default_region);
+            serializer(aquifer_cells);
         }
 
     private:
-        ExternalSearchMap getSearchMap() const;
-        void constructSearchMap(const ExternalSearchMap& searchMap);
+        using MULTREGTSearchMap = std::map<
+            std::pair<int, int>,
+            std::vector<MULTREGTRecord>::size_type
+        >;
 
-        void addKeyword( const DeckKeyword& deckKeyword, const std::string& defaultRegion);
+        GridDims gridDims{};
+        const FieldPropsManager* fp{nullptr};
+
+        std::vector<MULTREGTRecord> m_records{};
+        std::map<std::string, MULTREGTSearchMap> m_searchMap{};
+        std::map<std::string, std::vector<int>> regions{};
+        std::vector<std::size_t> aquifer_cells{};
+
+        void addKeyword(const DeckKeyword& deckKeyword);
         void assertKeywordSupported(const DeckKeyword& deckKeyword);
-        std::size_t nx = 0,ny = 0, nz = 0;
-        const FieldPropsManager* fp = nullptr;
-        std::vector< MULTREGTRecord > m_records;
-        std::map<std::string , MULTREGTSearchMap> m_searchMap;
-        std::map<std::string, std::vector<int>> regions;
-        std::string default_region;
+
+        bool isAquNNC(std::size_t globalCellIdx1, std::size_t globalCellIdx2) const;
+        bool isAquCell(std::size_t globalCellIdx) const;
     };
 
-}
+} // namespace Opm
 
 #endif // OPM_PARSER_MULTREGTSCANNER_HPP
