@@ -31,7 +31,6 @@
 #include <string>
 
 namespace Opm {
-
 class Deck;
 
 class Phases {
@@ -95,7 +94,8 @@ public:
         return this->nDynWlistMax;
     }
 
-    const std::optional<KeywordLocation>& location() const {
+    const std::optional<KeywordLocation>& location() const
+    {
         return this->m_location;
     }
 
@@ -113,7 +113,6 @@ public:
             rst_cmp(*this, data);
     }
 
-
     template<class Serializer>
     void serializeOp(Serializer& serializer)
     {
@@ -143,7 +142,6 @@ public:
 
     static WellSegmentDims serializationTestObject();
 
-
     int maxSegmentedWells() const
     {
         return this->nSegWellMax;
@@ -159,6 +157,11 @@ public:
         return this->nLatBranchMax;
     }
 
+    const std::optional<KeywordLocation>& location() const
+    {
+        return this->location_;
+    }
+
     bool operator==(const WellSegmentDims& data) const;
 
     template<class Serializer>
@@ -167,12 +170,14 @@ public:
         serializer(nSegWellMax);
         serializer(nSegmentMax);
         serializer(nLatBranchMax);
+        serializer(location_);
     }
 
 private:
     int nSegWellMax;
     int nSegmentMax;
     int nLatBranchMax;
+    std::optional<KeywordLocation> location_;
 };
 
 class NetworkDims {
@@ -294,6 +299,11 @@ public:
      */
     double curvatureCapPrs() const;
 
+    /*!
+     * \brief Wag hysteresis.
+     */
+    bool activeWag() const;
+
     bool operator==(const EclHysterConfig& data) const;
 
     template<class Serializer>
@@ -304,6 +314,7 @@ public:
         serializer(krHystMod);
         serializer(modParamTrappedValue);
         serializer(curvatureCapPrsValue);
+        serializer(activeWagHyst);
     }
 
 private:
@@ -317,6 +328,9 @@ private:
     double modParamTrappedValue { 0.1 };
     // curvature parameter for capillary pressure
     double curvatureCapPrsValue { 0.1 };
+
+    // enable WAG hysteresis
+    bool activeWagHyst  { false };
 };
 
 class SatFuncControls {
@@ -329,8 +343,9 @@ public:
 
     enum class KeywordFamily {
         Family_I,               // SGOF, SWOF, SLGOF
-        Family_II,              // SGFN, SOF{2,3}, SWFN
-        Family_III,              // GSF, WSF
+        Family_II,              // SGFN, SOF{2,3}, SWFN, SGWFN
+        Family_III,             // GSF, WSF
+
         Undefined,
     };
 
@@ -455,7 +470,9 @@ public:
     const Nupcol& nupcol() const noexcept;
     const Tracers& tracers() const;
     bool co2Storage() const noexcept;
+    bool h2Storage() const noexcept;
     bool micp() const noexcept;
+    bool mech() const noexcept;
 
     bool operator==(const Runspec& data) const;
     static bool rst_cmp(const Runspec& full_state, const Runspec& rst_state);
@@ -478,7 +495,9 @@ public:
         serializer(m_sfuncctrl);
         serializer(m_nupcol);
         serializer(m_co2storage);
+        serializer(m_h2storage);
         serializer(m_micp);
+        serializer(m_mech);
     }
 
 private:
@@ -498,7 +517,9 @@ private:
     Nupcol m_nupcol;
     Tracers m_tracers;
     bool m_co2storage;
+    bool m_h2storage;
     bool m_micp;
+    bool m_mech;
 };
 
 
