@@ -174,6 +174,31 @@ struct GroupInjectionProperties {
     }
 };
 
+struct GroupLimitAction
+{
+    ExceedAction allRates{ExceedAction::NONE};
+    ExceedAction water{ExceedAction::NONE};
+    ExceedAction gas{ExceedAction::NONE};
+    ExceedAction liquid{ExceedAction::NONE};
+
+    template<class Serializer>
+    void serializeOp(Serializer& serializer)
+    {
+        serializer(allRates);
+        serializer(water);
+        serializer(gas);
+        serializer(liquid); 
+    }
+
+    bool operator==(const GroupLimitAction& other) const
+    {
+        return (this->allRates == other.allRates)
+            && (this->water == other.water)
+            && (this->gas == other.gas)
+            && (this->liquid == other.liquid);
+    }
+};
+
 struct InjectionControls {
     Phase phase;
     InjectionCMode cmode;
@@ -194,7 +219,7 @@ struct GroupProductionProperties {
 
     std::string name;
     ProductionCMode cmode = ProductionCMode::NONE;
-    ExceedAction exceed_action = ExceedAction::NONE;
+    GroupLimitAction group_limit_action;
     UDAValue oil_target;
     UDAValue water_target;
     UDAValue gas_target;
@@ -216,7 +241,7 @@ struct GroupProductionProperties {
     {
         serializer(name);
         serializer(cmode);
-        serializer(exceed_action);
+        serializer(group_limit_action);
         serializer(oil_target);
         serializer(water_target);
         serializer(gas_target);
@@ -232,7 +257,7 @@ struct GroupProductionProperties {
 
 struct ProductionControls {
     ProductionCMode cmode;
-    ExceedAction exceed_action;
+    GroupLimitAction group_limit_action;
     double oil_target;
     double water_target;
     double gas_target;
@@ -253,9 +278,7 @@ struct ProductionControls {
     std::size_t insert_index() const;
     const std::string& name() const;
     bool is_field() const;
-    int getGroupNetVFPTable() const;
 
-    bool updateNetVFPTable(int vfp_arg);
     bool update_gefac(double gefac, bool transfer_gefac);
 
     // [[deprecated("use Group::control_group() or Group::flow_group()")]]
@@ -317,7 +340,6 @@ struct ProductionControls {
         serializer(group_type);
         serializer(gefac);
         serializer(transfer_gefac);
-        serializer(vfp_table);
         serializer(parent_group);
         serializer(m_wells);
         serializer(m_groups);
@@ -338,7 +360,6 @@ private:
     GroupType group_type;
     double gefac;
     bool transfer_gefac;
-    int vfp_table;
 
     std::string parent_group;
     IOrderSet<std::string> m_wells;
