@@ -35,6 +35,7 @@
 #include <opm/input/eclipse/Schedule/Tuning.hpp>
 #include <opm/input/eclipse/Schedule/OilVaporizationProperties.hpp>
 #include <opm/input/eclipse/Schedule/Events.hpp>
+#include <opm/input/eclipse/Schedule/BCProp.hpp>
 #include <opm/input/eclipse/Schedule/Group/Group.hpp>
 #include <opm/input/eclipse/Schedule/Well/WellEnums.hpp>
 #include <opm/input/eclipse/Schedule/MessageLimits.hpp>
@@ -63,6 +64,7 @@ namespace Opm {
     class GasLiftOpt;
     class GConSale;
     class GConSump;
+    class GroupEconProductionLimits;
     class GroupOrder;
     class GuideRateConfig;
     class NameOrder;
@@ -358,6 +360,7 @@ namespace Opm {
 
         ptr_member<GConSale> gconsale;
         ptr_member<GConSump> gconsump;
+        ptr_member<GroupEconProductionLimits> gecon;
         ptr_member<GuideRateConfig> guide_rate;
 
         ptr_member<WListManager> wlist_manager;
@@ -390,6 +393,8 @@ namespace Opm {
                                   return this->gconsale;
             else if constexpr ( std::is_same_v<T, GConSump> )
                                   return this->gconsump;
+            else if constexpr ( std::is_same_v<T, GroupEconProductionLimits> )
+                                  return this->gecon;
             else if constexpr ( std::is_same_v<T, WListManager> )
                                   return this->wlist_manager;
             else if constexpr ( std::is_same_v<T, Network::ExtNetwork> )
@@ -430,6 +435,8 @@ namespace Opm {
                                   return this->gconsale;
             else if constexpr ( std::is_same_v<T, GConSump> )
                                   return this->gconsump;
+            else if constexpr ( std::is_same_v<T, GroupEconProductionLimits> )
+                                  return this->gecon;
             else if constexpr ( std::is_same_v<T, WListManager> )
                                   return this->wlist_manager;
             else if constexpr ( std::is_same_v<T, Network::ExtNetwork> )
@@ -482,6 +489,7 @@ namespace Opm {
         map_member<std::string, Well> wells;
         // constant flux aquifers
         std::unordered_map<int, SingleAquiferFlux> aqufluxs;
+        BCProp bcprop;
         std::unordered_map<std::string, double> target_wellpi;
         std::optional<NextStep> next_tstep;
 
@@ -510,6 +518,7 @@ namespace Opm {
             serializer(m_whistctl_mode);
             serializer(target_wellpi);
             serializer(aqufluxs);
+            serializer(bcprop);
         }
 
 
