@@ -28,6 +28,10 @@
 #include <string>
 #include <vector>
 #include <optional>
+#include <limits>
+
+#include <opm/input/eclipse/Schedule/Well/FilterCake.hpp>
+#include <opm/input/eclipse/Schedule/Well/WINJMULT.hpp>
 
 namespace Opm {
 
@@ -76,7 +80,6 @@ namespace RestartIO {
             Defaulted,
         };
 
-
         Connection();
         Connection(int i, int j , int k ,
                    std::size_t global_index,
@@ -113,6 +116,7 @@ namespace RestartIO {
         int complnum() const;
         int segment() const;
         double CF() const;
+        double wpimult() const;
         double Kh() const;
         double rw() const;
         double r0() const;
@@ -120,9 +124,19 @@ namespace RestartIO {
         double connectionLength() const;
         double skinFactor() const;
         CTFKind kind() const;
+        const InjMult& injmult() const;
+        bool activeInjMult() const;
+        void setInjMult(const InjMult& inj_mult);
+        void setFilterCake(const FilterCake& filter_cake);
+        const FilterCake& getFilterCake() const;
+        bool filterCakeActive() const;
+        double getFilterCakeRadius() const;
+        double getFilterCakeArea() const;
 
         void setState(State state);
         void setComplnum(int compnum);
+        void setSkinFactor(double skin_factor);
+        void setCF(double CF);
         void scaleWellPi(double wellPi);
         bool prepareWellPIScaling();
         bool applyWellPIScaling(const double scaleFactor);
@@ -163,11 +177,13 @@ namespace RestartIO {
             serializer(ijk);
             serializer(m_global_index);
             serializer(m_ctfkind);
+            serializer(m_injmult);
             serializer(m_sort_value);
             serializer(m_perf_range);
             serializer(m_defaultSatTabId);
             serializer(segment_number);
             serializer(m_subject_to_welpi);
+            serializer(m_filter_cake);
         }
 
     private:
@@ -186,6 +202,7 @@ namespace RestartIO {
 
         std::array<int,3> ijk;
         CTFKind m_ctfkind;
+        std::optional<InjMult> m_injmult;
         std::size_t m_global_index;
         /*
           The sort_value member is a peculiar quantity. The connections are
@@ -250,6 +267,11 @@ namespace RestartIO {
         // Whether or not this Connection is subject to WELPI scaling.
         bool m_subject_to_welpi = false;
 
+        // For applying last known WPIMULT to when calculating connection transmissibilty factor in CSKIN
+        double m_wpimult = 1.0;
+
+        std::optional<FilterCake> m_filter_cake;
+
         static std::string CTFKindToString(const CTFKind);
     };
 }
