@@ -34,11 +34,12 @@
 #include <opm/input/eclipse/EclipseState/Phase.hpp>
 #include <opm/input/eclipse/Schedule/ScheduleTypes.hpp>
 #include <opm/input/eclipse/Schedule/VFPProdTable.hpp>
+#include <opm/input/eclipse/Schedule/Well/Connection.hpp>
 #include <opm/input/eclipse/Schedule/Well/PAvg.hpp>
-#include <opm/input/eclipse/Schedule/Well/PAvgCalculator.hpp>
 #include <opm/input/eclipse/Schedule/Well/WellEnums.hpp>
 #include <opm/input/eclipse/Schedule/Well/WellInjectionControls.hpp>
 #include <opm/input/eclipse/Schedule/Well/WellProductionControls.hpp>
+#include <opm/input/eclipse/Schedule/Well/WINJMULT.hpp>
 #include <opm/input/eclipse/Units/UnitSystem.hpp>
 
 namespace Opm {
@@ -66,6 +67,7 @@ struct WellPolymerProperties;
 class WellSegments;
 class WellTracerProperties;
 class WVFPEXP;
+class WVFPDP;
 
 namespace RestartIO {
 struct RstWell;
@@ -75,6 +77,11 @@ class Well {
 public:
     using Status = WellStatus;
 
+    /*
+     * The mode for the keyword WINJMULT.  It can have four different values: WREV, CREV, CIRR and NONE.
+     */
+    using InjMultMode = InjMult::InjMultMode;
+
     /*
       The elements in this enum are used as bitmasks to keep track
       of which controls are present, i.e. the 2^n structure must
@@ -161,7 +168,7 @@ public:
 
         void handleWELTARG(WELTARGCMode cmode, const UDAValue& new_arg, double SIFactorP);
         void handleWCONINJE(const DeckRecord& record, bool availableForGroupControl, const std::string& well_name);
-        void handleWCONINJH(const DeckRecord& record, bool is_producer, const std::string& well_name);
+        void handleWCONINJH(const DeckRecord& record, const bool is_producer, const std::string& well_name, const KeywordLocation& loc);
         bool hasInjectionControl(InjectorCMode controlModeArg) const {
             if (injectionControls & static_cast<int>(controlModeArg))
                 return true;
@@ -361,7 +368,6 @@ public:
     std::size_t firstTimeStep() const;
     const WellType& wellType() const;
     bool predictionMode() const;
-    bool canOpen() const;
     bool isProducer() const;
     bool isInjector() const;
     InjectorCMode injection_cmode() const;
@@ -383,11 +389,16 @@ public:
     Status getStatus() const;
     const std::string& groupName() const;
     Phase getPreferredPhase() const;
+    InjMultMode getInjMultMode() const;
+    const InjMult& getWellInjMult() const;
+    bool aciveWellInjMult() const;
 
     bool hasConnections() const;
     const std::vector<const Connection *> getConnections(int completion) const;
     const WellConnections& getConnections() const;
     const WellSegments& getSegments() const;
+    int maxSegmentID() const;
+    int maxBranchID() const;
 
     const WellProductionProperties& getProductionProperties() const;
     const WellInjectionProperties& getInjectionProperties() const;
@@ -397,6 +408,7 @@ public:
     const WellMICPProperties& getMICPProperties() const;
     const WellBrineProperties& getBrineProperties() const;
     const WellTracerProperties& getTracerProperties() const;
+    const WVFPDP& getWVFPDP() const;
     const WVFPEXP& getWVFPEXP() const;
     /* The rate of a given phase under the following assumptions:
      * * Returns zero if production is requested for an injector (and vice
@@ -461,13 +473,20 @@ public:
     bool updateWSEGAICD(const std::vector<std::pair<int, AutoICD> >& aicd_pairs, const KeywordLocation& location);
     bool updateWPAVE(const PAvg& pavg);
     void updateWPaveRefDepth(double ref_depth);
+    bool updateWVFPDP(std::shared_ptr<WVFPDP> wvfpdp);
     bool updateWVFPEXP(std::shared_ptr<WVFPEXP> wvfpexp);
 
     bool handleWELSEGS(const DeckKeyword& keyword);
     bool handleCOMPSEGS(const DeckKeyword& keyword, const ScheduleGrid& grid, const ParseContext& parseContext, ErrorGuard& errors);
     bool handleWELOPENConnections(const DeckRecord& record, Connection::State status);
+    bool handleCSKINConnections(const DeckRecord& record);
     bool handleCOMPLUMP(const DeckRecord& record);
     bool handleWPIMULT(const DeckRecord& record);
+    bool handleWINJCLN(const DeckRecord& record, const KeywordLocation& location);
+    bool handleWINJDAM(const DeckRecord& record, const KeywordLocation& location);
+    bool handleWINJMULT(const DeckRecord& record, const KeywordLocation& location);
+    void setFilterConc(const UDAValue& conc);
+    double evalFilterConc(const SummaryState& summary_sate) const;
     bool applyGlobalWPIMULT(double scale_factor);
 
     void filterConnections(const ActiveGridCells& grid);
@@ -493,7 +512,6 @@ public:
     void applyWellProdIndexScaling(const double       scalingFactor,
                                    std::vector<bool>& scalingApplicable);
     const PAvg& pavg() const;
-    PAvgCalculator pavg_calculator(const EclipseGrid& grid, const std::vector<double>& porv) const;
 
     template<class Serializer>
     void serializeOp(Serializer& serializer)
@@ -531,9 +549,13 @@ public:
         serializer(production);
         serializer(injection);
         serializer(segments);
+        serializer(wvfpdp);
         serializer(wvfpexp);
         serializer(m_pavg);
         serializer(well_temperature);
+        serializer(inj_mult_mode);
+        serializer(well_inj_mult);
+        serializer(m_filter_concentration);
     }
 
 private:
@@ -575,10 +597,14 @@ private:
     std::shared_ptr<WellProductionProperties> production;
     std::shared_ptr<WellInjectionProperties> injection;
     std::shared_ptr<WellSegments> segments;
+    std::shared_ptr<WVFPDP> wvfpdp;
     std::shared_ptr<WVFPEXP> wvfpexp;
     Status status;
     PAvg m_pavg;
     double well_temperature;
+    InjMultMode inj_mult_mode = InjMultMode::NONE;
+    std::optional<InjMult> well_inj_mult;
+    UDAValue m_filter_concentration;
 };
 
 std::ostream& operator<<( std::ostream&, const Well::WellInjectionProperties& );
