@@ -178,6 +178,8 @@ namespace Opm {
         constexpr const double Poise = prefix::deci*Pas;
         /// @}
 
+        constexpr const double ppm = 1. / prefix::mega;
+
         namespace perm_details {
             constexpr const double p_grad   = atm / (prefix::centi*meter);
             constexpr const double area     = square(prefix::centi*meter);
@@ -268,6 +270,7 @@ namespace Opm {
         constexpr const double LiquidSurfaceVolume  = cubic(meter);
         constexpr const double GasSurfaceVolume     = cubic(meter);
         constexpr const double ReservoirVolume      = cubic(meter);
+        constexpr const double Area                 = square(meter);
         constexpr const double GeomVolume           = cubic(meter);
         constexpr const double GasDissolutionFactor = GasSurfaceVolume/LiquidSurfaceVolume;
         constexpr const double OilDissolutionFactor = LiquidSurfaceVolume/GasSurfaceVolume;
@@ -281,6 +284,8 @@ namespace Opm {
         constexpr const double SurfaceTension       = dyne/(centi*meter);
         constexpr const double Energy               = kilo*joule;
         constexpr const double Moles                = kilo*mol;
+        constexpr const double PPM                  = ppm;
+        constexpr const double Ymodule              = giga*Pascal;
     }
 
 
@@ -300,6 +305,7 @@ namespace Opm {
         constexpr const double LiquidSurfaceVolume  = stb;
         constexpr const double GasSurfaceVolume     = 1000*cubic(feet);
         constexpr const double ReservoirVolume      = stb;
+        constexpr const double Area                 = square(feet);
         constexpr const double GeomVolume           = cubic(feet);
         constexpr const double GasDissolutionFactor = GasSurfaceVolume/LiquidSurfaceVolume;
         constexpr const double OilDissolutionFactor = LiquidSurfaceVolume/GasSurfaceVolume;
@@ -313,7 +319,8 @@ namespace Opm {
         constexpr const double SurfaceTension       = dyne/(centi*meter);
         constexpr const double Energy               = btu;
         constexpr const double Moles                = kilo*pound*mol;
-
+        constexpr const double PPM                  = ppm;
+        constexpr const double Ymodule              = giga*Pascal;
     }
 
 
@@ -333,6 +340,7 @@ namespace Opm {
         constexpr const double LiquidSurfaceVolume  = cubic(centi*meter);
         constexpr const double GasSurfaceVolume     = cubic(centi*meter);
         constexpr const double ReservoirVolume      = cubic(centi*meter);
+        constexpr const double Area                 = square(centi*meter);
         constexpr const double GeomVolume           = cubic(centi*meter);
         constexpr const double GasDissolutionFactor = GasSurfaceVolume/LiquidSurfaceVolume;
         constexpr const double OilDissolutionFactor = LiquidSurfaceVolume/GasSurfaceVolume;
@@ -346,7 +354,8 @@ namespace Opm {
         constexpr const double SurfaceTension       = dyne/(centi*meter);
         constexpr const double Energy               = joule;
         constexpr const double Moles                = mol;
-
+        constexpr const double PPM                  = ppm;
+        constexpr const double Ymodule              = giga*Pascal;
     }
 
 
@@ -366,6 +375,7 @@ namespace Opm {
         constexpr const double LiquidSurfaceVolume  = cubic(meter);
         constexpr const double GasSurfaceVolume     = cubic(meter);
         constexpr const double ReservoirVolume      = cubic(meter);
+        constexpr const double Area                 = square(meter);
         constexpr const double GeomVolume           = cubic(meter);
         constexpr const double GasDissolutionFactor = GasSurfaceVolume/LiquidSurfaceVolume;
         constexpr const double OilDissolutionFactor = LiquidSurfaceVolume/GasSurfaceVolume;
@@ -379,6 +389,8 @@ namespace Opm {
         constexpr const double SurfaceTension       = dyne/(centi*meter);
         constexpr const double Energy               = kilo*joule;
         constexpr const double Moles                = kilo*mol;
+        constexpr const double PPM                  = ppm;
+        constexpr const double Ymodule              = giga*Pascal;
     }
 }
 
