@@ -26,7 +26,7 @@
  */
 #ifndef OPM_ECL_HYSTERESIS_TWO_PHASE_LAW_HPP
 #define OPM_ECL_HYSTERESIS_TWO_PHASE_LAW_HPP
-
+#include <opm/common/TimingMacros.hpp>
 #include "EclHysteresisTwoPhaseLawParams.hpp"
 
 #include <stdexcept>
@@ -148,6 +148,7 @@ public:
     template <class Evaluation>
     static Evaluation twoPhaseSatPcnw(const Params& params, const Evaluation& Sw)
     {
+        OPM_TIMEFUNCTION_LOCAL();
         // if no pc hysteresis is enabled, use the drainage curve
         if (!params.config().enableHysteresis() || params.config().pcHysteresisModel() < 0)
             return EffectiveLaw::twoPhaseSatPcnw(params.drainageParams(), Sw);
@@ -267,6 +268,7 @@ public:
     template <class Evaluation>
     static Evaluation twoPhaseSatKrw(const Params& params, const Evaluation& Sw)
     {
+        OPM_TIMEFUNCTION_LOCAL();
         // if no relperm hysteresis is enabled, use the drainage curve
         if (!params.config().enableHysteresis() || params.config().krHysteresisModel() < 0)
             return EffectiveLaw::twoPhaseSatKrw(params.drainageParams(), Sw);
@@ -293,10 +295,49 @@ public:
     template <class Evaluation>
     static Evaluation twoPhaseSatKrn(const Params& params, const Evaluation& Sw)
     {
+        OPM_TIMEFUNCTION_LOCAL();
+        // If WAG hysteresis is enabled, the convential hysteresis model is ignored.
+        // (Two-phase model, non-wetting: only gas in oil.)
+        if (params.gasOilHysteresisWAG()) {
+
+            // Primary drainage
+            if (Sw <= params.krnSwMdc()+params.tolWAG() && params.nState()==1) {
+                Evaluation krn =  EffectiveLaw::twoPhaseSatKrn(params.drainageParams(), Sw);
+                return krn;
+            }
+
+            // Imbibition or reversion to two-phase drainage retracing imb curve
+            // (Shift along primary drainage curve.)
+            if (params.nState()==1) {
+                Evaluation Swf = params.computeSwf(Sw);
+                Evaluation krn = EffectiveLaw::twoPhaseSatKrn(params.drainageParams(), Swf);
+                return krn;
+            }
+
+            // Three-phase drainage along current secondary drainage curve
+            if (Sw <= params.krnSwDrainRevert()+params.tolWAG() /*&& params.nState()>=1 */) {
+                Evaluation Krg = EffectiveLaw::twoPhaseSatKrn(params.drainageParams(), Sw);
+                Evaluation KrgDrain2 = (Krg-params.krnDrainStart())*params.reductionDrain() + params.krnImbStart();
+                return KrgDrain2;
+            }
+
+            // Subsequent imbibition: Scanning curve derived from previous secondary drainage
+            if (Sw >= params.krnSwWAG()-params.tolWAG() /*&& Sw > params.krnSwDrainRevert() && params.nState()>=1 */) {
+                Evaluation KrgImb2 = params.computeKrImbWAG(Sw);
+                return KrgImb2;
+            }
+            else {/* Sw < params.krnSwWAG() */  // Reversion along "next" drainage curve
+                Evaluation Krg = EffectiveLaw::twoPhaseSatKrn(params.drainageParams(), Sw);
+                Evaluation KrgDrainNxt = (Krg-params.krnDrainStartNxt())*params.reductionDrainNxt() + params.krnImbStartNxt();
+                return KrgDrainNxt;
+            }
+        }
+
         // if no relperm hysteresis is enabled, use the drainage curve
         if (!params.config().enableHysteresis() || params.config().krHysteresisModel() < 0)
             return EffectiveLaw::twoPhaseSatKrn(params.drainageParams(), Sw);
 
+
         // if it is enabled, use either the drainage or the imbibition curve. if the
         // imbibition curve is used, the saturation must be shifted.
         if (Sw <= params.krnSwMdc())
