@@ -35,6 +35,7 @@
 
 #include <opm/material/fluidmatrixinteractions/SatCurveMultiplexer.hpp>
 #include <opm/material/fluidmatrixinteractions/EclEpsTwoPhaseLaw.hpp>
+#include <opm/input/eclipse/EclipseState/WagHysteresisConfig.hpp>
 #include <opm/material/fluidmatrixinteractions/EclHysteresisTwoPhaseLaw.hpp>
 #include <opm/material/fluidmatrixinteractions/EclMultiplexerMaterial.hpp>
 #include <opm/material/fluidmatrixinteractions/MaterialTraits.hpp>
@@ -170,7 +171,7 @@ private:
             std::shared_ptr<GasOilTwoPhaseHystParams> getGasOilParams();
             std::shared_ptr<OilWaterTwoPhaseHystParams> getOilWaterParams();
             std::shared_ptr<GasWaterTwoPhaseHystParams> getGasWaterParams();
-            void setConfig();
+            void setConfig(unsigned satRegionIdx);
             void setDrainageParamsOilGas(unsigned elemIdx, unsigned satRegionIdx);
             void setDrainageParamsOilWater(unsigned elemIdx, unsigned satRegionIdx);
             void setDrainageParamsGasWater(unsigned elemIdx, unsigned satRegionIdx);
@@ -251,13 +252,17 @@ public:
      * that the capillary pressure given depends on the particuars of how the simulator
      * calculates its initial condition.
      */
-    Scalar applySwatinit(unsigned elemIdx,
+    std::pair<Scalar, bool>
+    applySwatinit(unsigned elemIdx,
                          Scalar pcow,
                          Scalar Sw);
 
     bool enableEndPointScaling() const
     { return enableEndPointScaling_; }
 
+    bool enablePpcwmax() const
+    { return enablePpcwmax_; }
+
     bool enableHysteresis() const
     { return hysteresisConfig_->enableHysteresis(); }
 
@@ -314,20 +319,22 @@ public:
     { return imbnumRegionArray_[elemIdx]; }
 
     template <class FluidState>
-    void updateHysteresis(const FluidState& fluidState, unsigned elemIdx)
+    bool updateHysteresis(const FluidState& fluidState, unsigned elemIdx)
     {
+        OPM_TIMEFUNCTION_LOCAL();
         if (!enableHysteresis())
-            return;
-
-        MaterialLaw::updateHysteresis(materialLawParams(elemIdx), fluidState);
+            return false;
+        bool changed = MaterialLaw::updateHysteresis(materialLawParams(elemIdx), fluidState);
         if (hasDirectionalRelperms() || hasDirectionalImbnum()) {
             using Dir = FaceDir::DirEnum;
             constexpr int ndim = 3;
             Dir facedirs[ndim] = {Dir::XPlus, Dir::YPlus, Dir::ZPlus};
             for (int i = 0; i<ndim; i++) {
-                MaterialLaw::updateHysteresis(materialLawParams(elemIdx, facedirs[i]), fluidState);
+                bool ischanged =  MaterialLaw::updateHysteresis(materialLawParams(elemIdx, facedirs[i]), fluidState);
+                changed = changed || ischanged;
             }
         }
+        return changed;
     }
 
     void oilWaterHysteresisParams(Scalar& pcSwMdc,
@@ -374,6 +381,8 @@ private:
 
     bool enableEndPointScaling_;
     std::shared_ptr<EclHysteresisConfig> hysteresisConfig_;
+    std::vector<std::shared_ptr<WagHysteresisConfig::WagHysteresisConfigRecord>> wagHystersisConfig_;
+
 
     std::shared_ptr<EclEpsConfig> oilWaterEclEpsConfig_;
     std::vector<EclEpsScalingPointsInfo<Scalar>> unscaledEpsInfo_;
@@ -406,6 +415,10 @@ private:
     std::vector<int> imbnumRegionArray_;
     std::vector<Scalar> stoneEtas_;
 
+    bool enablePpcwmax_;
+    std::vector<Scalar> maxAllowPc_;
+    std::vector<bool> modifySwl_;
+
     bool hasGas;
     bool hasOil;
     bool hasWater;
