@@ -136,7 +136,7 @@ public:
                                    const Params& params,
                                    const FluidState& fluidState)
     {
-        OPM_TIMEBLOCK_LOCAL(capillaryPressures);
+        OPM_TIMEFUNCTION_LOCAL();
         switch (params.approach()) {
         case EclMultiplexerApproach::Stone1:
             Stone1Material::capillaryPressures(values,
@@ -178,6 +178,7 @@ public:
                                          Scalar& krnSwMdc,
                                          const Params& params)
     {
+        OPM_TIMEFUNCTION_LOCAL();
         switch (params.approach()) {
         case EclMultiplexerApproach::Stone1:
             Stone1Material::oilWaterHysteresisParams(pcSwMdc, krnSwMdc,
@@ -215,6 +216,7 @@ public:
                                             const Scalar& krnSwMdc,
                                             Params& params)
     {
+        OPM_TIMEFUNCTION_LOCAL();
         switch (params.approach()) {
         case EclMultiplexerApproach::Stone1:
             Stone1Material::setOilWaterHysteresisParams(pcSwMdc, krnSwMdc,
@@ -252,6 +254,7 @@ public:
                                        Scalar& krnSwMdc,
                                        const Params& params)
     {
+        OPM_TIMEFUNCTION_LOCAL();
         switch (params.approach()) {
         case EclMultiplexerApproach::Stone1:
             Stone1Material::gasOilHysteresisParams(pcSwMdc, krnSwMdc,
@@ -279,6 +282,28 @@ public:
         }
     }
 
+    static Scalar trappedGasSaturation(const Params& params)
+    {
+        OPM_TIMEFUNCTION_LOCAL();
+        switch (params.approach()) {
+        case EclMultiplexerApproach::Stone1:
+            return params.template getRealParams<EclMultiplexerApproach::Stone1>().gasOilParams().SnTrapped();
+        case EclMultiplexerApproach::Stone2:
+            return params.template getRealParams<EclMultiplexerApproach::Stone2>().gasOilParams().SnTrapped();
+        case EclMultiplexerApproach::Default:
+            return params.template getRealParams<EclMultiplexerApproach::Default>().gasOilParams().SnTrapped();
+        case EclMultiplexerApproach::TwoPhase:
+            if(params.template getRealParams<EclMultiplexerApproach::TwoPhase>().approach() == EclTwoPhaseApproach::GasOil)
+                return params.template getRealParams<EclMultiplexerApproach::TwoPhase>().gasOilParams().SnTrapped();
+            if(params.template getRealParams<EclMultiplexerApproach::TwoPhase>().approach() == EclTwoPhaseApproach::GasWater)
+                return params.template getRealParams<EclMultiplexerApproach::TwoPhase>().gasWaterParams().SnTrapped();
+            return 0.0; // oil-water case
+        case EclMultiplexerApproach::OnePhase:
+            return 0.0;
+        }
+        return 0.0;
+    }
+
     /*
      * Hysteresis parameters for gas-oil
      * @see EclHysteresisTwoPhaseLawParams::pcSwMdc(...)
@@ -289,6 +314,7 @@ public:
                                           const Scalar& krnSwMdc,
                                           Params& params)
     {
+        OPM_TIMEFUNCTION_LOCAL();
         switch (params.approach()) {
         case EclMultiplexerApproach::Stone1:
             Stone1Material::setGasOilHysteresisParams(pcSwMdc, krnSwMdc,
@@ -409,7 +435,7 @@ public:
                                        const Params& params,
                                        const FluidState& fluidState)
     {
-        OPM_TIMEBLOCK_LOCAL(relativePermeabilities);
+        OPM_TIMEFUNCTION_LOCAL();
         switch (params.approach()) {
         case EclMultiplexerApproach::Stone1:
             Stone1Material::relativePermeabilities(values,
@@ -452,6 +478,7 @@ public:
     static Evaluation relpermOilInOilGasSystem(const Params& params,
                                                const FluidState& fluidState)
     {
+        OPM_TIMEFUNCTION_LOCAL();
         switch (params.approach()) {
         case EclMultiplexerApproach::Stone1:
             return Stone1Material::template relpermOilInOilGasSystem<Evaluation>
@@ -482,6 +509,7 @@ public:
     static Evaluation relpermOilInOilWaterSystem(const Params& params,
                                                  const FluidState& fluidState)
     {
+        OPM_TIMEFUNCTION_LOCAL();
         switch (params.approach()) {
         case EclMultiplexerApproach::Stone1:
             return Stone1Material::template relpermOilInOilWaterSystem<Evaluation>
@@ -544,31 +572,34 @@ public:
      * error. (But not calling it will still work.)
      */
     template <class FluidState>
-    static void updateHysteresis(Params& params, const FluidState& fluidState)
+    static bool updateHysteresis(Params& params, const FluidState& fluidState)
     {
+        OPM_TIMEFUNCTION_LOCAL();
         switch (params.approach()) {
         case EclMultiplexerApproach::Stone1:
-            Stone1Material::updateHysteresis(params.template getRealParams<EclMultiplexerApproach::Stone1>(),
+            return Stone1Material::updateHysteresis(params.template getRealParams<EclMultiplexerApproach::Stone1>(),
                                              fluidState);
             break;
 
         case EclMultiplexerApproach::Stone2:
-            Stone2Material::updateHysteresis(params.template getRealParams<EclMultiplexerApproach::Stone2>(),
+            return Stone2Material::updateHysteresis(params.template getRealParams<EclMultiplexerApproach::Stone2>(),
                                              fluidState);
             break;
 
         case EclMultiplexerApproach::Default:
-            DefaultMaterial::updateHysteresis(params.template getRealParams<EclMultiplexerApproach::Default>(),
+            return DefaultMaterial::updateHysteresis(params.template getRealParams<EclMultiplexerApproach::Default>(),
                                               fluidState);
             break;
 
         case EclMultiplexerApproach::TwoPhase:
-            TwoPhaseMaterial::updateHysteresis(params.template getRealParams<EclMultiplexerApproach::TwoPhase>(),
+            return TwoPhaseMaterial::updateHysteresis(params.template getRealParams<EclMultiplexerApproach::TwoPhase>(),
                                                fluidState);
             break;
         case EclMultiplexerApproach::OnePhase:
+            return false;
             break;
         }
+        return false;
     }
 };
 
