@@ -32,6 +32,7 @@
 #include "blackoilpvt/GasPvtMultiplexer.hpp"
 #include "blackoilpvt/WaterPvtMultiplexer.hpp"
 #include "blackoilpvt/BrineCo2Pvt.hpp"
+#include "blackoilpvt/BrineH2Pvt.hpp"
 
 #include <opm/common/TimingMacros.hpp>
 
@@ -305,6 +306,19 @@ public:
     static void setWaterPvt(std::shared_ptr<WaterPvt> pvtObj)
     { waterPvt_ = pvtObj; }
 
+    static void setVapPars(const Scalar par1, const Scalar par2)
+    {
+        if (gasPvt_) {
+            gasPvt_->setVapPars(par1, par2);
+        }
+        if (oilPvt_) {
+            oilPvt_->setVapPars(par1, par2);
+        }
+        if (waterPvt_) {
+            waterPvt_->setVapPars(par1, par2);
+        }
+    }
+
     /*!
      * \brief Initialize the values of the reference densities
      *
