@@ -28,10 +28,12 @@
 #define OPM_BRINE_CO2_PVT_HPP
 
 #include <opm/common/Exceptions.hpp>
+#include <opm/common/TimingMacros.hpp>
+#include <opm/common/ErrorMacros.hpp>
 
 #include <opm/material/Constants.hpp>
 
-#include <opm/material/components/Brine.hpp>
+#include <opm/material/components/BrineDynamic.hpp>
 #include <opm/material/components/SimpleHuDuanH2O.hpp>
 #include <opm/material/components/CO2.hpp>
 #include <opm/material/common/UniformTabulated2DFunction.hpp>
@@ -67,7 +69,7 @@ class BrineCo2Pvt
 
 public:
     using H2O = SimpleHuDuanH2O<Scalar>;
-    using Brine = ::Opm::Brine<Scalar, H2O>;
+    using Brine = ::Opm::BrineDynamic<Scalar, H2O>;
     using CO2 = ::Opm::CO2<Scalar>;
 
     //! The binary coefficients for brine and CO2 used by this fluid system
@@ -80,13 +82,17 @@ public:
                 Scalar P_ref = 101325)
         : salinity_(salinity)
     {
+        // Throw an error if reference state is not (T, p) = (15.56 C, 1 atm) = (288.71 K, 1.01325e5 Pa)
+        if (T_ref != Scalar(288.71) || P_ref != Scalar(1.01325e5)) {    
+            OPM_THROW(std::runtime_error, 
+                "BrineCo2Pvt class can only be used with default reference state (T, P) = (288.71 K, 1.01325e5 Pa)!");
+        }
         int num_regions =  salinity_.size();
         co2ReferenceDensity_.resize(num_regions);
         brineReferenceDensity_.resize(num_regions);
-        Brine::salinity = salinity[0];
         for (int i = 0; i < num_regions; ++i) {
             co2ReferenceDensity_[i] = CO2::gasDensity(T_ref, P_ref, true);
-            brineReferenceDensity_[i] = Brine::liquidDensity(T_ref, P_ref, true);
+            brineReferenceDensity_[i] = Brine::liquidDensity(T_ref, P_ref, salinity_[i], true);
         }
     }
 
@@ -105,6 +111,9 @@ public:
         salinity_.resize(numRegions);
     }
 
+    void setVapPars(const Scalar, const Scalar)
+    {
+    }
 
     /*!
      * \brief Initialize the reference densities of all fluids for a given PVT region
@@ -136,6 +145,15 @@ public:
     void setEnableDissolvedGas(bool yesno)
     { enableDissolution_ = yesno; }
 
+    /*!
+     * \brief Specify whether the PVT model should consider salt concentration from
+     * the fluidstate or a fixed salinty
+     *
+     * By default, fixed salinity is considered
+     */
+    void setEnableSaltConcentration(bool yesno)
+    { enableSaltConcentration_ = yesno; }
+
     /*!
      * \brief Return the number of PVT regions which are considered by this PVT-object.
      */
@@ -150,9 +168,16 @@ public:
                               const Evaluation& temperature,
                               const Evaluation& pressure,
                               const Evaluation& Rs,
-                              const Evaluation& /*saltConcentration*/) const
+                              const Evaluation& saltConcentration) const
     {
-        return internalEnergy(regionIdx, temperature, pressure, Rs);
+        OPM_TIMEFUNCTION_LOCAL();
+        const Evaluation salinity = salinityFromConcentration(regionIdx, temperature, pressure, saltConcentration);
+        const Evaluation xlCO2 = convertRsToXoG_(Rs,regionIdx);
+        return (liquidEnthalpyBrineCO2_(temperature,
+                                       pressure,
+                                       salinity,
+                                       xlCO2)
+        - pressure / density_(regionIdx, temperature, pressure, Rs, salinity ));
     }
     /*!
      * \brief Returns the specific enthalpy [J/kg] of gas given a set of parameters.
@@ -163,13 +188,13 @@ public:
                         const Evaluation& pressure,
                         const Evaluation& Rs) const
     {
-
+        OPM_TIMEFUNCTION_LOCAL();
         const Evaluation xlCO2 = convertRsToXoG_(Rs,regionIdx);
         return (liquidEnthalpyBrineCO2_(temperature,
                                        pressure,
-                                       salinity_[regionIdx],
+                                       Evaluation(salinity_[regionIdx]),
                                        xlCO2)
-        - pressure / density_(regionIdx, temperature, pressure, Rs));
+        - pressure / density_(regionIdx, temperature, pressure, Rs, Evaluation(salinity_[regionIdx])));
     }
 
     /*!
@@ -192,9 +217,11 @@ public:
     Evaluation saturatedViscosity(unsigned regionIdx,
                                  const Evaluation& temperature,
                                  const Evaluation& pressure,
-                                 const Evaluation& /*saltConcentration*/) const
+                                 const Evaluation& saltConcentration) const
     {
-        return saturatedViscosity(regionIdx, temperature, pressure);
+        OPM_TIMEFUNCTION_LOCAL();
+        const Evaluation salinity = salinityFromConcentration(regionIdx, temperature, pressure, saltConcentration);
+        return Brine::liquidViscosity(temperature, pressure, salinity);
     }
 
     /*!
@@ -205,21 +232,23 @@ public:
                          const Evaluation& temperature,
                          const Evaluation& pressure,
                          const Evaluation& /*Rsw*/,
-                         const Evaluation& /*saltConcentration*/) const
+                         const Evaluation& saltConcentration) const
     {
+        OPM_TIMEFUNCTION_LOCAL();
         //TODO: The viscosity does not yet depend on the composition
-        return saturatedViscosity(regionIdx, temperature, pressure);
+        return saturatedViscosity(regionIdx, temperature, pressure, saltConcentration);
     }
 
     /*!
      * \brief Returns the dynamic viscosity [Pa s] of oil saturated gas at given pressure.
      */
     template <class Evaluation>
-    Evaluation saturatedViscosity(unsigned /*regionIdx*/,
+    Evaluation saturatedViscosity(unsigned regionIdx,
                                   const Evaluation& temperature,
                                   const Evaluation& pressure) const
     {
-        return Brine::liquidViscosity(temperature, pressure);
+        OPM_TIMEFUNCTION_LOCAL();
+        return Brine::liquidViscosity(temperature, pressure, Evaluation(salinity_[regionIdx]));
     }
 
 
@@ -230,9 +259,12 @@ public:
     Evaluation saturatedInverseFormationVolumeFactor(unsigned regionIdx,
                                                      const Evaluation& temperature,
                                                      const Evaluation& pressure,
-                                                     const Evaluation& /*saltconcentration*/) const
+                                                     const Evaluation& saltconcentration) const
     {
-        return saturatedInverseFormationVolumeFactor(regionIdx, temperature, pressure);
+        OPM_TIMEFUNCTION_LOCAL();
+        const Evaluation salinity = salinityFromConcentration(regionIdx, temperature, pressure, saltconcentration);
+        Evaluation rsSat = rsSat_(regionIdx, temperature, pressure, salinity);
+        return (1.0 - convertRsToXoG_(rsSat,regionIdx)) * density_(regionIdx, temperature, pressure, rsSat, salinity)/brineReferenceDensity_[regionIdx];
     }
     /*!
      * \brief Returns the formation volume factor [-] of the fluid phase.
@@ -242,9 +274,11 @@ public:
                                             const Evaluation& temperature,
                                             const Evaluation& pressure,
                                             const Evaluation& Rs,
-                                            const Evaluation& /*saltConcentration*/) const
+                                            const Evaluation& saltConcentration) const
     {
-        return inverseFormationVolumeFactor(regionIdx, temperature, pressure, Rs);
+        OPM_TIMEFUNCTION_LOCAL();
+        const Evaluation salinity = salinityFromConcentration(regionIdx, temperature, pressure, saltConcentration);
+        return (1.0 - convertRsToXoG_(Rs,regionIdx)) * density_(regionIdx, temperature, pressure, Rs, salinity)/brineReferenceDensity_[regionIdx];
     }
     /*!
      * \brief Returns the formation volume factor [-] of the fluid phase.
@@ -255,7 +289,7 @@ public:
                                             const Evaluation& pressure,
                                             const Evaluation& Rs) const
     {
-        return (1.0 - convertRsToXoG_(Rs,regionIdx)) * density_(regionIdx, temperature, pressure, Rs)/brineReferenceDensity_[regionIdx];
+        return (1.0 - convertRsToXoG_(Rs,regionIdx)) * density_(regionIdx, temperature, pressure, Rs, Evaluation(salinity_[regionIdx]))/brineReferenceDensity_[regionIdx];
     }
 
     /*!
@@ -266,8 +300,9 @@ public:
                                                      const Evaluation& temperature,
                                                      const Evaluation& pressure) const
     {
-        Evaluation rsSat = rsSat_(regionIdx, temperature, pressure);
-        return (1.0 - convertRsToXoG_(rsSat,regionIdx)) * density_(regionIdx, temperature, pressure, rsSat)/brineReferenceDensity_[regionIdx];
+        OPM_TIMEFUNCTION_LOCAL();
+        Evaluation rsSat = rsSat_(regionIdx, temperature, pressure, Evaluation(salinity_[regionIdx]));
+        return (1.0 - convertRsToXoG_(rsSat,regionIdx)) * density_(regionIdx, temperature, pressure, rsSat, Evaluation(salinity_[regionIdx]))/brineReferenceDensity_[regionIdx];
     }
 
     /*!
@@ -310,7 +345,7 @@ public:
                                              const Evaluation& /*maxOilSaturation*/) const
     {
         //TODO support VAPPARS
-        return rsSat_(regionIdx, temperature, pressure);
+        return rsSat_(regionIdx, temperature, pressure, Evaluation(salinity_[regionIdx]));
     }
 
     /*!
@@ -320,9 +355,10 @@ public:
     Evaluation saturatedGasDissolutionFactor(unsigned regionIdx,
                                              const Evaluation& temperature,
                                              const Evaluation& pressure,
-                                             const Evaluation& /*saltConcentration*/) const
+                                             const Evaluation& saltConcentration) const
     {
-        return rsSat_(regionIdx, temperature, pressure);
+        const Evaluation salinity = salinityFromConcentration(regionIdx, temperature, pressure, saltConcentration);
+        return rsSat_(regionIdx, temperature, pressure, salinity);
     }
 
     /*!
@@ -333,7 +369,7 @@ public:
                                              const Evaluation& temperature,
                                              const Evaluation& pressure) const
     {
-        return rsSat_(regionIdx, temperature, pressure);
+        return rsSat_(regionIdx, temperature, pressure, Evaluation(salinity_[regionIdx]));
     }
 
     const Scalar oilReferenceDensity(unsigned regionIdx) const
@@ -348,17 +384,19 @@ public:
     const Scalar salinity(unsigned regionIdx) const
     { return salinity_[regionIdx]; }
 
+
     template <class Evaluation>
     Evaluation diffusionCoefficient(const Evaluation& temperature,
                                     const Evaluation& pressure,
                                     unsigned /*compIdx*/) const
     {
+        OPM_TIMEFUNCTION_LOCAL();
         //Diffusion coefficient of CO2 in pure water according to (McLachlan and Danckwerts, 1972)
         const Evaluation log_D_H20 = -4.1764 + 712.52 / temperature - 2.5907e5 / (temperature*temperature);
 
         //Diffusion coefficient of CO2 in the brine phase modified following (Ratcliff and Holdcroft,1963 and Al-Rawajfeh, 2004)
         const Evaluation& mu_H20 = H2O::liquidViscosity(temperature, pressure, extrapolate); // Water viscosity
-        const Evaluation& mu_Brine = Brine::liquidViscosity(temperature, pressure); // Brine viscosity
+        const Evaluation& mu_Brine = Brine::liquidViscosity(temperature, pressure, Evaluation(salinity_[0])); // Brine viscosity
         const Evaluation log_D_Brine = log_D_H20 - 0.87*log10(mu_Brine / mu_H20);
 
         return pow(Evaluation(10), log_D_Brine) * 1e-4; // convert from cm2/s to m2/s
@@ -369,17 +407,21 @@ private:
     std::vector<Scalar> co2ReferenceDensity_;
     std::vector<Scalar> salinity_;
     bool enableDissolution_ = true;
+    bool enableSaltConcentration_ = false;
 
     template <class LhsEval>
     LhsEval density_(unsigned regionIdx,
                      const LhsEval& temperature,
                      const LhsEval& pressure,
-                     const LhsEval& Rs) const
+                     const LhsEval& Rs,
+                     const LhsEval& salinity) const
     {
-        LhsEval xlCO2 = convertXoGToxoG_(convertRsToXoG_(Rs,regionIdx));
+        OPM_TIMEFUNCTION_LOCAL();
+        LhsEval xlCO2 = convertXoGToxoG_(convertRsToXoG_(Rs,regionIdx), salinity);
         LhsEval result = liquidDensity_(temperature,
                                         pressure,
-                                        xlCO2);
+                                        xlCO2,
+                                        salinity);
 
         Valgrind::CheckDefined(result);
         return result;
@@ -389,8 +431,10 @@ private:
     template <class LhsEval>
     LhsEval liquidDensity_(const LhsEval& T,
                            const LhsEval& pl,
-                           const LhsEval& xlCO2) const
+                           const LhsEval& xlCO2,
+                           const LhsEval& salinity) const
     {
+        OPM_TIMEFUNCTION_LOCAL();
         Valgrind::CheckDefined(T);
         Valgrind::CheckDefined(pl);
         Valgrind::CheckDefined(xlCO2);
@@ -410,7 +454,7 @@ private:
             throw NumericalProblem(msg);
         }
 
-        const LhsEval& rho_brine = Brine::liquidDensity(T, pl, extrapolate);
+        const LhsEval& rho_brine = Brine::liquidDensity(T, pl, salinity, extrapolate);
         const LhsEval& rho_pure = H2O::liquidDensity(T, pl, extrapolate);
         const LhsEval& rho_lCO2 = liquidDensityWaterCO2_(T, pl, xlCO2);
         const LhsEval& contribCO2 = rho_lCO2 - rho_pure;
@@ -423,6 +467,7 @@ private:
                                           const LhsEval& pl,
                                           const LhsEval& xlCO2) const
     {
+        OPM_TIMEFUNCTION_LOCAL();
         Scalar M_CO2 = CO2::molarMass();
         Scalar M_H2O = H2O::molarMass();
 
@@ -448,6 +493,7 @@ private:
     template <class LhsEval>
     LhsEval convertRsToXoG_(const LhsEval& Rs, unsigned regionIdx) const
     {
+        OPM_TIMEFUNCTION_LOCAL();
         Scalar rho_oRef = brineReferenceDensity_[regionIdx];
         Scalar rho_gRef = co2ReferenceDensity_[regionIdx];
 
@@ -460,10 +506,11 @@ private:
      * \brief Convert a gas mass fraction in the oil phase the corresponding mole fraction.
      */
     template <class LhsEval>
-    LhsEval convertXoGToxoG_(const LhsEval& XoG) const
+    LhsEval convertXoGToxoG_(const LhsEval& XoG, const LhsEval& salinity) const
     {
+        OPM_TIMEFUNCTION_LOCAL();
         Scalar M_CO2 = CO2::molarMass();
-        Scalar M_Brine = Brine::molarMass();
+        LhsEval M_Brine = Brine::molarMass(salinity);
         return XoG*M_Brine / (M_CO2*(1 - XoG) + XoG*M_Brine);
     }
 
@@ -472,10 +519,11 @@ private:
      * \brief Convert a gas mole fraction in the oil phase the corresponding mass fraction.
      */
     template <class LhsEval>
-    LhsEval convertxoGToXoG(const LhsEval& xoG) const
+    LhsEval convertxoGToXoG(const LhsEval& xoG, const LhsEval& salinity) const
     {
+        OPM_TIMEBLOCK_LOCAL(convertxoGToXoG);
         Scalar M_CO2 = CO2::molarMass();
-        Scalar M_Brine = Brine::molarMass();
+        LhsEval M_Brine = Brine::molarMass(salinity);
 
         return xoG*M_CO2 / (xoG*(M_CO2 - M_Brine) + M_Brine);
     }
@@ -498,18 +546,20 @@ private:
     template <class LhsEval>
     LhsEval rsSat_(unsigned regionIdx,
                    const LhsEval& temperature,
-                   const LhsEval& pressure) const
+                   const LhsEval& pressure,
+                   const LhsEval& salinity) const
     {
+        OPM_TIMEFUNCTION_LOCAL();
         if (!enableDissolution_)
             return 0.0;
 
         // calulate the equilibrium composition for the given
-        // temperature and pressure. 
+        // temperature and pressure.
         LhsEval xgH2O;
         LhsEval xlCO2;
         BinaryCoeffBrineCO2::calculateMoleFractions(temperature,
                                                     pressure,
-                                                    salinity_[regionIdx],
+                                                    salinity,
                                                     /*knownPhaseIdx=*/-1,
                                                     xlCO2,
                                                     xgH2O,
@@ -518,15 +568,16 @@ private:
         // normalize the phase compositions
         xlCO2 = max(0.0, min(1.0, xlCO2));
 
-        return convertXoGToRs(convertxoGToXoG(xlCO2), regionIdx);
+        return convertXoGToRs(convertxoGToXoG(xlCO2, salinity), regionIdx);
     }
 
     template <class LhsEval>
     static LhsEval liquidEnthalpyBrineCO2_(const LhsEval& T,
                                            const LhsEval& p,
-                                           Scalar S, // salinity
+                                           const LhsEval& salinity,
                                            const LhsEval& X_CO2_w)
     {
+        OPM_TIMEFUNCTION_LOCAL();
         /* X_CO2_w : mass fraction of CO2 in brine */
 
         /* same function as enthalpy_brine, only extended by CO2 content */
@@ -554,6 +605,8 @@ private:
         // Regularization
         Scalar scalarTheta = scalarValue(theta);
         Scalar S_lSAT = f[0] + scalarTheta*(f[1] + scalarTheta*(f[2] + scalarTheta*f[3]));
+
+        LhsEval S = salinity;
         if (S > S_lSAT)
             S = S_lSAT;
 
@@ -563,14 +616,14 @@ private:
         /*U=*/h_NaCl = (3.6710E4*T + 0.5*(6.2770E1)*T*T - ((6.6670E-2)/3)*T*T*T
                         +((2.8000E-5)/4)*(T*T*T*T))/(58.44E3)- 2.045698e+02; /* kJ/kg */
 
-        Scalar m = 1E3/58.44 * S/(1-S);
+        LhsEval m = 1E3/58.44 * S/(1-S);
         int i = 0;
         int j = 0;
         d_h = 0;
 
         for (i = 0; i<=3; i++) {
             for (j=0; j<=2; j++) {
-                d_h = d_h + a[i][j] * pow(theta, static_cast<Scalar>(i)) * std::pow(m, j);
+                d_h = d_h + a[i][j] * pow(theta, static_cast<Scalar>(i)) * pow(m, j);
             }
         }
         /* heat of dissolution for halite according to Michaelides 1971 */
@@ -590,6 +643,15 @@ private:
         /* Enthalpy of brine with dissolved CO2 */
         return (h_ls1 - X_CO2_w*hw + hg*X_CO2_w)*1E3; /*J/kg*/
     }
+
+    template <class LhsEval>
+    const LhsEval salinityFromConcentration(unsigned regionIdx, const LhsEval&T, const LhsEval& P, const LhsEval& saltConcentration) const
+    {
+        if (enableSaltConcentration_)
+            return saltConcentration/H2O::liquidDensity(T, P, true);
+
+        return salinity(regionIdx);
+    }
 };
 
 } // namespace Opm
