@@ -124,6 +124,11 @@ public:
         rhoRefG_.resize(numRegions);
     }
 
+    void setVapPars(const Scalar par1, const Scalar par2)
+    {
+        isothermalPvt_->setVapPars(par1, par2);
+    }
+
     /*!
      * \brief Finish initializing the thermal part of the oil phase PVT properties.
      */
