@@ -132,6 +132,7 @@ namespace Opm {
         , m_transMult(         GridDims(deck), deck, field_props)
         , tracer_config(       m_deckUnitSystem, deck)
         , m_micppara(          deck)
+        , wag_hyst_config(     deck)
     {
         this->assignRunTitle(deck);
         this->reportNumberOfActivePhases();
@@ -139,6 +140,7 @@ namespace Opm {
         this->conveyNumericalAquiferEffects();
         this->m_inputGrid.resetACTNUM(this->field_props.actnum());
         this->field_props.reset_actnum(this->getInputGrid().getACTNUM());
+        this->initLgrs(deck);
         this->aquifer_config.load_connections(deck, this->getInputGrid());
 
         this->applyMULTXYZ();
@@ -153,7 +155,7 @@ namespace Opm {
     catch (const OpmInputError& opm_error) {
         OpmLog::error(opm_error.what());
         throw;
-    } 
+    }
     catch (const std::exception& std_error) {
         OpmLog::error(fmt::format("\nAn error occurred while creating the reservoir properties\n"
                                   "Internal error: {}\n", std_error.what()));
@@ -234,10 +236,18 @@ namespace Opm {
         return m_faults;
     }
 
+    const LgrCollection& EclipseState::getLgrs() const {
+        return m_lgrs;
+    }
+
     const MICPpara& EclipseState::getMICPpara() const {
         return m_micppara;
     }
 
+    const WagHysteresisConfig& EclipseState::getWagHysteresis() const {
+        return wag_hyst_config;
+    }
+
     const TransMult& EclipseState::getTransMult() const {
         return m_transMult;
     }
@@ -264,6 +274,17 @@ namespace Opm {
         return !m_inputNnc.input().empty();
     }
 
+    void EclipseState::initLgrs(const Deck& deck) {
+        if (!DeckSection::hasGRID(deck))
+            return;
+
+        const GRIDSection gridSection ( deck );
+
+        m_lgrs = LgrCollection(gridSection, m_inputGrid);
+
+    }
+
+
     std::string EclipseState::getTitle() const {
         return m_title;
     }
@@ -331,6 +352,8 @@ namespace Opm {
         // Add NNCs between aquifer cells and first aquifer cell and aquifer
         // connections.
         this->appendInputNNC(numerical_aquifer.aquiferCellNNCs());
+
+        this->m_transMult.applyNumericalAquifer(numerical_aquifer.allAquiferCellIds());
     }
 
     void EclipseState::applyMULTXYZ() {
@@ -357,16 +380,17 @@ namespace Opm {
         setMULTFLT(gridSection);
 
         if (DeckSection::hasEDIT(deck)) {
-            setMULTFLT(EDITSection ( deck ));
+            setMULTFLT(EDITSection(deck), true);
         }
 
         m_transMult.applyMULTFLT( m_faults );
     }
 
 
-    void EclipseState::setMULTFLT(const DeckSection& section) {
+    void EclipseState::setMULTFLT(const DeckSection& section, bool edit) {
         // Set error to false
         bool error = false;
+        std::map<std::string,double> prev;
         for (size_t index=0; index < section.count("MULTFLT"); index++) {
             const auto& faultsKeyword = section.getKeyword("MULTFLT" , index);
             OpmLog::info(OpmInputError::format("\nApplying {keyword} in {file} line {line}", faultsKeyword.location()));
@@ -377,7 +401,18 @@ namespace Opm {
                 double multFlt = faultRecord.getItem(1).get< double >(0);
                 try
                 {
-                    m_faults.setTransMult( faultName , multFlt );
+                    if (edit) {
+                        if (m_faults.hasFault(faultName)) {
+                            const auto it = prev.find(faultName);
+                            const auto& fault = m_faults.getFault(faultName);
+                            if (it == prev.end()) {
+                                prev[faultName] = fault.getTransMult();
+                                multFlt *= m_faults.getFault(faultName).getTransMult();
+                            } else
+                                multFlt *= it->second;
+                        }
+                    }
+                    m_faults.setTransMult(faultName, multFlt);
                     logger(fmt::format("Setting fault transmissibility multiplier {} for fault {}", multFlt, faultName));
                 }
                 catch(const std::exception& std_error)
@@ -386,15 +421,16 @@ namespace Opm {
                        "MULTFLT(FLTNAME) equals {} and MULT(FLT-TRS) equals {}\n"
                        "Error creating reservoir properties: {}" , faultName, multFlt, std_error.what()));
                     error = true;
+
                 }
             }
         }
         // Throw if errors
         if (error) {
             throw std::invalid_argument("Error Processing MULTFLT");
-       }
-    } 
-    
+        }
+    }
+
 
     void EclipseState::complainAboutAmbiguousKeyword(const Deck& deck, const std::string& keywordName) {
         OpmLog::error("The " + keywordName + " keyword must be unique in the deck. Ignoring all!");
