@@ -155,7 +155,7 @@ EclipseGrid::EclipseGrid(const GridDims& gd)
 
 
 EclipseGrid::EclipseGrid(size_t nx, size_t ny , size_t nz,
-                         double dx, double dy, double dz)
+                         double dx, double dy, double dz, double top)
     : GridDims(nx, ny, nz),
       m_minpvMode(MinpvMode::Inactive),
       m_pinchoutMode(PinchMode::TOPBOT),
@@ -186,8 +186,8 @@ EclipseGrid::EclipseGrid(size_t nx, size_t ny , size_t nz,
                 // top face of cell
                 int zind = i*2 + j*nx*4 + k*nx*ny*8;
 
-                double zt = k*dz;
-                double zb = (k+1)*dz;
+                double zt = top + k*dz;
+                double zb = top + (k+1)*dz;
 
                 m_zcorn[zind] = zt;
                 m_zcorn[zind + 1] = zt;
@@ -431,7 +431,7 @@ EclipseGrid::EclipseGrid(const Deck& deck, const int * actnum)
 
         if (gridunit[0] != "METRES") {
 
-            const auto length = ::Opm::UnitSystem::measure::length;
+            constexpr auto length = ::Opm::UnitSystem::measure::length;
 
             if (gridunit[0] == "FEET"){
                 Opm::UnitSystem units(Opm::UnitSystem::UnitType::UNIT_TYPE_FIELD );
@@ -564,7 +564,7 @@ EclipseGrid::EclipseGrid(const Deck& deck, const int * actnum)
     void EclipseGrid::initBinaryGrid(const Deck& deck) {
 
         const DeckKeyword& gdfile_kw = deck["GDFILE"].back();
-        const std::string& gdfile_arg = gdfile_kw.getRecord(0).getItem("filename").get<std::string>(0);
+        const std::string& gdfile_arg = gdfile_kw.getRecord(0).getItem("filename").getTrimmedString(0);
         std::string filename = deck.makeDeckPath(gdfile_arg);
 
         std::unique_ptr<Opm::EclIO::EclFile> egridfile;
@@ -1678,6 +1678,12 @@ std::vector<double> EclipseGrid::createDVector(const std::array<int,3>& dims, st
 
     double EclipseGrid::getCellDepth(size_t globalIndex) const {
         assertGlobalIndex( globalIndex );
+
+        auto it = this->m_aquifer_cell_depths.find(globalIndex);
+        return it != this->m_aquifer_cell_depths.end() ? it->second : computeCellGeometricDepth(globalIndex);
+    }
+
+    double EclipseGrid::computeCellGeometricDepth(size_t globalIndex) const {
         std::array<double,8> X;
         std::array<double,8> Y;
         std::array<double,8> Z;
@@ -1719,7 +1725,7 @@ std::vector<double> EclipseGrid::createDVector(const std::array<int,3>& dims, st
     void EclipseGrid::save(const std::string& filename, bool formatted, const std::vector<Opm::NNCdata>& nnc, const Opm::UnitSystem& units) const {
 
         Opm::UnitSystem::UnitType unitSystemType = units.getType();
-        const auto length = ::Opm::UnitSystem::measure::length;
+        constexpr auto length = ::Opm::UnitSystem::measure::length;
 
         const std::array<int, 3> dims = getNXYZ();
 
@@ -1891,6 +1897,9 @@ std::vector<double> EclipseGrid::createDVector(const std::array<int,3>& dims, st
                 const size_t k = record.getItem<AQUNUM::K>().get<int>(0) - 1;
                 const size_t global_index = this->getGlobalIndex(i, j, k);
                 this->m_aquifer_cells.insert(global_index);
+
+                if (! record.getItem<AQUNUM::DEPTH>().defaultApplied(0))
+                    this->m_aquifer_cell_depths.insert_or_assign(global_index, record.getItem<AQUNUM::DEPTH>().getSIDouble(0));
             }
         }
     }
