@@ -18,17 +18,25 @@
 
 #include <opm/input/eclipse/EclipseState/Grid/FieldProps.hpp>
 
-#include <functional>
-#include <algorithm>
-#include <unordered_map>
-#include <array>
-#include <vector>
-#include <set>
-#include <unordered_set>
+#include <opm/common/OpmLog/LogUtil.hpp>
+#include <opm/common/OpmLog/OpmLog.hpp>
 
-#include <fmt/format.h>
 #include <opm/common/utility/OpmInputError.hpp>
 
+#include <opm/input/eclipse/EclipseState/Aquifer/NumericalAquifer/NumericalAquifers.hpp>
+#include <opm/input/eclipse/EclipseState/Grid/Box.hpp>
+#include <opm/input/eclipse/EclipseState/Grid/EclipseGrid.hpp>
+#include <opm/input/eclipse/EclipseState/Grid/FieldPropsManager.hpp> // Layering violation.  Needed for apply_tran() function.
+#include <opm/input/eclipse/EclipseState/Grid/SatfuncPropertyInitializers.hpp>
+#include <opm/input/eclipse/EclipseState/Runspec.hpp>
+#include <opm/input/eclipse/EclipseState/Tables/RtempvdTable.hpp>
+#include <opm/input/eclipse/EclipseState/Tables/TableManager.hpp>
+#include <opm/input/eclipse/EclipseState/Util/OrderedMap.hpp>
+
+#include <opm/input/eclipse/Units/UnitSystem.hpp>
+
+#include <opm/input/eclipse/Deck/Deck.hpp>
+
 #include <opm/input/eclipse/Parser/ParserKeywords/A.hpp>
 #include <opm/input/eclipse/Parser/ParserKeywords/B.hpp>
 #include <opm/input/eclipse/Parser/ParserKeywords/C.hpp>
@@ -38,19 +46,23 @@
 #include <opm/input/eclipse/Parser/ParserKeywords/P.hpp>
 #include <opm/input/eclipse/Parser/ParserKeywords/T.hpp>
 
-#include <opm/input/eclipse/Units/UnitSystem.hpp>
-#include <opm/input/eclipse/Deck/Deck.hpp>
-#include <opm/input/eclipse/EclipseState/Tables/RtempvdTable.hpp>
-#include <opm/input/eclipse/EclipseState/Grid/EclipseGrid.hpp>
-#include <opm/input/eclipse/EclipseState/Grid/Box.hpp>
-#include <opm/input/eclipse/EclipseState/Grid/FieldPropsManager.hpp>
-#include <opm/input/eclipse/EclipseState/Grid/SatfuncPropertyInitializers.hpp>
-#include <opm/input/eclipse/EclipseState/Runspec.hpp>
-#include <opm/input/eclipse/EclipseState/Aquifer/NumericalAquifer/NumericalAquifers.hpp>
-#include <opm/input/eclipse/EclipseState/Tables/TableManager.hpp>
-
 #include "Operate.hpp"
 
+#include <algorithm>
+#include <array>
+#include <cstddef>
+#include <functional>
+#include <optional>
+#include <set>
+#include <stdexcept>
+#include <string>
+#include <tuple>
+#include <unordered_map>
+#include <unordered_set>
+#include <utility>
+#include <vector>
+
+#include <fmt/format.h>
 
 namespace {
     Opm::Box makeGlobalGridBox(const Opm::EclipseGrid* gridPtr)
@@ -67,6 +79,12 @@ namespace {
             }
         };
     }
+
+    bool is_capillary_pressure(const std::string& keyword)
+    {
+        return (keyword == "PCW")  || (keyword == "PCG")
+            || (keyword == "IPCG") || (keyword == "IPCW");
+    }
 }
 
 namespace Opm {
@@ -81,6 +99,12 @@ static const std::set<std::string> region_oper_keywords = {"MULTIREG", "ADDREG",
 static const std::set<std::string> box_keywords = {"BOX", "ENDBOX"};
 
 
+bool is_oper_keyword(const std::string& name)
+{
+    return (oper_keywords.find(name) != oper_keywords.end()
+            || region_oper_keywords.find(name) != region_oper_keywords.end());
+}
+
 std::string get_keyword_from_alias(const std::string& name) {
     if (ALIAS::aliased_keywords.count(name))
         return ALIAS::aliased_keywords.at(name);
@@ -133,6 +157,9 @@ keyword_info<int> global_kw_info(const std::string& name, bool) {
     if (SCHEDULE::int_keywords.count(name))
         return SCHEDULE::int_keywords.at(name);
 
+    if (isFipxxx(name))
+        return keyword_info<int>{}.init(1);
+
     throw std::out_of_range("No such keyword: " + name);
 }
 
@@ -416,9 +443,10 @@ bool FieldProps::operator==(const FieldProps& other) const {
            this->m_default_region == other.m_default_region &&
            this->m_rtep == other.m_rtep &&
            this->tables == other.tables &&
+           this->multregp == other.multregp &&
            this->int_data == other.int_data &&
            this->double_data == other.double_data &&
-           this->multregp == other.multregp &&
+           this->fipreg_shortname_translation == other.fipreg_shortname_translation &&
            this->tran == other.tran;
 }
 
@@ -657,8 +685,11 @@ Fieldprops::FieldData<double>& FieldProps::init_get(const std::string& keyword_n
     if (keyword == ParserKeywords::TEMPI::keywordName)
         this->init_tempi(this->double_data[keyword]);
 
-    if (Fieldprops::keywords::PROPS::satfunc.count(keyword) == 1)
+    if ((Fieldprops::keywords::PROPS::satfunc.count(keyword) == 1) ||
+        is_capillary_pressure(keyword))
+    {
         this->init_satfunc(keyword, this->double_data[keyword]);
+    }
 
     return this->double_data[keyword];
 }
@@ -686,7 +717,7 @@ Fieldprops::FieldData<int>& FieldProps::init_get(const std::string& keyword, boo
     if (Fieldprops::keywords::isFipxxx(keyword)) {
         auto kw_info = Fieldprops::keywords::keyword_info<int>{};
         kw_info.init(1);
-        return this->init_get(keyword, kw_info);
+        return this->init_get(this->canonical_fipreg_name(keyword), kw_info);
     } else {
         const Fieldprops::keywords::keyword_info<int>& kw_info = Fieldprops::keywords::global_kw_info<int>(keyword);
         return this->init_get(keyword, kw_info);
@@ -726,7 +757,9 @@ bool FieldProps::has<double>(const std::string& keyword_name) const {
 
 template <>
 bool FieldProps::has<int>(const std::string& keyword) const {
-    return (this->int_data.count(keyword) != 0);
+    return Fieldprops::keywords::isFipxxx(keyword)
+        ? this->int_data.count(this->canonical_fipreg_name(keyword)) != 0
+        : this->int_data.count(keyword) != 0;
 }
 
 
@@ -921,6 +954,8 @@ void FieldProps::handle_region_operation(const DeckKeyword& keyword) {
     for (const auto& record : keyword) {
         const std::string& target_kw = Fieldprops::keywords::get_keyword_from_alias(record.getItem(0).get<std::string>(0));
         int region_value = record.getItem("REGION_NUMBER").get<int>(0);
+        const auto warn_empty_region = R"({} region {} has no active cells when processing operation {} on keyword {}.
+Please check whether this is on purpose or you missed defining the region properly.)";
 
         if (this->tran.find(target_kw) != this->tran.end())
             throw std::logic_error("The region operations can not be used for manipulations of TRANX, TRANY or TRANZ");
@@ -935,6 +970,11 @@ void FieldProps::handle_region_operation(const DeckKeyword& keyword) {
                 const auto& src_data = this->init_get<double>(src_kw);
                 auto& field_data = this->init_get<double>(target_kw);
                 FieldProps::operate(record, field_data, src_data, index_list);
+                if (index_list.empty()) {
+                    OpmLog::warning(Log::fileMessage(keyword.location(),
+                                                     fmt::format(warn_empty_region, region_name, region_value,
+                                                                 keyword.name(), src_kw)));
+                }
             } else {
                 auto operation = fromString(keyword.name());
                 const double scalar_value = this->getSIValue(operation, target_kw, record.getItem(1).get<double>(0));
@@ -961,6 +1001,11 @@ void FieldProps::handle_region_operation(const DeckKeyword& keyword) {
                 }
 
                 FieldProps::apply(fromString(keyword.name()), field_data.data, field_data.value_status, scalar_value, index_list);
+                if (index_list.empty()) {
+                    OpmLog::warning(Log::fileMessage(keyword.location(),
+                                                     fmt::format(warn_empty_region, region_name, region_value, keyword.name(),
+                                                                 target_kw)));
+                }
             }
 
             continue;
@@ -1154,6 +1199,41 @@ void FieldProps::init_porv(Fieldprops::FieldData<double>& porv) {
     }
 }
 
+std::string FieldProps::canonical_fipreg_name(const std::string& fipreg)
+{
+    constexpr auto num_unique_chars = std::string::size_type{6};
+    const auto shortname = fipreg.substr(0, num_unique_chars);
+
+    auto canonicalPos = std::unordered_map<std::string, std::string>::const_iterator{};
+    auto inserted = false;
+
+    std::tie(canonicalPos, inserted) = this->fipreg_shortname_translation
+        .emplace(shortname, fipreg);
+
+    if (inserted || (fipreg.size() <= num_unique_chars)) {
+        // New FIP keyword (inserted == true) or we're looking up the
+        // canonical name of an existing FIP array based on a unique prefix
+        // string, such as "FIPUNIT" from "FIPUNI" (size <= ...).
+        return canonicalPos->second;
+    }
+
+    // New FIP keyword with the same unique prefix as an existing FIP
+    // keyword.  Override the translation table entry for this prefix,
+    // because "last entry wins".
+    canonicalPos = this->fipreg_shortname_translation
+        .insert_or_assign(canonicalPos, shortname, fipreg);
+
+    return canonicalPos->second;
+}
+
+const std::string& FieldProps::canonical_fipreg_name(const std::string& fipreg) const
+{
+    auto canonicalPos = this->fipreg_shortname_translation.find(fipreg.substr(0, 6));
+
+    return (canonicalPos != this->fipreg_shortname_translation.end())
+        ? canonicalPos->second
+        : fipreg;
+}
 
 /*
   This function generates a new ACTNUM vector.The ACTNUM vector which is
@@ -1382,7 +1462,7 @@ const std::string& FieldProps::default_region() const {
 }
 
 void FieldProps::apply_tran(const std::string& keyword, std::vector<double>& data) {
-    Opm::apply_tran(this->tran, this->double_data, this->active_size, keyword, data);
+    ::Opm::apply_tran(this->tran, this->double_data, this->active_size, keyword, data);
 }
 
 bool FieldProps::tran_active(const std::string& keyword) const {
