@@ -239,7 +239,7 @@ namespace Opm {
         return m_output_enabled;
     }
 
-    void IOConfig::setOutputEnabled(bool enabled){
+    void IOConfig::setOutputEnabled(bool enabled) {
         m_output_enabled = enabled;
     }
 
@@ -247,6 +247,15 @@ namespace Opm {
         return m_output_dir;
     }
 
+    std::string IOConfig::getInputDir() const {
+        std::filesystem::path path(m_deck_filename);
+        if (path.has_parent_path()) {
+            return path.parent_path();
+        } else {
+            return std::filesystem::current_path();
+        }
+    }
+
     void IOConfig::setOutputDir(const std::string& outputDir) {
         m_output_dir = outputDir;
     }
