@@ -26,6 +26,7 @@
 #include <opm/input/eclipse/Parser/ParserKeywords/C.hpp>
 #include <opm/input/eclipse/Parser/ParserKeywords/F.hpp>
 #include <opm/input/eclipse/Parser/ParserKeywords/G.hpp>
+#include <opm/input/eclipse/Parser/ParserKeywords/H.hpp>
 #include <opm/input/eclipse/Parser/ParserKeywords/M.hpp>
 #include <opm/input/eclipse/Parser/ParserKeywords/N.hpp>
 #include <opm/input/eclipse/Parser/ParserKeywords/O.hpp>
@@ -41,7 +42,6 @@
 #include <ostream>
 #include <stdexcept>
 #include <type_traits>
-
 #include <fmt/format.h>
 
 namespace {
@@ -96,12 +96,13 @@ namespace {
         // note: we allow for SOF2 to be part of family1 for threeP +
         // solvent simulations.
 
-        const auto family2 =      // SGFN, SOF{2,3}, SWFN
-            (gas && deck.hasKeyword<Opm::ParserKeywords::SGFN>()) ||
+        const auto family2 =      // SGFN, SOF{2,3}, SWFN, SGWFN
+            (gas && (deck.hasKeyword<Opm::ParserKeywords::SGFN>() ||
+                     deck.hasKeyword<Opm::ParserKeywords::SGWFN>())) ||
             (oil && ((threeP && deck.hasKeyword<Opm::ParserKeywords::SOF3>()) ||
                      (twoP && deck.hasKeyword<Opm::ParserKeywords::SOF2>()))) ||
-            (wat && deck.hasKeyword<Opm::ParserKeywords::SWFN>());
-
+            (wat && (deck.hasKeyword<Opm::ParserKeywords::SWFN>() ||
+                     deck.hasKeyword<Opm::ParserKeywords::SGWFN>()));
         const auto family3 = //WSF, GSF gas-water CO2STORE case
             deck.hasKeyword<Opm::ParserKeywords::GSF>() &&
             deck.hasKeyword<Opm::ParserKeywords::WSF>();
@@ -194,6 +195,7 @@ Welldims::Welldims(const Deck& deck)
         // Maximum number of dynamic well lists
         this->nDynWlistMax = wd.getItem<WD::MAX_DYNAMIC_WELLIST>().get<int>(0);
 
+
         this->m_location = keyword.location();
     }
 }
@@ -211,38 +213,61 @@ Welldims Welldims::serializationTestObject()
     return result;
 }
 
-WellSegmentDims::WellSegmentDims() :
-    nSegWellMax( ParserKeywords::WSEGDIMS::NSWLMX::defaultValue ),
-    nSegmentMax( ParserKeywords::WSEGDIMS::NSEGMX::defaultValue ),
-    nLatBranchMax( ParserKeywords::WSEGDIMS::NLBRMX::defaultValue )
+WellSegmentDims::WellSegmentDims()
+    : nSegWellMax   { ParserKeywords::WSEGDIMS::NSWLMX::defaultValue }
+    , nSegmentMax   { ParserKeywords::WSEGDIMS::NSEGMX::defaultValue }
+    , nLatBranchMax { ParserKeywords::WSEGDIMS::NLBRMX::defaultValue }
 {}
 
-WellSegmentDims::WellSegmentDims(const Deck& deck) : WellSegmentDims()
+WellSegmentDims::WellSegmentDims(const Deck& deck)
+    : WellSegmentDims {}
 {
-    if (deck.hasKeyword("WSEGDIMS")) {
-        const auto& wsd = deck["WSEGDIMS"][0].getRecord(0);
+    using WSD = ParserKeywords::WSEGDIMS;
 
-        this->nSegWellMax   = wsd.getItem("NSWLMX").get<int>(0);
-        this->nSegmentMax   = wsd.getItem("NSEGMX").get<int>(0);
-        this->nLatBranchMax = wsd.getItem("NLBRMX").get<int>(0);
+    if (deck.hasKeyword<WSD>()) {
+        const auto& keyword = deck.get<WSD>().front();
+        const auto& wsd = keyword.getRecord(0);
+
+        if (const auto& maxMSW = wsd.getItem<WSD::NSWLMX>();
+            ! maxMSW.defaultApplied(0))
+        {
+            this->nSegWellMax = maxMSW.get<int>(0);
+        }
+
+        if (const auto& maxSeg = wsd.getItem<WSD::NSEGMX>();
+            ! maxSeg.defaultApplied(0))
+        {
+            this->nSegmentMax = maxSeg.get<int>(0);
+        }
+
+        if (const auto& maxBranch = wsd.getItem<WSD::NLBRMX>();
+            ! maxBranch.defaultApplied(0))
+        {
+            this->nLatBranchMax = maxBranch.get<int>(0);
+        }
+
+        this->location_ = keyword.location();
     }
 }
 
 WellSegmentDims WellSegmentDims::serializationTestObject()
 {
     WellSegmentDims result;
+
     result.nSegWellMax = 1;
     result.nSegmentMax = 2;
     result.nLatBranchMax = 3;
+    result.location_ = KeywordLocation::serializationTestObject();
 
     return result;
 }
 
 bool WellSegmentDims::operator==(const WellSegmentDims& data) const
 {
-    return this->maxSegmentedWells() == data.maxSegmentedWells() &&
-           this->maxSegmentsPerWell() == data.maxSegmentsPerWell() &&
-           this->maxLateralBranchesPerWell() == data.maxLateralBranchesPerWell();
+    return (this->maxSegmentedWells() == data.maxSegmentedWells())
+        && (this->maxSegmentsPerWell() == data.maxSegmentsPerWell())
+        && (this->maxLateralBranchesPerWell() == data.maxLateralBranchesPerWell())
+        && (this->location() == data.location());
 }
 
 NetworkDims::NetworkDims() :
@@ -344,18 +369,34 @@ EclHysterConfig::EclHysterConfig(const Opm::Deck& deck)
         if (!activeHyst)
 	      return;
 
-        if (!deck.hasKeyword("EHYSTR"))
-            throw std::runtime_error("Enabling hysteresis via the HYST parameter for SATOPTS requires the "
-                                     "presence of the EHYSTR keyword");
+        if (deck.hasKeyword("WAGHYSTR")) {
+            if ( !(deck.hasKeyword<Opm::ParserKeywords::OIL>() &&
+                   deck.hasKeyword<Opm::ParserKeywords::GAS>() &&
+                   deck.hasKeyword<Opm::ParserKeywords::WATER>()) )
+                throw std::runtime_error("WAG hysteresis (kw 'WAGHYSTR') requires 'OIL', 'WATER' and 'GAS' present in model. ");
+
+            activeWagHyst = true;
+        }
+
+        if (!deck.hasKeyword("EHYSTR")) {
+            std::string msg = "Hysteresis is enabled via the HYST parameter for SATOPTS, but the EHYSTR " 
+                              "keyword is not present in the deck. \n"
+                              "Default values are used for the EHYSTR keyword.";
+
+            OpmLog::warning(msg);
+            krHystMod = 0;
+            pcHystMod = 0;
+            return;
+        }
 	    /*!
-	* \brief Set the type of the hysteresis model which is used for relative permeability.
-	*
-	* -1: relperm hysteresis is disabled
-	* 0: use the Carlson model for relative permeability hysteresis of the non-wetting
-	*    phase and the drainage curve for the relperm of the wetting phase
-	* 1: use the Carlson model for relative permeability hysteresis of the non-wetting
-	*    phase and the imbibition curve for the relperm of the wetting phase
-	*/
+       * \brief Set the type of the hysteresis model which is used for relative permeability.
+       *
+       * -1: relperm hysteresis is disabled
+       * 0: use the Carlson model for relative permeability hysteresis of the non-wetting
+       *    phase and the drainage curve for the relperm of the wetting phase
+       * 1: use the Carlson model for relative permeability hysteresis of the non-wetting
+       *    phase and the imbibition curve for the relperm of the wetting phase
+       */
         const auto& ehystrKeyword = deck["EHYSTR"].back();
         std::string whereFlag = ehystrKeyword.getRecord(0).getItem("limiting_hyst_flag").getTrimmedString(0);
 
@@ -374,11 +415,11 @@ EclHysterConfig::EclHysterConfig(const Opm::Deck& deck)
         // models with HYPC/NOHYPC and the fifth item of EHYSTR. Let's ignore that for
         // now.
             /*!
-	* \brief Return the type of the hysteresis model which is used for capillary pressure.
-	*
-	* -1: capillary pressure hysteresis is disabled
-	* 0: use the Killough model for capillary pressure hysteresis
-	*/
+       * \brief Return the type of the hysteresis model which is used for capillary pressure.
+       *
+       * -1: capillary pressure hysteresis is disabled
+       * 0: use the Killough model for capillary pressure hysteresis
+       */
         if (deck.hasKeyword("NOHYPC") || whereFlag == "KR")
             pcHystMod = -1;
         else {
@@ -405,6 +446,7 @@ EclHysterConfig EclHysterConfig::serializationTestObject()
     result.krHystMod = 2;
     result.modParamTrappedValue = 3;
     result.curvatureCapPrsValue = 4;
+    result.activeWagHyst = true;
 
     return result;
 }
@@ -424,12 +466,16 @@ double EclHysterConfig::modParamTrapped() const
 double EclHysterConfig::curvatureCapPrs() const
     { return curvatureCapPrsValue; }
 
+bool EclHysterConfig::activeWag() const
+    { return activeWagHyst; }
+
 bool EclHysterConfig::operator==(const EclHysterConfig& data) const {
     return this->active() == data.active() &&
            this->pcHysteresisModel() == data.pcHysteresisModel() &&
            this->krHysteresisModel() == data.krHysteresisModel() &&
            this->modParamTrapped() == data.modParamTrapped() &&
-           this->curvatureCapPrs() == data.curvatureCapPrs();
+           this->curvatureCapPrs() == data.curvatureCapPrs() &&
+           this->activeWag() == data.activeWag();
 }
 
 SatFuncControls::SatFuncControls()
@@ -578,7 +624,9 @@ Runspec::Runspec( const Deck& deck )
     , m_nupcol( )
     , m_tracers( deck )
     , m_co2storage (false)
+    , m_h2storage (false)
     , m_micp (false)
+    , m_mech (false)
 {
     if (DeckSection::hasRUNSPEC(deck)) {
         const RUNSPECSection runspecSection{deck};
@@ -612,6 +660,13 @@ Runspec::Runspec( const Deck& deck )
 
         }
 
+        if (runspecSection.hasKeyword<ParserKeywords::H2STORE>()) {
+            m_h2storage = true;
+            std::string msg = "The H2 storage option is given. PVT properties from the Brine-H2 system is used \n"
+                              "See the OPM manual for details on the used models.";
+            OpmLog::note(msg);
+        }
+
         if (runspecSection.hasKeyword<ParserKeywords::MICP>()) {
             m_micp = true;
             std::string msg = "The MICP option is given. Single phase (WATER) + 3 transported components + \n"
@@ -619,6 +674,12 @@ Runspec::Runspec( const Deck& deck )
                               "for details on the used model.";
             OpmLog::note(msg);
         }
+
+        if (runspecSection.hasKeyword<ParserKeywords::MECH>()) {
+            m_mech = true;
+            const std::string msg = "Simulation will solve for mechanical quantities";
+            OpmLog::note(msg);
+        }
     }
 }
 
@@ -638,7 +699,9 @@ Runspec Runspec::serializationTestObject()
     result.m_sfuncctrl = SatFuncControls::serializationTestObject();
     result.m_nupcol = Nupcol::serializationTestObject();
     result.m_co2storage = true;
+    result.m_h2storage = true;
     result.m_micp = true;
+    result.m_mech = true;
 
     return result;
 }
@@ -703,11 +766,21 @@ bool Runspec::co2Storage() const noexcept
     return this->m_co2storage;
 }
 
+bool Runspec::h2Storage() const noexcept
+{
+    return this->m_h2storage;
+}
+
 bool Runspec::micp() const noexcept
 {
     return this->m_micp;
 }
 
+bool Runspec::mech() const noexcept
+{
+    return this->m_mech;
+}
+
 std::time_t Runspec::start_time() const noexcept
 {
     return this->m_start_time;
@@ -745,7 +818,9 @@ bool Runspec::rst_cmp(const Runspec& full_spec, const Runspec& rst_spec) {
         full_spec.saturationFunctionControls() == rst_spec.saturationFunctionControls() &&
         full_spec.m_nupcol == rst_spec.m_nupcol &&
         full_spec.m_co2storage == rst_spec.m_co2storage &&
+        full_spec.m_h2storage == rst_spec.m_h2storage &&
         full_spec.m_micp == rst_spec.m_micp &&
+        full_spec.m_mech == rst_spec.m_mech &&
         Welldims::rst_cmp(full_spec.wellDimensions(), rst_spec.wellDimensions());
 }
 
@@ -762,7 +837,9 @@ bool Runspec::operator==(const Runspec& data) const {
            this->saturationFunctionControls() == data.saturationFunctionControls() &&
            this->m_nupcol == data.m_nupcol &&
            this->m_co2storage == data.m_co2storage &&
-           this->m_micp == data.m_micp;
+           this->m_h2storage == data.m_h2storage &&
+           this->m_micp == data.m_micp &&
+           this->m_mech == data.m_mech;
 }
 
 
