@@ -50,6 +50,7 @@
 #include <algorithm>
 #include <array>
 #include <cassert>
+#include <iterator>
 #include <map>
 #include <regex>
 #include <set>
@@ -384,6 +385,15 @@ struct SummaryConfigContext {
         return std::regex_match(keyword, well_compl_kw);
     }
 
+    bool is_well_comp(const std::string& keyword)
+    {
+        static const auto well_comp_kw = keyword_set {
+             "WXMF", "WYMF", "WZMF", "WCGMR", "WCOMR"
+        };
+
+        return is_in_set(well_comp_kw, keyword);
+    }
+
     bool is_node_keyword(const std::string& keyword)
     {
         static const auto nodekw = keyword_set {
@@ -842,8 +852,33 @@ inline void keywordB( SummaryConfig::keyword_list& list,
     .parameterType( parseKeywordType(keyword.name()) )
     .isUserDefined( is_udq(keyword.name()) );
 
+    auto isValid = [&dims](const std::array<int,3>& ijk)
+    {
+        return (static_cast<std::size_t>(ijk[0]) < dims.getNX())
+            && (static_cast<std::size_t>(ijk[1]) < dims.getNY())
+            && (static_cast<std::size_t>(ijk[2]) < dims.getNZ());
+    };
+
   for( const auto& record : keyword ) {
       auto ijk = getijk( record );
+
+      if (! isValid(ijk)) {
+          const auto msg_fmt =
+              fmt::format("Block level summary keyword "
+                          "{{keyword}}\n"
+                          "In {{file}} line {{line}}\n"
+                          "References invalid cell "
+                          "{},{},{} in grid of dimensions "
+                          "{},{},{}.\nThis block summary "
+                          "vector request is ignored.",
+                          ijk[0] + 1  , ijk[1] + 1  , ijk[2] + 1,
+                          dims.getNX(), dims.getNY(), dims.getNZ());
+
+          OpmLog::warning(OpmInputError::format(msg_fmt, keyword.location()));
+
+          continue;
+      }
+
       int global_index = 1 + dims.getGlobalIndex(ijk[0], ijk[1], ijk[2]);
       list.push_back( param.number(global_index) );
   }
@@ -1032,7 +1067,7 @@ inline void keywordR(SummaryConfig::keyword_list& list,
     auto param = SummaryConfigNode {
         keyword, SummaryConfigNode::Category::Region, deck_keyword.location()
     }
-    .parameterType(parseKeywordType(keyword))
+    .parameterType(parseKeywordType(EclIO::SummaryNode::normalise_region_keyword(keyword)))
     .fip_region( region_name.value() )
     .isUserDefined( is_udq(keyword) );
 
@@ -1106,16 +1141,22 @@ inline void keywordMISC( SummaryConfig::keyword_list& list,
     {
         const auto& kw = keyword.name();
 
-        if (kw.size() > std::string::size_type{5}) {
-            // Easy check first--handles SUMMARY and SUMTHIN &c.
+        if ((kw == "SUMMARY") || (kw == "SUMTHIN")) {
             return false;
         }
 
+        if (kw[1] == 'U') {
+            // User-defined quantity at segment level.  Unbounded set, so
+            // assume this is well defined.
+            return true;
+        }
+
         const auto kw_whitelist = std::vector<const char*> {
-            "SOFR" , "SOFRF", "SOFRS",
-            "SGFR" , "SGFRF", "SGFRS",
-            "SWFR" ,
+            "SOFR" , "SOFRF", "SOFRS", "SOFT", "SOFV", "SOHF", "SOVIS",
+            "SGFR" , "SGFRF", "SGFRS", "SGFT", "SGFV", "SGHF", "SGVIS",
+            "SWFR" ,                   "SWFT", "SWFV", "SWHF", "SWVIS",
             "SGOR" , "SOGR" , "SWCT" , "SWGR" ,
+            "SODEN", "SGDEN", "SWDEN", "SMDEN", "SDENM",
             "SPR"  , "SPRD" , "SPRDH", "SPRDF", "SPRDA",
         };
 
@@ -1337,6 +1378,12 @@ inline void handleKW( SummaryConfig::keyword_list& list,
     const auto cat = parseKeywordCategory(name);
     switch (cat) {
     case Cat::Well:
+        if (is_well_comp(keyword.name())) {
+            Opm::OpmLog::warning(Opm::OpmInputError::format("Unhandled summary keyword {keyword}\n"
+                                                        "In {file} line {line}", keyword.location()));
+        return;
+        }
+
         keywordW(list, parseContext, errors, keyword, schedule);
         break;
 
@@ -1841,12 +1888,16 @@ bool SummaryConfig::match(const std::string& keywordPattern) const {
     return false;
 }
 
-SummaryConfig::keyword_list SummaryConfig::keywords(const std::string& keywordPattern) const {
-    keyword_list kw_list;
-    for (const auto& keyword : this->m_keywords) {
-        if (shmatch(keywordPattern, keyword.keyword()))
-            kw_list.push_back(keyword);
-    }
+SummaryConfig::keyword_list
+SummaryConfig::keywords(const std::string& keywordPattern) const
+{
+    auto kw_list = keyword_list{};
+
+    std::copy_if(this->m_keywords.begin(), this->m_keywords.end(),
+                 std::back_inserter(kw_list),
+                 [&keywordPattern](const auto& kw)
+                 { return shmatch(keywordPattern, kw.keyword()); });
+
     return kw_list;
 }
 
