@@ -135,6 +135,7 @@ std::optional<JFunc> make_jfunc(const Deck& deck) {
         m_tabdims( Tabdims(deck)),
         m_aqudims( Aqudims(deck)),
         m_tlmixpar( deck ),
+        m_ppcwmax( deck ),
         hasImptvd (deck.hasKeyword("IMPTVD")),
         hasEnptvd (deck.hasKeyword("ENPTVD")),
         hasEqlnum (deck.hasKeyword("EQLNUM")),
@@ -322,6 +323,7 @@ std::optional<JFunc> make_jfunc(const Deck& deck) {
         result.m_rtemp = 1.0;
         result.m_salinity = 1.0;
         result.m_tlmixpar = TLMixpar::serializationTestObject();
+        result.m_ppcwmax = Ppcwmax::serializationTestObject();
         return result;
     }
 
@@ -581,6 +583,7 @@ std::optional<JFunc> make_jfunc(const Deck& deck) {
         initSimpleTableContainer<SpecheatTable>(deck, "SPECHEAT", m_tabdims.getNumPVTTables());
         initSimpleTableContainer<SpecrockTable>(deck, "SPECROCK", m_tabdims.getNumSatTables());
         initSimpleTableContainer<OilvisctTable>(deck, "OILVISCT", m_tabdims.getNumPVTTables());
+        initSimpleTableContainer<GasvisctTable>(deck, "GASVISCT", m_tabdims.getNumPVTTables());
         initSimpleTableContainer<WatvisctTable>(deck, "WATVISCT", m_tabdims.getNumPVTTables());
 
         initSimpleTableContainer<PlyadsTable>(deck, "PLYADS", m_tabdims.getNumSatTables());
@@ -592,7 +595,6 @@ std::optional<JFunc> make_jfunc(const Deck& deck) {
 
         initPlyrockTables(deck);
         initPlymaxTables(deck);
-        initGasvisctTables(deck);
         initRTempTables(deck);
         initRocktabTables(deck);
         initPlyshlogTables(deck);
@@ -618,34 +620,7 @@ std::optional<JFunc> make_jfunc(const Deck& deck) {
         } else if (hasRTEMPVD) {
             initSimpleTableContainer<RtempvdTable>(deck, "RTEMPVD", "RTEMPVD", m_eqldims.getNumEquilRegions());
         }
-    }
-
-
-    void TableManager::initGasvisctTables(const Deck& deck) {
-
-        const std::string keywordName = "GASVISCT";
-        size_t numTables = m_tabdims.getNumPVTTables();
-
-        if (!deck.hasKeyword(keywordName))
-            return; // the table is not featured by the deck...
-
-        auto& container = forceGetTables(keywordName , numTables);
-
-        if (deck.count(keywordName) > 1) {
-            complainAboutAmbiguousKeyword(deck, keywordName);
-            return;
-        }
-
-        const auto& tableKeyword = deck[keywordName].back();
-        for (size_t tableIdx = 0; tableIdx < tableKeyword.size(); ++tableIdx) {
-            const auto& tableRecord = tableKeyword.getRecord( tableIdx );
-            const auto& dataItem = tableRecord.getItem( 0 );
-            if (dataItem.data_size() > 0) {
-                std::shared_ptr<GasvisctTable> table = std::make_shared<GasvisctTable>( deck , dataItem );
-                container.addTable( tableIdx , table );
-            }
-        }
-    }
+    }   
 
 
     void TableManager::initPlyshlogTables(const Deck& deck) {
@@ -1201,6 +1176,10 @@ std::optional<JFunc> make_jfunc(const Deck& deck) {
         return m_tlmixpar;
     }
 
+    const Ppcwmax& TableManager::getPpcwmax() const {
+        return m_ppcwmax;
+    }
+
     const JFunc& TableManager::getJFunc() const {
         if (!this->jfunc.has_value())
             throw std::invalid_argument("Cannot get JFUNC table when JFUNC not in deck");
@@ -1302,6 +1281,7 @@ std::optional<JFunc> make_jfunc(const Deck& deck) {
                m_skprwatTables == data.m_skprwatTables &&
                m_skprpolyTables == data.m_skprpolyTables &&
                m_tlmixpar == data.m_tlmixpar &&
+               m_ppcwmax == data.m_ppcwmax &&
                m_tabdims == data.m_tabdims &&
                m_regdims == data.m_regdims &&
                m_eqldims == data.m_eqldims &&
@@ -1564,6 +1544,8 @@ std::optional<JFunc> make_jfunc(const Deck& deck) {
                     container.addTable( tableIdx , table );
                 } catch (const std::runtime_error& err) {
                     throw OpmInputError(err, tableKeyword.location());
+                } catch (const std::invalid_argument& err) {
+                    throw OpmInputError(err, tableKeyword.location());
                 }
             }
         }
