@@ -15,51 +15,143 @@
 
   You should have received a copy of the GNU General Public License
   along with OPM.  If not, see <http://www.gnu.org/licenses/>.
- */
+*/
 
-#include <cctype>
-#include <filesystem>
-#include <iterator>
-#include <iostream>
-#include <optional>
-#include <stack>
-#include <string>
-#include <utility>
-#include <vector>
-#include <algorithm>
-
-#include <fmt/format.h>
+#include <opm/input/eclipse/Parser/Parser.hpp>
 
 #include <opm/common/OpmLog/OpmLog.hpp>
 #include <opm/common/OpmLog/LogUtil.hpp>
 #include <opm/common/utility/OpmInputError.hpp>
-#include <opm/input/eclipse/Deck/ImportContainer.hpp>
 
-#include <opm/json/JsonObject.hpp>
+#include <opm/input/eclipse/Parser/ErrorGuard.hpp>
+#include <opm/input/eclipse/Parser/ParseContext.hpp>
+#include <opm/input/eclipse/Parser/ParserItem.hpp>
+#include <opm/input/eclipse/Parser/ParserKeyword.hpp>
+#include <opm/input/eclipse/Parser/ParserKeywords/I.hpp>
+#include <opm/input/eclipse/Parser/ParserKeywords/P.hpp>
+#include <opm/input/eclipse/Parser/ParserKeywords/R.hpp>
+#include <opm/input/eclipse/Parser/ParserKeywords/S.hpp>
+#include <opm/input/eclipse/Parser/ParserKeywords/T.hpp>
+#include <opm/input/eclipse/Parser/ParserRecord.hpp>
 
-#include <opm/input/eclipse/Python/Python.hpp>
 #include <opm/input/eclipse/Deck/Deck.hpp>
 #include <opm/input/eclipse/Deck/DeckItem.hpp>
 #include <opm/input/eclipse/Deck/DeckKeyword.hpp>
 #include <opm/input/eclipse/Deck/DeckRecord.hpp>
 #include <opm/input/eclipse/Deck/DeckSection.hpp>
+#include <opm/input/eclipse/Deck/ImportContainer.hpp>
+
 #include <opm/input/eclipse/EclipseState/EclipseState.hpp>
 #include <opm/input/eclipse/EclipseState/Grid/EclipseGrid.hpp>
-#include <opm/input/eclipse/Parser/ErrorGuard.hpp>
-#include <opm/input/eclipse/Parser/ParseContext.hpp>
-#include <opm/input/eclipse/Parser/ParserKeywords/I.hpp>
-#include <opm/input/eclipse/Parser/Parser.hpp>
-#include <opm/input/eclipse/Parser/ParserItem.hpp>
-#include <opm/input/eclipse/Parser/ParserKeyword.hpp>
-#include <opm/input/eclipse/Parser/ParserRecord.hpp>
+#include <opm/input/eclipse/EclipseState/Grid/FieldProps.hpp>
+
+#include <opm/input/eclipse/Python/Python.hpp>
+
+#include <opm/json/JsonObject.hpp>
+
 #include <opm/common/utility/String.hpp>
 
 #include "raw/RawConsts.hpp"
 #include "raw/RawEnums.hpp"
-#include "raw/RawRecord.hpp"
 #include "raw/RawKeyword.hpp"
+#include "raw/RawRecord.hpp"
 #include "raw/StarToken.hpp"
 
+#include <algorithm>
+#include <cctype>
+#include <cstdio>
+#include <filesystem>
+#include <iostream>
+#include <iterator>
+#include <optional>
+#include <stack>
+#include <stdexcept>
+#include <string>
+#include <regex>
+#include <utility>
+#include <vector>
+
+#include <fmt/format.h>
+
+namespace {
+    /// \brief Whether a keyword is a global keyword.
+    ///
+    /// Those are allowed before RUNSPEC.
+    /// \return whether keyword is global (ECHO, NOECHO, INCLUDE,
+    ///         COLUMNS, FORMFEED, SKIP, SKIP100, SKIP300, END)
+    bool isGlobalKeyword(const Opm::DeckKeyword& keyword ) {
+        const auto& name = keyword.name();
+        for( const auto& x : { "ECHO", "NOECHO",
+                              "INCLUDE", "COLUMNS", "FORMFEED",
+                              "SKIP", "ENDSKIP",
+                              "SKIP100", "SKIP300" } )
+            if( name == x ) return true;
+
+        return false;
+    }
+
+    // If ROCKOPTS does NOT exist, then the number of records is NTPVT (= TABDIMS(2))
+    //
+    // Otherwise, the number of records depends on ROCKOPTS(3), (= TABLE_TYPE)
+    //
+    //  1) ROCKOPTS(3) == "SATNUM"  => NTSFUN (= TABDIMS( 1))
+    //  2) ROCKOPTS(3) == "ROCKNUM" => NTROCC (= TABDIMS(13))
+    //  3) ROCKOPTS(3) == "PVTNUM"  => NTPVT  (= TABDIMS( 2)) (default setting)
+    //
+    // Finally, if ROCKOPTS(3) == "ROCKNUM", but NTROCC is defaulted, then
+    // the number of records is NTPVT.
+    std::size_t targetSizeRockFromTabdims(const Opm::Deck& deck)
+    {
+        const auto& tabd = deck.get<Opm::ParserKeywords::TABDIMS>().back().getRecord(0);
+        const auto ntPvt = tabd.getItem<Opm::ParserKeywords::TABDIMS::NTPVT>().get<int>(0);
+
+        if (! deck.hasKeyword<Opm::ParserKeywords::ROCKOPTS>()) {
+            return ntPvt;
+        }
+
+        const auto& tableTypeItem = deck.get<Opm::ParserKeywords::ROCKOPTS>().back()
+            .getRecord(0).getItem<Opm::ParserKeywords::ROCKOPTS::TABLE_TYPE>();
+
+        if (tableTypeItem.defaultApplied(0)) {
+            return ntPvt;
+        }
+
+        const auto tableType = tableTypeItem.getTrimmedString(0);
+
+        if (tableType == Opm::ParserKeywords::PVTNUM::keywordName) {
+            return ntPvt;
+        }
+
+        if (tableType == Opm::ParserKeywords::SATNUM::keywordName) {
+            return tabd.getItem<Opm::ParserKeywords::TABDIMS::NTSFUN>().get<int>(0);
+        }
+
+        if (tableType == Opm::ParserKeywords::ROCKNUM::keywordName) {
+            const auto& ntrocc = tabd.getItem<Opm::ParserKeywords::TABDIMS::NTROCC>();
+
+            return ntrocc.defaultApplied(0) ? ntPvt : ntrocc.get<int>(0);
+        }
+
+        throw std::invalid_argument {
+            fmt::format("Unknown {} table type \"{}\"",
+                        Opm::ParserKeywords::ROCKOPTS::keywordName, tableType)
+        };
+    }
+
+    std::size_t defaultTargetSizeRock()
+    {
+        // No TABDIMS => NTSFUN == NTPVT == NTROCC
+        return Opm::ParserKeywords::TABDIMS::NTPVT::defaultValue;
+    }
+
+    std::size_t targetSizeRock(const Opm::Deck& deck)
+    {
+        return deck.hasKeyword<Opm::ParserKeywords::TABDIMS>()
+            ? targetSizeRockFromTabdims(deck)
+            : defaultTargetSizeRock();
+    }
+}
+
 namespace Opm {
 
 namespace {
@@ -606,7 +698,10 @@ std::optional<std::filesystem::path> ParserState::getIncludeFilePath( std::strin
         OpmLog::warning("Replaced one or more backslash with a slash in an INCLUDE path.");
     }
 
-    std::filesystem::path includeFilePath(path);
+    // trim leading and trailing whitespace just like the other simulator
+    std::regex trim_regex("^\\s+|\\s+$");
+    const auto& trimmed_path = std::regex_replace(path, trim_regex, "");
+    std::filesystem::path includeFilePath(trimmed_path);
 
     if (includeFilePath.is_relative())
         includeFilePath = this->rootPath / includeFilePath;
@@ -614,7 +709,11 @@ std::optional<std::filesystem::path> ParserState::getIncludeFilePath( std::strin
     try {
         includeFilePath = std::filesystem::canonical(includeFilePath);
     } catch (const std::filesystem::filesystem_error& fs_error) {
-        parseContext.handleError( ParseContext::PARSE_MISSING_INCLUDE , fmt::format("No such file: {}", includeFilePath.string()), {}, errors);
+        parseContext.handleError( ParseContext::PARSE_MISSING_INCLUDE ,
+                                  fmt::format("File '{}' included via INCLUDE"
+                                              " directive does not exist.",
+                                              trimmed_path),
+                                  {}, errors);
         return {};
     }
 
@@ -625,8 +724,12 @@ void ParserState::addPathAlias( const std::string& alias, const std::string& pat
     this->pathMap.emplace( alias, path );
 }
 
-
-RawKeyword * newRawKeyword(const ParserKeyword& parserKeyword, const std::string& keywordString, ParserState& parserState, const Parser& parser) {
+RawKeyword*
+newRawKeyword(const ParserKeyword& parserKeyword,
+              const std::string&   keywordString,
+              ParserState&         parserState,
+              const Parser&        parser)
+{
     for (const auto& keyword : parserKeyword.prohibitedKeywords()) {
         if (parserState.deck.hasKeyword(keyword)) {
             parserState
@@ -653,14 +756,16 @@ RawKeyword * newRawKeyword(const ParserKeyword& parserKeyword, const std::string
         }
     }
 
-    bool raw_string_keyword = parserKeyword.rawStringKeyword();
-
-    if( parserKeyword.getSizeType() == SLASH_TERMINATED || parserKeyword.getSizeType() == UNKNOWN || parserKeyword.getSizeType() == DOUBLE_SLASH_TERMINATED) {
+    const bool raw_string_keyword = parserKeyword.rawStringKeyword();
 
+    if ((parserKeyword.getSizeType() == SLASH_TERMINATED) ||
+        (parserKeyword.getSizeType() == UNKNOWN) ||
+        (parserKeyword.getSizeType() == DOUBLE_SLASH_TERMINATED))
+    {
         const auto size_type = parserKeyword.getSizeType();
         Raw::KeywordSizeEnum rawSizeType;
 
-        switch(size_type) {
+        switch (size_type) {
             case SLASH_TERMINATED:        rawSizeType = Raw::SLASH_TERMINATED; break;
             case UNKNOWN:                 rawSizeType = Raw::UNKNOWN; break;
             case DOUBLE_SLASH_TERMINATED: rawSizeType = Raw::DOUBLE_SLASH_TERMINATED; break;
@@ -668,87 +773,146 @@ RawKeyword * newRawKeyword(const ParserKeyword& parserKeyword, const std::string
                 throw std::logic_error("Should not be here!");
         }
 
-        return new RawKeyword( keywordString,
-                               parserState.current_path().string(),
-                               parserState.line(),
-                               raw_string_keyword,
-                               rawSizeType);
+        return new RawKeyword(keywordString,
+                              parserState.current_path().string(),
+                              parserState.line(),
+                              raw_string_keyword,
+                              rawSizeType);
     }
 
-    if( parserKeyword.hasFixedSize() ) {
-        auto size_type = Raw::FIXED;
-        if (parserKeyword.isCodeKeyword())
-            size_type = Raw::CODE;
+    if (parserKeyword.getSizeType() == SPECIAL_CASE_ROCK) {
+        if (parserKeyword.getName() != ParserKeywords::ROCK::keywordName) {
+            throw std::logic_error {
+                fmt::format("Special case size handling for ROCK "
+                            "cannot be applied to keyword {}",
+                            parserKeyword.getName())
+            };
+        }
 
-        return new RawKeyword( keywordString,
-                               parserState.current_path().string(),
-                               parserState.line(),
-                               raw_string_keyword,
-                               size_type,
-                               parserKeyword.min_size(),
-                               parserKeyword.getFixedSize());
+        return new RawKeyword {
+            keywordString,
+            parserState.current_path().string(),
+            parserState.line(),
+            raw_string_keyword,
+            Raw::FIXED,
+            parserKeyword.min_size(),
+            targetSizeRock(parserState.deck)
+        };
+    }
+
+    if (parserKeyword.hasFixedSize()) {
+        auto size_type = Raw::FIXED;
+        if (parserKeyword.isCodeKeyword()) {
+            size_type = Raw::CODE;
+        }
+
+        return new RawKeyword(keywordString,
+                              parserState.current_path().string(),
+                              parserState.line(),
+                              raw_string_keyword,
+                              size_type,
+                              parserKeyword.min_size(),
+                              parserKeyword.getFixedSize());
     }
 
     const auto& keyword_size = parserKeyword.getKeywordSize();
     const auto& deck = parserState.deck;
-    auto size_type = parserKeyword.isTableCollection() ? Raw::TABLE_COLLECTION : Raw::FIXED;
+    const auto size_type = parserKeyword.isTableCollection()
+        ? Raw::TABLE_COLLECTION : Raw::FIXED;
 
-    if( deck.hasKeyword(keyword_size.keyword() ) ) {
+    if (deck.hasKeyword(keyword_size.keyword())) {
         const auto& sizeDefinitionKeyword = deck[keyword_size.keyword()].back();
         const auto& record = sizeDefinitionKeyword.getRecord(0);
-        auto targetSize = record.getItem( keyword_size.item() ).get< int >( 0 ) + keyword_size.size_shift();
-        if (parserKeyword.isAlternatingKeyword())
-            targetSize *= std::distance( parserKeyword.begin(), parserKeyword.end() );
 
-        return new RawKeyword( keywordString,
-                               parserState.current_path().string(),
-                               parserState.line(),
-                               raw_string_keyword,
-                               size_type,
-                               parserKeyword.min_size(),
-                               targetSize);
+        auto targetSize = record.getItem(keyword_size.item()).get<int>(0) + keyword_size.size_shift();
+        if (parserKeyword.isAlternatingKeyword()) {
+            targetSize *= std::distance(parserKeyword.begin(), parserKeyword.end());
+        }
+
+        return new RawKeyword(keywordString,
+                              parserState.current_path().string(),
+                              parserState.line(),
+                              raw_string_keyword,
+                              size_type,
+                              parserKeyword.min_size(),
+                              targetSize);
     }
 
-    std::string msg_fmt = fmt::format("Problem with {{keyword}} - missing {0}\n"
-                                      "In {{file}} line {{line}}\n"
-                                      "For the keyword {{keyword}} we expect to read the number of records from keyword {0}, {0} was not found", keyword_size.keyword());
-    parserState.parseContext.handleError(ParseContext::PARSE_MISSING_DIMS_KEYWORD ,
+    const auto msg_fmt = fmt::format("Problem with {{keyword}} - missing {0}\n"
+                                     "In {{file}} line {{line}}\n"
+                                     "For the keyword {{keyword}} we expect "
+                                     "to read the number of records from "
+                                     "keyword {0}, {0} was not found",
+                                     keyword_size.keyword());
+
+    parserState.parseContext.handleError(ParseContext::PARSE_MISSING_DIMS_KEYWORD,
                                          msg_fmt,
-                                         KeywordLocation{keywordString, parserState.current_path().string(), parserState.line()},
-                                         parserState.errors );
+                                         KeywordLocation {
+                                             keywordString,
+                                             parserState.current_path().string(),
+                                             parserState.line()
+                                         },
+                                         parserState.errors);
 
-    const auto& keyword = parser.getKeyword( keyword_size.keyword() );
-    const auto& record = keyword.getRecord(0);
-    const auto& int_item = record.get( keyword_size.item());
+    const auto& keyword  = parser.getKeyword(keyword_size.keyword());
+    const auto& record   = keyword.getRecord(0);
+    const auto& int_item = record.get(keyword_size.item());
 
-    const auto targetSize = int_item.getDefault< int >( ) + keyword_size.size_shift();
-    return new RawKeyword( keywordString,
-                           parserState.current_path().string(),
-                           parserState.line(),
-                           raw_string_keyword,
-                           size_type,
-                           parserKeyword.min_size(),
-                           targetSize);
+    const auto targetSize = int_item.getDefault<int>() + keyword_size.size_shift();
+    return new RawKeyword(keywordString,
+                          parserState.current_path().string(),
+                          parserState.line(),
+                          raw_string_keyword,
+                          size_type,
+                          parserKeyword.min_size(),
+                          targetSize);
 }
 
-
-RawKeyword * newRawKeyword( const std::string& deck_name, ParserState& parserState, const Parser& parser, const std::string_view& line ) {
+RawKeyword*
+newRawKeyword(const std::string&      deck_name,
+              ParserState&            parserState,
+              const Parser&           parser,
+              const std::string_view& line)
+{
     if (deck_name.size() > RawConsts::maxKeywordLength) {
         const std::string keyword8 = deck_name.substr(0, RawConsts::maxKeywordLength);
         if (parser.isRecognizedKeyword(keyword8)) {
-            std::string msg = "Keyword {keyword} to long - only eight first characters recognized\n"
-                              "In {file} line {line}\n";
+            const auto msg = std::string {
+                "Keyword {keyword} to long - only eight "
+                "first characters recognized\n"
+                "In {file} line {line}\n"
+            };
+
             parserState.parseContext.handleError(ParseContext::PARSE_LONG_KEYWORD,
                                                  msg,
-                                                 KeywordLocation{deck_name, parserState.current_path().string(), parserState.line()},
+                                                 KeywordLocation {
+                                                     deck_name,
+                                                     parserState.current_path().string(),
+                                                     parserState.line()
+                                                 },
                                                  parserState.errors);
 
             parserState.unknown_keyword = false;
-            const auto& parserKeyword = parser.getParserKeywordFromDeckName( keyword8 );
+            const auto& parserKeyword = parser.getParserKeywordFromDeckName(keyword8);
+
             return newRawKeyword(parserKeyword, keyword8, parserState, parser);
-        } else {
-            parserState.parseContext.handleUnknownKeyword( deck_name, KeywordLocation{}, parserState.errors );
+        }
+        else if (parser.isBaseRecognizedKeyword(deck_name)) {
+            // Typically an OPM extended keyword such as STRESSEQUILNUM.
+            parserState.unknown_keyword = false;
+            const auto& parserKeyword = parser.getParserKeywordFromDeckName(deck_name);
+
+            return newRawKeyword(parserKeyword, deck_name, parserState, parser);
+        }
+        else {
+            parserState.parseContext.handleUnknownKeyword(deck_name,
+                                                          KeywordLocation {
+                                                              deck_name,
+                                                              parserState.current_path().string(),
+                                                              parserState.line()
+                                                          }, parserState.errors);
             parserState.unknown_keyword = true;
+
             return nullptr;
         }
     }
@@ -756,17 +920,25 @@ RawKeyword * newRawKeyword( const std::string& deck_name, ParserState& parserSta
     if (parser.isRecognizedKeyword(deck_name)) {
         parserState.unknown_keyword = false;
         const auto& parserKeyword = parser.getParserKeywordFromDeckName(deck_name);
+
         return newRawKeyword(parserKeyword, deck_name, parserState, parser);
     }
 
-    if( ParserKeyword::validDeckName(deck_name) ) {
-        parserState.parseContext.handleUnknownKeyword( deck_name, KeywordLocation{}, parserState.errors );
+    if (ParserKeyword::validDeckName(deck_name)) {
+        parserState.parseContext.handleUnknownKeyword(deck_name,
+                                                      KeywordLocation{
+                                                          deck_name,
+                                                          parserState.current_path().string(),
+                                                          parserState.line()},
+                                                      parserState.errors);
         parserState.unknown_keyword = true;
+
         return nullptr;
     }
 
-    if (!parserState.unknown_keyword)
+    if (! parserState.unknown_keyword) {
         parserState.handleRandomText(line);
+    }
 
     return nullptr;
 }
@@ -1046,7 +1218,7 @@ bool parseState( ParserState& parserState, const Parser& parser ) {
 
                     if (deck_keyword.name() == ParserKeywords::IMPORT::keywordName) {
                         bool formatted = deck_keyword.getRecord(0).getItem(1).get<std::string>(0)[0] == 'F';
-                        const auto& import_file = parserState.getIncludeFilePath(deck_keyword.getRecord(0).getItem(0).get<std::string>(0));
+                        const auto& import_file = parserState.getIncludeFilePath(deck_keyword.getRecord(0).getItem(0).getTrimmedString(0));
 
                         ImportContainer import(parser, parserState.deck.getActiveUnitSystem(), import_file.value().string(), formatted, parserState.deck.size());
                         for (auto kw : import)
@@ -1249,14 +1421,20 @@ bool parseState( ParserState& parserState, const Parser& parser ) {
         return (m_wildCardKeywords.count(internalKeywordName) > 0);
     }
 
-    bool Parser::isRecognizedKeyword(const std::string_view& name ) const {
-        if( !ParserKeyword::validDeckName( name ) )
+    bool Parser::isRecognizedKeyword(std::string_view name) const
+    {
+        if (! ParserKeyword::validDeckName(name)) {
             return false;
+        }
 
-        if( m_deckParserKeywords.count( name ) )
-            return true;
+        return (this->m_deckParserKeywords.find(name) != this->m_deckParserKeywords.end())
+            || (this->matchingKeyword(name) != nullptr);
+    }
 
-        return bool( matchingKeyword( name ) );
+    bool Parser::isBaseRecognizedKeyword(std::string_view name) const
+    {
+        return ParserKeyword::validDeckName(name)
+            && (this->m_deckParserKeywords.find(name) != this->m_deckParserKeywords.end());
     }
 
 void Parser::addParserKeyword( ParserKeyword parserKeyword ) {
@@ -1406,6 +1584,7 @@ std::vector<std::string> Parser::getAllDeckNames () const {
 
     bool DeckSection::checkSectionTopology(const Deck& deck,
                                            const Parser& parser,
+                                           Opm::ErrorGuard& errorGuard,
                                            bool ensureKeywordSectionAffiliation)
     {
         if( deck.size() == 0 ) {
@@ -1415,17 +1594,48 @@ std::vector<std::string> Parser::getAllDeckNames () const {
         }
 
         bool deckValid = true;
+        const std::string errorKey = "SECTION_TOPOLOGY_ERROR";
+        // We put errors on the top level to the end of the list to make them
+        // more prominent
+        std::vector<std::string> topLevelErrors;
+        size_t curKwIdx = 0;
 
-        if( deck[0].name() != "RUNSPEC" ) {
-            std::string msg = "The first keyword of a valid deck must be RUNSPEC\n";
+        while(curKwIdx < deck.size() && isGlobalKeyword(deck[curKwIdx])) {
+            ++curKwIdx;
+        }
+
+        bool validKwIdx = curKwIdx < deck.size();
+        if( !validKwIdx || deck[curKwIdx].name() != "RUNSPEC" ) {
+            std::string msg = "The first keyword of a valid deck must be RUNSPEC (is {})\n";
             auto curKeyword = deck[0];
-            OpmLog::warning(Log::fileMessage(curKeyword.location(), msg) );
+            topLevelErrors.push_back(Log::fileMessage(curKeyword.location(),
+                                                      fmt::format(msg,
+                                                                  (validKwIdx)? deck[curKwIdx].name() : "")));
             deckValid = false;
         }
 
-        std::string curSectionName = deck[0].name();
-        size_t curKwIdx = 1;
-        for (; curKwIdx < deck.size(); ++curKwIdx) {
+        std::string curSectionName = validKwIdx? deck[curKwIdx].name() : "";
+
+        for (++curKwIdx; curKwIdx < deck.size(); ++curKwIdx) {
+                auto checker = [&errorGuard, &deckValid, &parser, curSectionName,
+                                ensureKeywordSectionAffiliation, errorKey]
+                    (const std::string& curKeywordName, const KeywordLocation& location)
+                {
+                    const auto& parserKeyword =
+                        parser.getParserKeywordFromDeckName( curKeywordName );
+                    if (ensureKeywordSectionAffiliation && !parserKeyword.isValidSection(curSectionName)) {
+                        std::string msg =
+                            "The keyword '{}' is located in the '{}' section where it is invalid";
+                        errorGuard.addError(errorKey,
+                                            Log::fileMessage(location,
+                                                             fmt::format(msg,
+                                                                         curKeywordName,
+                                                                         curSectionName)
+                                                         ) );
+                        deckValid = false;
+                    }
+                };
+
             const auto& curKeyword = deck[curKwIdx];
             const std::string& curKeywordName = curKeyword.name();
 
@@ -1434,13 +1644,21 @@ std::vector<std::string> Parser::getAllDeckNames () const {
                     // ignore unknown keywords for now (i.e. they can appear in any section)
                     continue;
 
-                const auto& parserKeyword = parser.getParserKeywordFromDeckName( curKeywordName );
-                if (ensureKeywordSectionAffiliation && !parserKeyword.isValidSection(curSectionName)) {
-                    std::string msg =
-                        "The keyword '"+curKeywordName+"' is located in the '"+curSectionName
-                        +"' section where it is invalid";
-                    OpmLog::warning(Log::fileMessage(curKeyword.location(), msg) );
-                    deckValid = false;
+                const bool isOperateKeyword =
+                    Fieldprops::keywords::is_oper_keyword(curKeywordName);
+
+                if (isOperateKeyword) {
+                    for (const auto& record : curKeyword) {
+                        const auto& operName = record.getItem(0).getTrimmedString(0);
+                        if (!parser.isRecognizedKeyword(operName)) {
+                            // ignore unknown keywords
+                            continue;
+                        }
+                        checker(operName, curKeyword.location());
+                    }
+                }
+                else {
+                    checker(curKeyword.name(), curKeyword.location());
                 }
 
                 continue;
@@ -1450,7 +1668,7 @@ std::vector<std::string> Parser::getAllDeckNames () const {
                 if (curKeywordName != "GRID") {
                     std::string msg =
                         "The RUNSPEC section must be followed by GRID instead of "+curKeywordName;
-                    OpmLog::warning(Log::fileMessage(curKeyword.location(), msg) );
+                    topLevelErrors.push_back(Log::fileMessage(curKeyword.location(), msg) );
                     deckValid = false;
                 }
 
@@ -1460,7 +1678,7 @@ std::vector<std::string> Parser::getAllDeckNames () const {
                 if (curKeywordName != "EDIT" && curKeywordName != "PROPS") {
                     std::string msg =
                         "The GRID section must be followed by EDIT or PROPS instead of "+curKeywordName;
-                    OpmLog::warning(Log::fileMessage(curKeyword.location(), msg) );
+                    topLevelErrors.push_back(Log::fileMessage(curKeyword.location(), msg) );
                     deckValid = false;
                 }
 
@@ -1470,7 +1688,7 @@ std::vector<std::string> Parser::getAllDeckNames () const {
                 if (curKeywordName != "PROPS") {
                     std::string msg =
                         "The EDIT section must be followed by PROPS instead of "+curKeywordName;
-                    OpmLog::warning(Log::fileMessage(curKeyword.location(), msg) );
+                    topLevelErrors.push_back(Log::fileMessage(curKeyword.location(), msg) );
                     deckValid = false;
                 }
 
@@ -1480,7 +1698,7 @@ std::vector<std::string> Parser::getAllDeckNames () const {
                 if (curKeywordName != "REGIONS" && curKeywordName != "SOLUTION") {
                     std::string msg =
                         "The PROPS section must be followed by REGIONS or SOLUTION instead of "+curKeywordName;
-                    OpmLog::warning(Log::fileMessage(curKeyword.location(), msg) );
+                    topLevelErrors.push_back(Log::fileMessage(curKeyword.location(), msg) );
                     deckValid = false;
                 }
 
@@ -1490,7 +1708,7 @@ std::vector<std::string> Parser::getAllDeckNames () const {
                 if (curKeywordName != "SOLUTION") {
                     std::string msg =
                         "The REGIONS section must be followed by SOLUTION instead of "+curKeywordName;
-                    OpmLog::warning(Log::fileMessage(curKeyword.location(), msg) );
+                    topLevelErrors.push_back(Log::fileMessage(curKeyword.location(), msg) );
                     deckValid = false;
                 }
 
@@ -1500,7 +1718,7 @@ std::vector<std::string> Parser::getAllDeckNames () const {
                 if (curKeywordName != "SUMMARY" && curKeywordName != "SCHEDULE") {
                     std::string msg =
                         "The SOLUTION section must be followed by SUMMARY or SCHEDULE instead of "+curKeywordName;
-                    OpmLog::warning(Log::fileMessage(curKeyword.location(), msg) );
+                    topLevelErrors.push_back(Log::fileMessage(curKeyword.location(), msg) );
                     deckValid = false;
                 }
 
@@ -1510,7 +1728,7 @@ std::vector<std::string> Parser::getAllDeckNames () const {
                 if (curKeywordName != "SCHEDULE") {
                     std::string msg =
                         "The SUMMARY section must be followed by SCHEDULE instead of "+curKeywordName;
-                    OpmLog::warning(Log::fileMessage(curKeyword.location(), msg) );
+                    topLevelErrors.push_back(Log::fileMessage(curKeyword.location(), msg) );
                     deckValid = false;
                 }
 
@@ -1521,7 +1739,7 @@ std::vector<std::string> Parser::getAllDeckNames () const {
                 std::string msg =
                     "The SCHEDULE section must be the last one ("
                     +curKeywordName+" specified after SCHEDULE)";
-                OpmLog::warning(Log::fileMessage(curKeyword.location(), msg) );
+                topLevelErrors.push_back(Log::fileMessage(curKeyword.location(), msg) );
                 deckValid = false;
             }
         }
@@ -1531,10 +1749,14 @@ std::vector<std::string> Parser::getAllDeckNames () const {
             const auto& curKeyword = deck[deck.size() - 1];
             std::string msg =
                 "The last section of a valid deck must be SCHEDULE (is "+curSectionName+")";
-            OpmLog::warning(Log::fileMessage(curKeyword.location(), msg) );
+            topLevelErrors.push_back(Log::fileMessage(curKeyword.location(), msg) );
             deckValid = false;
         }
 
+        for(const auto& err: topLevelErrors) {
+            errorGuard.addError(errorKey, err);
+        }
+
         return deckValid;
     }
 
