@@ -63,35 +63,36 @@ KeywordSize::KeywordSize(std::size_t in_min_size, const std::string& in_keyword,
 
 KeywordSize::KeywordSize()
     : KeywordSize(SLASH_TERMINATED)
-{
-}
+{}
 
 KeywordSize::KeywordSize(ParserKeywordSizeEnum size_type)
     : m_size_type(size_type)
 {
-    if (size_type == SLASH_TERMINATED)
+    if ((size_type == ParserKeywordSizeEnum::SLASH_TERMINATED) ||
+        (size_type == ParserKeywordSizeEnum::UNKNOWN) ||
+        (size_type == ParserKeywordSizeEnum::DOUBLE_SLASH_TERMINATED) ||
+        (size_type == ParserKeywordSizeEnum::SPECIAL_CASE_ROCK))
+    {
         return;
+    }
 
-    if (size_type == UNKNOWN)
-        return;
-
-    if (size_type == DOUBLE_SLASH_TERMINATED)
-        return;
-
-    throw std::logic_error("This constructor only allows size type UNKNOWN and SLASH_TERMINATED");
+    throw std::logic_error {
+        "This constructor only allows size types "
+        "UNKNOWN, SLASH_TERMINATED, "
+        "DOUBLE_SLASH_TERMINATED, "
+        "and SPECIAL_CASE_ROCK"
+    };
 }
 
 KeywordSize::KeywordSize(std::size_t fixed_size)
     : KeywordSize(fixed_size, false)
-{
-}
+{}
 
 KeywordSize::KeywordSize(std::size_t fixed_size, bool code)
     : m_size_type(code ? FIXED_CODE : FIXED)
     , m_max_size(fixed_size)
     , is_code(code)
-{
-}
+{}
 
 KeywordSize::KeywordSize(std::size_t in_min_size, std::size_t fixed_size, bool code)
     : KeywordSize(fixed_size, code)
@@ -162,25 +163,39 @@ void KeywordSize::min_size(int s) {
     this->m_min_size = s;
 }
 
-std::string
-KeywordSize::construct() const
+std::string KeywordSize::construct() const
 {
-    if (this->m_size_type == UNKNOWN || this->m_size_type == DOUBLE_SLASH_TERMINATED || this->m_size_type == SLASH_TERMINATED)
+    if ((this->m_size_type == UNKNOWN) ||
+        (this->m_size_type == DOUBLE_SLASH_TERMINATED) ||
+        (this->m_size_type == SLASH_TERMINATED) ||
+        (this->m_size_type == SPECIAL_CASE_ROCK))
+    {
         return fmt::format("KeywordSize({})", ParserKeywordSizeEnum2String(this->m_size_type));
+    }
 
-    if (this->m_size_type == FIXED || this->m_size_type == FIXED_CODE) {
-        if (this->min_size().has_value())
-            return fmt::format("KeywordSize({}, {}, {})", this->min_size().value(), std::get<std::size_t>(this->m_max_size.value()), this->is_code);
-        else
-            return fmt::format("KeywordSize({}, {})", std::get<std::size_t>(this->m_max_size.value()), this->is_code);
+    if ((this->m_size_type == FIXED) || (this->m_size_type == FIXED_CODE)) {
+        if (this->min_size().has_value()) {
+            return fmt::format("KeywordSize({}, {}, {})",
+                               this->min_size().value(),
+                               std::get<std::size_t>(this->m_max_size.value()),
+                               this->is_code);
+        }
+
+        return fmt::format("KeywordSize({}, {})",
+                           std::get<std::size_t>(this->m_max_size.value()),
+                           this->is_code);
     }
 
     if (this->m_size_type == OTHER_KEYWORD_IN_DECK) {
         const auto& [size_kw, size_item] = std::get<1>(this->m_max_size.value());
-        if (this->min_size().has_value())
-            return fmt::format("KeywordSize({}, \"{}\", \"{}\", {}, {})", this->min_size().value(), size_kw, size_item, this->is_table_collection, this->shift);
-        else
-            return fmt::format("KeywordSize(\"{}\", \"{}\", {}, {})", size_kw, size_item, this->is_table_collection, this->shift);
+        if (this->min_size().has_value()) {
+            return fmt::format("KeywordSize({}, \"{}\", \"{}\", {}, {})",
+                               this->min_size().value(), size_kw, size_item,
+                               this->is_table_collection, this->shift);
+        }
+
+        return fmt::format("KeywordSize(\"{}\", \"{}\", {}, {})",
+                           size_kw, size_item, this->is_table_collection, this->shift);
     }
 
     throw std::logic_error("No string serialization known?");
@@ -303,11 +318,15 @@ KeywordSize::construct() const
         ParserKeyword(jsonConfig.get_string("name"))
     {
 
-        if (jsonConfig.has_item("deck_names") || jsonConfig.has_item("deck_name_regex") )
+        if (jsonConfig.has_item("deck_names") ||
+            jsonConfig.has_item("deck_name_regex") ||
+            jsonConfig.has_item("deck_name_regex_suffix"))
+        {
             // if either the deck names or the regular expression for deck names are
             // explicitly specified, we do not implicitly add the contents of the 'name'
             // item to the deck names...
             clearDeckNames();
+        }
 
         initSize(jsonConfig);
         if (jsonConfig.has_item("min_size")) {
@@ -488,14 +507,32 @@ KeywordSize::construct() const
     }
 
     void ParserKeyword::initMatchRegex(const Json::JsonObject& jsonObject) {
-        if (!jsonObject.has_item("deck_name_regex"))
-            return;
+        if (jsonObject.has_item("deck_name_regex")) {
+            const Json::JsonObject regexStringObject = jsonObject.get_item("deck_name_regex");
+            if (!regexStringObject.is_string())
+                throw std::invalid_argument("The 'deck_name_regex' JSON item of keyword "+m_name+" need to be a string");
 
-        const Json::JsonObject regexStringObject = jsonObject.get_item("deck_name_regex");
-        if (!regexStringObject.is_string())
-            throw std::invalid_argument("The 'deck_name_regex' JSON item of keyword "+m_name+" need to be a string");
+            setMatchRegex(regexStringObject.as_string());
+        }
 
-        setMatchRegex(regexStringObject.as_string());
+        if (jsonObject.has_item("deck_name_regex_suffix")) {
+            if (!jsonObject.has_item("deck_names")) {
+                throw std::invalid_argument {
+                    fmt::format("The 'deck_name_regex_suffix' JSON item of "
+                                "keyword {} requires an explicit list "
+                                "of 'deck_names' too", this->m_name)
+                };
+            }
+
+            const Json::JsonObject regexStringObject = jsonObject.get_item("deck_name_regex_suffix");
+            if (!regexStringObject.is_string())
+                throw std::invalid_argument {
+                    fmt::format("The 'deck_name_regex_suffix' JSON item of "
+                                "keyword {} must be a string", this->m_name)
+                };
+
+            setMatchRegexSuffix(regexStringObject.as_string());
+        }
     }
 
     void ParserKeyword::addItems(const Json::JsonObject& itemsConfig) {
@@ -764,9 +801,14 @@ void set_dimensions( ParserItem& item,
         return std::get<std::size_t>(max_size.value());
     }
 
-    bool ParserKeyword::hasFixedSize() const {
-        auto size_type = this->keyword_size.size_type();
-        return (size_type == FIXED || size_type == FIXED_CODE || this->m_records.empty());
+    bool ParserKeyword::hasFixedSize() const
+    {
+        const auto size_type = this->keyword_size.size_type();
+
+        return (size_type == FIXED)
+            || (size_type == FIXED_CODE)
+            || (size_type == SPECIAL_CASE_ROCK)
+            || this->m_records.empty();
     }
 
     enum ParserKeywordSizeEnum ParserKeyword::getSizeType() const {
@@ -811,40 +853,91 @@ void set_dimensions( ParserItem& item,
         return !m_matchRegexString.empty();
     }
 
-    void ParserKeyword::setMatchRegex(const std::string& deckNameRegexp) {
+    bool ParserKeyword::hasMatchRegexSuffix() const {
+        return !this->m_matchRegexSuffix.empty();
+    }
+
+    void ParserKeyword::setMatchRegex(const std::string& deckNameRegexp)
+    {
+        if (deckNameRegexp.empty()) {
+            return;
+        }
+
         try {
             m_matchRegex = std::regex(deckNameRegexp);
             m_matchRegexString = deckNameRegexp;
         }
-        catch (const std::exception &e) {
-            std::cerr << "Warning: Malformed regular expression for keyword '" << getName() << "':\n"
-                      << "\n"
-                      << e.what() << "\n"
-                      << "\n"
-                      << "Ignoring expression!\n";
+        catch (const std::regex_error&) {
+            throw std::invalid_argument {
+                fmt::format("'{}' is not a valid regular expression ('deck_name_regex') "
+                            "for keyword '{}'", deckNameRegexp, this->getName())
+            };
+        }
+    }
+
+    void ParserKeyword::setMatchRegexSuffix(const std::string& deckNameRegexp)
+    {
+        if (deckNameRegexp.empty()) {
+            return;
+        }
+
+        try {
+            const auto sfx = std::regex { deckNameRegexp };
+            this->m_matchRegexSuffix = deckNameRegexp;
+        }
+        catch (const std::regex_error&) {
+            throw std::invalid_argument {
+                fmt::format("'{}' is not a valid regular expression suffix "
+                            "('deck_name_regex_suffix') for keyword '{}'",
+                            deckNameRegexp, this->getName())
+            };
         }
     }
 
     bool ParserKeyword::matches(const std::string_view& name ) const {
-        if (!validDeckName(name ))
+        if (!validDeckName(name))
             return false;
 
-        else if( m_deckNames.count( std::string(name) ) )
+        else if (matchesDeckNames(name))
             return true;
 
-        else if (hasMatchRegex()) {
-            bool match = std::regex_match( name.begin(), name.end(), m_matchRegex);
-            if (match)
-                return true;
+        else if (hasMatchRegex() && std::regex_match(std::string{name}, m_matchRegex))
+            return true;
+
+        return false;
+    }
+
+    bool ParserKeyword::matchesDeckNames(std::string_view name) const
+    {
+        const auto nameStr = std::string { name };
+
+        if (m_deckNames.find(nameStr) != m_deckNames.end()) {
+            return true;
         }
 
-        // Last desperate attempt - go through the deckNames list and
-        // interpret the elements as a regular expression.
-        for (const auto& deck_name : this->m_deckNames) {
-            if (std::regex_match(name.begin(), name.end(), std::regex(deck_name)))
-                return true;
-        }
-        return false;
+        // Try to match nameStr against all m_deckNames while treating each
+        // deck name as a regular expression, or, if applicable, when the
+        // name is extended by appending the m_matchRegexSuffix.  The latter
+        // is mostly a special case for region-level summary keywords whose
+        // strings may include a tag referring to a user-defined region set
+        // entered using the FIP* keywords--FIPNUM excluded.  Examples of
+        // such strings are
+        //
+        //   RPR__XYZ
+        //   ROPR_XYZ
+        //   RODENXYZ
+        //
+        // denoting the region-level average pressures, region-level oil
+        // production rate, and region-level average mass density of oil
+        // respectively defined for the FIPXYZ region set.
+        return std::any_of(this->m_deckNames.begin(),
+                           this->m_deckNames.end(),
+            [&nameStr, this](const std::string& deckName)
+        {
+            return std::regex_match(nameStr, std::regex { deckName })
+                || (this->hasMatchRegexSuffix() &&
+                    std::regex_match(nameStr, std::regex { deckName + m_matchRegexSuffix }));
+        });
     }
 
     std::string ParserKeyword::createDeclaration(const std::string& indent) const {
@@ -950,6 +1043,10 @@ void set_dimensions( ParserItem& item,
         if (hasMatchRegex())
             ss << indent << "setMatchRegex(\"" << m_matchRegexString << "\");" << '\n';
 
+        // set the deck name match regex suffix
+        if (hasMatchRegexSuffix())
+            ss << indent << "setMatchRegexSuffix(\"" << m_matchRegexSuffix << "\");\n";
+
         if (this->keyword_size.code())
             ss << indent << "setCodeEnd(\"" << this->code_end << "\");" << '\n';
 
@@ -1007,6 +1104,7 @@ void set_dimensions( ParserItem& item,
         if(    m_name              != rhs.m_name
             || this->code_end      != rhs.code_end
             || m_matchRegexString  != rhs.m_matchRegexString
+            || m_matchRegexSuffix  != rhs.m_matchRegexSuffix
             || isCodeKeyword()     != rhs.isCodeKeyword()
             || isDataKeyword()     != rhs.isDataKeyword())
           return false;
