@@ -24,9 +24,11 @@
 #include <opm/input/eclipse/Schedule/MSW/SICD.hpp>
 #include <opm/input/eclipse/Schedule/MSW/Valve.hpp>
 
+#include <algorithm>
 #include <cassert>
 #include <stdexcept>
 #include <string>
+#include <vector>
 
 namespace {
 
@@ -63,7 +65,7 @@ namespace Opm {
         , m_branch(rst_segment.branch)
         , m_outlet_segment(rst_segment.outlet_segment)
         , m_total_length( rst_segment.dist_bhp_ref )
-        , m_depth(rst_segment.bhp_ref_dz)
+        , m_depth(rst_segment.node_depth)
         , m_internal_diameter(if_invalid_value(rst_segment.diameter))
         , m_roughness(if_invalid_value(rst_segment.roughness))
         , m_cross_area(if_invalid_value(rst_segment.area))
@@ -277,8 +279,15 @@ namespace Opm {
         return m_inlet_segments;
     }
 
-    void Segment::addInletSegment(const int segment_number_in) {
-        m_inlet_segments.push_back(segment_number_in);
+    void Segment::addInletSegment(const int segment_number_in)
+    {
+        auto segPos = std::find(this->m_inlet_segments.begin(),
+                                this->m_inlet_segments.end(),
+                                segment_number_in);
+
+        if (segPos == this->m_inlet_segments.end()) {
+            this->m_inlet_segments.push_back(segment_number_in);
+        }
     }
 
     double Segment::invalidValue() {
