@@ -161,15 +161,16 @@ bool is_RPTRST_mnemonic( const std::string& kw ) {
      */
     static constexpr const char* valid[] = {
         "ACIP",     "ACIS",     "ALLPROPS", "BASIC",  "BG",       "BO",
-        "BW",       "CELLINDX", "COMPRESS", "CONV",   "DEN",      "DRAIN",
+        "BW",       "CELLINDX", "COMPRESS", "CONV",   "DEN",      "DENG",
+        "DENO", "DENW", "DRAIN",
         "DRAINAGE", "DYNREG",   "FIP",      "FLORES", "FLOWS",    "FREQ",
         "GIMULT",   "HYDH",     "HYDHFW",   "KRG",    "KRO",      "KRW",
         "NOGRAD",   "NORST",    "NPMREB",   "PBPD",   "PCOG",     "PCOW",
         "PERMREDN", "POIS",     "PORO",     "PORV",   "POT",      "PRES",
         "RFIP",     "RK",       "ROCKC",    "RPORV",  "RSSAT",    "RVSAT",
         "SAVE",     "SDENO",    "SFIP",     "SFREQ",  "SGTRAP",   "SIGM_MOD",
-        "STREAM",   "SURFBLK",  "TRAS",     "VELGAS", "VELOCITY", "VELOIL",
-        "VELWAT",   "VISC",
+        "STREAM",   "SURFBLK",  "TEMP",     "TRAS",     "VELGAS",  "VELOCITY",
+        "VELOIL", "VELWAT", "VGAS", "VISC", "VOIL", "VWAT",
     };
 
     return std::binary_search( std::begin( valid ), std::end( valid ), kw );
