@@ -36,10 +36,12 @@
 #include <opm/input/eclipse/Schedule/Well/WellMICPProperties.hpp>
 #include <opm/input/eclipse/Schedule/Well/WellPolymerProperties.hpp>
 #include <opm/input/eclipse/Schedule/Well/WellTracerProperties.hpp>
+#include <opm/input/eclipse/Schedule/Well/WVFPDP.hpp>
 #include <opm/input/eclipse/Schedule/Well/WVFPEXP.hpp>
 
 #include <opm/input/eclipse/Units/Units.hpp>
 
+#include <opm/common/utility/OpmInputError.hpp>
 #include <opm/common/utility/shmatch.hpp>
 
 #include <opm/input/eclipse/Parser/ParserKeywords/S.hpp>
@@ -49,7 +51,9 @@
 #include <opm/input/eclipse/Deck/DeckKeyword.hpp>
 
 #include "../MSW/Compsegs.hpp"
+#include "../eval_uda.hpp"
 
+#include <cassert>
 #include <cmath>
 #include <cstddef>
 #include <memory>
@@ -305,9 +309,11 @@ Well::Well(const RestartIO::RstWell& rst_well,
     connections(std::make_shared<WellConnections>(order_from_int(rst_well.completion_ordering), headI, headJ)),
     production(std::make_shared<WellProductionProperties>(unit_system_arg, wname)),
     injection(std::make_shared<WellInjectionProperties>(unit_system_arg, wname)),
+    wvfpdp(std::make_shared<WVFPDP>()),
     wvfpexp(explicitTHPOptions(rst_well)),
     status(status_from_int(rst_well.well_status)),
-    well_temperature(Metric::TemperatureOffset + ParserKeywords::STCOND::TEMPERATURE::defaultValue)
+    well_temperature(Metric::TemperatureOffset + ParserKeywords::STCOND::TEMPERATURE::defaultValue),
+    well_inj_mult(std::nullopt)
 {
     if (this->wtype.producer()) {
         auto p = std::make_shared<WellProductionProperties>(this->unit_system, wname);
@@ -483,10 +489,11 @@ Well::Well(const std::string& wname_arg,
     connections(std::make_shared<WellConnections>(ordering_arg, headI, headJ)),
     production(std::make_shared<WellProductionProperties>(unit_system, wname)),
     injection(std::make_shared<WellInjectionProperties>(unit_system, wname)),
+    wvfpdp(std::make_shared<WVFPDP>()),
     wvfpexp(std::make_shared<WVFPEXP>()),
     status(Status::SHUT),
-    well_temperature(Metric::TemperatureOffset + ParserKeywords::STCOND::TEMPERATURE::defaultValue)
-
+    well_temperature(Metric::TemperatureOffset + ParserKeywords::STCOND::TEMPERATURE::defaultValue),
+    well_inj_mult(std::nullopt)
 {
     auto p = std::make_shared<WellProductionProperties>(this->unit_system, this->wname);
     p->whistctl_cmode = whistctl_cmode;
@@ -529,6 +536,8 @@ Well Well::serializationTestObject()
     result.wvfpexp = std::make_shared<WVFPEXP>(WVFPEXP::serializationTestObject());
     result.m_pavg = PAvg();
     result.well_temperature = 10.0;
+    result.well_inj_mult = InjMult::serializationTestObject();
+    result.m_filter_concentration = UDAValue::serializationTestObject();
 
     return result;
 }
@@ -625,6 +634,15 @@ bool Well::updateEconLimits(std::shared_ptr<WellEconProductionLimits> econ_limit
     return false;
 }
 
+bool Well::updateWVFPDP(std::shared_ptr<WVFPDP> wvfpdp_arg) {
+    if (*this->wvfpdp != *wvfpdp_arg) {
+        this->wvfpdp = std::move(wvfpdp_arg);
+        return true;
+    }
+
+    return false;
+}
+
 bool Well::updateWVFPEXP(std::shared_ptr<WVFPEXP> wvfpexp_arg) {
     if (*this->wvfpexp != *wvfpexp_arg) {
         this->wvfpexp = std::move(wvfpexp_arg);
@@ -1103,6 +1121,10 @@ const WellTracerProperties& Well::getTracerProperties() const {
     return *this->tracer_properties;
 }
 
+const WVFPDP& Well::getWVFPDP() const {
+    return *this->wvfpdp;
+}
+
 const WVFPEXP& Well::getWVFPEXP() const {
     return *this->wvfpexp;
 }
@@ -1122,6 +1144,20 @@ const WellSegments& Well::getSegments() const {
         throw std::logic_error("Asked for segment information in not MSW well: " + this->name());
 }
 
+int Well::maxSegmentID() const
+{
+    return (this->segments == nullptr)
+        ? 0
+        : this->segments->maxSegmentID();
+}
+
+int Well::maxBranchID() const
+{
+    return (this->segments == nullptr)
+        ? 0
+        : this->segments->maxBranchID();
+}
+
 const Well::WellInjectionProperties& Well::getInjectionProperties() const {
     return *this->injection;
 }
@@ -1212,9 +1248,46 @@ bool Well::handleWELOPENConnections(const DeckRecord& record, Connection::State
     return this->updateConnections(std::move(new_connections), false);
 }
 
+bool Well::handleCSKINConnections(const DeckRecord& record) {
+    // Lambda expression to check if record coordinates match connection coordinates
+    auto match = [=]( const Connection &c) -> bool {
+        if (!match_eq(c.getI(), record, "I" , -1)) return false;
+        if (!match_eq(c.getJ(), record, "J" , -1)) return false;
+        if (!match_ge(c.getK(), record, "K_UPPER", -1)) return false;
+        if (!match_le(c.getK(), record, "K_LOWER", -1)) return false;
 
+        return true;
+    };
 
+    // Generate a new connection which will be updated with new connection skin factor
+    auto new_connections = std::make_shared<WellConnections>(this->connections->ordering(), this->headI, this->headJ);
 
+    // Update skin factor
+    double skin_factor = record.getItem("CONNECTION_SKIN_FACTOR").get<double>(0);
+    const double angle = 6.2831853071795864769252867665590057683943387987502116419498;
+    for (auto c : *this->connections) {
+        if (match(c)) {
+            // Check for potential negative new CF
+            if ((std::log(c.r0() / std::min(c.rw(), c.r0())) + skin_factor) < 0.0) {
+                throw std::runtime_error("Negative connection transmissibility factor produced by CSKIN for well " 
+                                         + name());
+            }
+
+            // Calculate new connection transmissibility factor
+            double CF = angle * c.Kh() / (std::log(c.r0() / std::min(c.rw(), c.r0())) + skin_factor);
+
+            // Apply last known WPIMULT (defaulted to 1.0)
+            CF *= c.wpimult();
+
+            // Set skin factor and connection factor
+            c.setSkinFactor(skin_factor);
+            c.setCF(CF);
+        }
+        new_connections->add(c);
+    }
+
+    return this->updateConnections(std::move(new_connections), false);
+}
 
 bool Well::handleCOMPLUMP(const DeckRecord& record) {
 
@@ -1269,6 +1342,101 @@ bool Well::handleWPIMULT(const DeckRecord& record) {
     return this->updateConnections(std::move(new_connections), false);
 }
 
+bool Well::handleWINJCLN(const DeckRecord& record, const KeywordLocation& location) {
+    auto match = [=] ( const Connection& c) -> bool {
+        if (!match_eq(c.getI()  , record, "I", -1)) return false;
+        if (!match_eq(c.getJ()  , record, "J", -1)) return false;
+        if (!match_eq(c.getK()  , record, "K", -1)) return false;
+
+        return true;
+    };
+
+    const double fraction_removal = record.getItem<ParserKeywords::WINJCLN::FRAC_REMOVE>().getSIDouble(0);
+    if (fraction_removal < 0. || fraction_removal > 1.) {
+        const auto reason = fmt::format(" the item 2 in keyword WINJCLN must be between 0 and 1, while a "
+                                                    "value {} is given.", fraction_removal);
+        throw OpmInputError(reason, location);
+    }
+    const double fraction_remain = 1. - fraction_removal;
+    auto new_connections = std::make_shared<WellConnections>(this->connections->ordering(), this->headI, this->headJ);
+    for (auto c : *(this->connections)) {
+        if (match(c)) {
+             auto filter_cake = c.getFilterCake();
+             filter_cake.applyCleanMultiplier(fraction_remain);
+             c.setFilterCake(filter_cake);
+        }
+        new_connections->add(c);
+    }
+    return this->updateConnections(std::move(new_connections), false);
+}
+
+bool Well::handleWINJDAM(const DeckRecord& record, const KeywordLocation& location) {
+    auto match = [=] ( const Connection& c) -> bool {
+        if (!match_eq(c.getI()  , record, "I", -1)) return false;
+        if (!match_eq(c.getJ()  , record, "J", -1)) return false;
+        if (!match_eq(c.getK()  , record, "K", -1)) return false;
+
+        return true;
+    };
+
+    const FilterCake filter_cake {record, location};
+    auto new_connections = std::make_shared<WellConnections>(this->connections->ordering(),
+                                                                          this->headI, this->headJ);
+    for (auto c : *(this->connections)) {
+        if (match(c)) {
+            c.setFilterCake(filter_cake);
+        }
+        new_connections->add(c);
+    }
+    return this->updateConnections(std::move(new_connections), false);
+}
+
+bool Well::handleWINJMULT(const Opm::DeckRecord& record, const KeywordLocation& location) {
+    // for this keyword, the default for I, J, K will be negative
+    // it is not totally clear how specifying 0 or a negative values will work
+    // current match_eq function only treats 0 and default values for all connections,
+    // we might need to revisit this part later when complication regarding this occurs.
+    // it is possible that changing (item.get<int>(0) == 0); to (item.get<int>(0) <= 0) is solution to go
+    // while it remains to be discussed.
+    auto match = [=] ( const Connection& c) -> bool {
+        if (!match_eq(c.getI()  , record, "I", -1)) return false;
+        if (!match_eq(c.getJ()  , record, "J", -1)) return false;
+        if (!match_eq(c.getK()  , record, "K", -1)) return false;
+
+        return true;
+    };
+
+    using Kw = ParserKeywords::WINJMULT;
+    const InjMultMode mode = InjMult::injMultModeFromString(record.getItem<Kw::MODE>().getTrimmedString(0), location);
+    const bool mode_change = (this->inj_mult_mode != mode);
+    if (mode_change) {
+        this->inj_mult_mode = mode;
+    }
+    const double fracture_pressure = record.getItem<Kw::FRACTURING_PRESSURE>().getSIDouble(0);
+    const double multiple_gradient = record.getItem<Kw::MULTIPLIER_GRADIENT>().getSIDouble(0);
+    auto new_connections = std::make_shared<WellConnections>(this->connections->ordering(), this->headI, this->headJ);
+    const InjMult inj_mult {fracture_pressure, multiple_gradient};
+    bool connections_update = false;
+    bool well_inj_update = false;
+
+    if (mode == InjMultMode::WREV) {
+        // all the connections will share the same INJMULT setup when under WREV
+        // it is stored in the Well object
+        this->well_inj_mult = inj_mult;
+        well_inj_update = true;
+    } else if (mode == InjMultMode::CREV || mode == InjMultMode::CIRR){
+        for (auto c : *this->connections) {
+            if (match(c)) {
+                c.setInjMult(inj_mult);
+            }
+            new_connections->add(c);
+        }
+        connections_update = this->updateConnections(std::move(new_connections), false);
+    }
+
+    return mode_change || connections_update || well_inj_update;
+}
+
 
 bool Opm::Well::applyGlobalWPIMULT(const double scaling_factor)
 {
@@ -1357,45 +1525,6 @@ Well::GasInflowEquation Well::gas_inflow_equation() const {
     return this->gas_inflow;
 }
 
-bool Well::canOpen() const {
-    if (this->allow_cross_flow)
-        return true;
-
-    /*
-      If the UDAValue is in string mode we return true unconditionally, without
-      evaluating the internal UDA value.
-    */
-    if (this->wtype.producer()) {
-        const auto& prod = *this->production;
-        if (prod.OilRate.is<std::string>())
-            return true;
-
-        if (prod.GasRate.is<std::string>())
-          return true;
-
-        if (prod.WaterRate.is<std::string>())
-          return true;
-
-        if (!prod.OilRate.zero())
-            return true;
-
-        if (!prod.GasRate.zero())
-            return true;
-
-        if (!prod.WaterRate.zero())
-            return true;
-
-        return false;
-    } else {
-        const auto& inj = *this->injection;
-        if (inj.surfaceInjectionRate.is<std::string>())
-            return true;
-
-        return !inj.surfaceInjectionRate.zero();
-    }
-}
-
-
 bool Well::predictionMode() const {
     return this->prediction_mode;
 }
@@ -1574,14 +1703,12 @@ bool Well::operator==(const Well& data) const {
         && (this->m_pavg == data.m_pavg)
         && (this->getInjectionProperties() == data.getInjectionProperties())
         && (this->well_temperature == data.well_temperature)
+        && (this->inj_mult_mode == data.inj_mult_mode)
+        && (this->well_inj_mult == data.well_inj_mult)
+        && (this->m_filter_concentration == data.m_filter_concentration)
         ;
 }
 
-
-PAvgCalculator Well::pavg_calculator(const EclipseGrid& grid, const std::vector<double>& porv) const {
-    return PAvgCalculator(this->name(), this->getWPaveRefDepth(), grid, porv, this->getConnections(), this->m_pavg);
-}
-
 } // namespace Opm
 
 int Opm::Well::eclipseControlMode(const Well::InjectorCMode imode,
@@ -1659,3 +1786,25 @@ int Opm::Well::eclipseControlMode(const Well&         well,
         return eclipseControlMode(ctrl.cmode, well.injectorType());
     }
 }
+
+Opm::Well::InjMultMode Opm::Well::getInjMultMode() const {
+    return this->inj_mult_mode;
+}
+
+const Opm::InjMult& Opm::Well::getWellInjMult() const {
+    assert(this->aciveWellInjMult());
+    return this->well_inj_mult.value();
+}
+
+bool Opm::Well::aciveWellInjMult() const {
+   return this->well_inj_mult.has_value();
+}
+
+
+void Opm::Well::setFilterConc(const UDAValue& conc) {
+    this->m_filter_concentration = conc;
+}
+
+double Opm::Well::evalFilterConc(const SummaryState& summary_sate) const {
+    return UDA::eval_well_uda(this->m_filter_concentration, this->name(), summary_sate, 0.);
+}
\ No newline at end of file
