@@ -23,6 +23,7 @@
 #include <opm/common/ErrorMacros.hpp>
 
 #include <algorithm>
+#include <cmath>
 #include <cstring>
 #include <filesystem>
 #include <fstream>
@@ -30,11 +31,8 @@
 #include <iomanip>
 #include <numeric>
 #include <string>
-#include <sstream>
 #include <stdexcept>
 
-#include <math.h>
-
 namespace Opm { namespace EclIO {
 
 using NNCentry = std::tuple<int, int, int, int, int,int, float>;
