@@ -105,7 +105,9 @@ void RstState::load_tuning(const std::vector<int>& intehead,
     this->tuning.MXWSIT  = intehead[ VI::intehead::MXWSIT ];
     this->tuning.MXWPIT  = intehead[ VI::intehead::MXWPIT ];
 
-    tuning.TSINIT = this->unit_system.to_si(M::time, doubhead[VI::doubhead::TsInit]);
+    double tsinit = this->unit_system.to_si(M::time, doubhead[VI::doubhead::TsInit]);
+    //tuning.TSINIT = tsinit > 0 ? std::optional<double>{ tsinit } : std::nullopt;
+    tuning.TSINIT = VI::DoubHeadValue::TSINITHasNoValue(tsinit) ? std::nullopt : std::optional<double>{ tsinit };
     tuning.TSMAXZ = this->unit_system.to_si(M::time, doubhead[VI::doubhead::TsMaxz]);
     tuning.TSMINZ = this->unit_system.to_si(M::time, doubhead[VI::doubhead::TsMinz]);
     tuning.TSMCHP = this->unit_system.to_si(M::time, doubhead[VI::doubhead::TsMchp]);
@@ -321,10 +323,11 @@ void RstState::add_actions(const Parser& parser,
         for (std::size_t icond = 0; icond < actdims.max_conditions(); icond++) {
             const auto zacn_offset = index * zacn_action_size + icond * zacn_cond_size;
             const auto iacn_offset = index * iacn_action_size + icond * iacn_cond_size;
-            const auto sacn_offset = index * sacn_action_size + icond * sacn_cond_size;
 
-            if (RstAction::Condition::valid(&zacn[zacn_offset], &iacn[iacn_offset]))
+            if (RstAction::Condition::valid(&zacn[zacn_offset], &iacn[iacn_offset])) {
+                const auto sacn_offset = index * sacn_action_size + icond * sacn_cond_size;
                 conditions.emplace_back(&zacn[zacn_offset], &iacn[iacn_offset], &sacn[sacn_offset]);
+            }
         }
 
         const auto& name = zact[index * zact_action_size + 0];
