@@ -115,6 +115,8 @@ RstWell::RstWell(const ::Opm::UnitSystem& unit_system,
     resv_target(                                                     swel_value(swel[VI::SWell::ResVRateTarget])),
     thp_target(                                                      swel_value(swel[VI::SWell::THPTarget])),
     bhp_target_float(                                                swel[VI::SWell::BHPTarget]),
+    vfp_bhp_adjustment(  unit_system.to_si(M::pressure,              swel[VI::SWell::VfpBhpAdjustment])),
+    vfp_bhp_scaling_factor(                                          swel[VI::SWell::VfpBhpScalingFact]),
     hist_lrat_target(    unit_system.to_si(M::liquid_surface_rate,   swel[VI::SWell::HistLiqRateTarget])),
     hist_grat_target(    unit_system.to_si(M::gas_surface_rate,      swel[VI::SWell::HistGasRateTarget])),
     hist_bhp_target(     unit_system.to_si(M::pressure,              swel[VI::SWell::HistBHPTarget])),
