@@ -81,6 +81,22 @@ initFromState(const EclipseState& eclState)
                 stoneEtas_.push_back(table.eta);
             }
         }
+        
+        const auto& ppcwmaxTables = tables.getPpcwmax();
+        this->enablePpcwmax_ = !ppcwmaxTables.empty();
+
+        if (this->enablePpcwmax_) {
+            maxAllowPc_.clear();
+            modifySwl_.clear();
+
+            maxAllowPc_.reserve(numSatRegions);
+            modifySwl_.reserve(numSatRegions);
+
+            for (const auto& table : ppcwmaxTables) {
+                maxAllowPc_.push_back(table.max_cap_pres);
+                modifySwl_.push_back(table.option);
+            }
+        }
     }
 
     this->unscaledEpsInfo_.resize(numSatRegions);
@@ -101,6 +117,17 @@ initFromState(const EclipseState& eclState)
         this->unscaledEpsInfo_[satRegionIdx]
             .extractUnscaled(rtep, rfunc, satRegionIdx);
     }
+
+    // WAG hysteresis parameters per SATNUM.
+    if (eclState.runspec().hysterPar().activeWag()) {
+        if (numSatRegions != eclState.getWagHysteresis().size())
+            throw std::runtime_error("Inconsistent Wag-hysteresis data");
+        wagHystersisConfig_.resize(numSatRegions);
+        for (unsigned satRegionIdx = 0; satRegionIdx < numSatRegions; ++satRegionIdx) {
+            wagHystersisConfig_[satRegionIdx] = std::make_shared<WagHysteresisConfig::
+                WagHysteresisConfigRecord >(eclState.getWagHysteresis()[satRegionIdx]);
+        }
+    }
 }
 
 template<class TraitsT>
@@ -112,12 +139,13 @@ initParamsForElements(const EclipseState& eclState, size_t numCompressedElems)
 }
 
 template<class TraitsT>
-typename TraitsT::Scalar EclMaterialLawManager<TraitsT>::
+std::pair<typename TraitsT::Scalar, bool> EclMaterialLawManager<TraitsT>::
 applySwatinit(unsigned elemIdx,
               Scalar pcow,
               Scalar Sw)
 {
     auto& elemScaledEpsInfo = oilWaterScaledEpsInfoDrainage_[elemIdx];
+    bool newSwatInit = false;
 
     // TODO: Mixed wettability systems - see ecl kw OPTIONS switch 74
 
@@ -152,7 +180,27 @@ applySwatinit(unsigned elemIdx,
         constexpr const Scalar pcowAtSwThreshold = 1.0; //Pascal
         // avoid divison by very small number
         if (std::abs(pcowAtSw) > pcowAtSwThreshold) {
-            elemScaledEpsInfo.maxPcow *= pcow/pcowAtSw;
+            // Scale max. capillary pressure to honor SWATINIT value
+            Scalar newMaxPcow = elemScaledEpsInfo.maxPcow * (pcow/pcowAtSw);
+
+            // Limit max. capillary pressure with PPCWMAX
+            int satRegionIdx = satnumRegionIdx(elemIdx);
+            if (enablePpcwmax() && (newMaxPcow > maxAllowPc_[satRegionIdx])) {
+                // Two options in PPCWMAX to modify connate Sw or not.  In both cases, init. Sw needs to be
+                // re-calculated (done in opm-simulators)
+                newSwatInit = true;
+                if (modifySwl_[satRegionIdx] == false) {
+                    // Max. cap. pressure set to PCWO in PPCWMAX
+                    elemScaledEpsInfo.maxPcow = maxAllowPc_[satRegionIdx];
+                }
+                else {
+                    // Max. cap. pressure remains unscaled and connate Sw is set to SWATINIT value
+                    elemScaledEpsInfo.Swl = Sw;
+                }
+            }
+            // Max. cap. pressure adjusted from SWATINIT data
+            else
+                elemScaledEpsInfo.maxPcow = newMaxPcow;
             auto& elemEclEpsScalingPoints = oilWaterScaledEpsPointsDrainage(elemIdx);
             elemEclEpsScalingPoints.init(elemScaledEpsInfo,
                                          *oilWaterEclEpsConfig_,
@@ -160,7 +208,7 @@ applySwatinit(unsigned elemIdx,
         }
     }
 
-    return Sw;
+    return {Sw, newSwatInit};
 }
 
 template<class TraitsT>
