@@ -21,6 +21,7 @@
 #include <opm/common/ErrorMacros.hpp>
 
 #include <iostream>
+#include <sstream>
 #include <string>
 #include <getopt.h>
 #include <fstream>
@@ -251,8 +252,12 @@ int main(int argc, char** argv) {
             comparator.results_rft();
         }
 
-        if (comparator.getNoErrors() > 0)
-            OPM_THROW(std::runtime_error, comparator.getNoErrors() << " errors encountered in comparisons.");
+        if (comparator.getNoErrors() > 0) {
+            std::ostringstream str;
+            str << comparator.getNoErrors()
+                << " errors encountered in comparisons.";
+            OPM_THROW(std::runtime_error, str.str());
+        }
     }
 
     catch (const std::exception& e) {
