@@ -28,7 +28,12 @@
 #define BOOST_TEST_MODULE FieldPropsTests
 
 #include <boost/test/unit_test.hpp>
+#include <boost/version.hpp>
+#if BOOST_VERSION / 100000 == 1 && BOOST_VERSION / 100 % 1000 < 71
+#include <boost/test/floating_point_comparison.hpp>
+#else
 #include <boost/test/tools/floating_point_comparison.hpp>
+#endif
 
 #include <opm/common/utility/OpmInputError.hpp>
 
@@ -787,6 +792,80 @@ SOGCR    -- Requires 'OIL'
 )"), Opm::OpmInputError);
 }
 
+BOOST_AUTO_TEST_CASE(SGWFN) {
+    std::string deck_string = R"(
+RUNSPEC
+
+DIMENS
+6 6 3 /
+
+WATER
+GAS
+CO2STORE
+
+TABDIMS
+/
+
+GRID
+
+DXV
+  6*100.0 /
+DYV
+  6*100.0 /
+DZV
+  3*5.0 /
+
+TOPS
+  36*2000.0 /
+
+PERMX
+  108*100.0 /
+PERMY
+  108*100.0 /
+PERMZ
+  108*10.0 /
+PORO
+  108*0.3 /
+
+PROPS
+
+SGWFN
+0.00   0.00     0.9      0.0
+0.05   0.02     0.8      5.
+0.10   0.03     0.5      10.0
+0.80   1.00     0.0      20.0
+/
+)";
+    const auto es = ::Opm::EclipseState {
+        ::Opm::Parser{}.parseString(deck_string)
+    };
+    const auto& tm      = es.getTableManager();
+    const auto& ph      = es.runspec().phases();
+    const auto  tolcrit = 0.0;
+
+    auto rtepPtr = satfunc::getRawTableEndpoints(tm, ph, tolcrit);
+
+    // Water end-points
+    {
+        const auto swl  = rtepPtr.connate .water;
+        const auto swcr = rtepPtr.critical.water;
+        const auto swu  = rtepPtr.maximum .water;
+        BOOST_CHECK_CLOSE(swl [0], 0.2, 1.0e-10);
+        BOOST_CHECK_CLOSE(swcr[0], 0.2, 1.0e-10);
+        BOOST_CHECK_CLOSE(swu [0], 1.0, 1.0e-10);
+    }
+
+    // Gas end-points
+    {
+        const auto sgl  = rtepPtr.connate .gas;
+        const auto sgcr = rtepPtr.critical.gas;
+        const auto sgu  = rtepPtr.maximum .gas;
+        BOOST_CHECK_CLOSE(sgl [0], 0.0, 1.0e-10);
+        BOOST_CHECK_CLOSE(sgcr[0], 0.0, 1.0e-10);
+        BOOST_CHECK_CLOSE(sgu [0], 0.8, 1.0e-10);
+    }
+}
+
 namespace {
     std::string satfunc_model_setup()
     {
@@ -1849,6 +1928,47 @@ BOOST_AUTO_TEST_CASE(SatFunc_EndPts_Family_II_TolCrit_Large) {
 
 // =====================================================================
 
+BOOST_AUTO_TEST_CASE(Equivalent_FIP_Keys) {
+    std::string deck_string = R"(
+GRID
+
+PORO
+   200*0.15 /
+
+REGIONS
+
+FIPUNIT
+  100*1 100*2 /
+
+FIPUNIX
+  100*3 100*4 /
+)";
+
+    const EclipseGrid grid { 10, 10, 2 };
+    const Deck deck = Parser{}.parseString(deck_string);
+
+    BOOST_CHECK_MESSAGE(deck.hasKeyword("FIPUNIT"),
+                        R"(Input deck must have "FIPUNIT" region set)");
+
+    BOOST_CHECK_MESSAGE(deck.hasKeyword("FIPUNIX"),
+                        R"(Input deck must have "FIPUNIX" region set)");
+
+    const FieldPropsManager fpm {
+        deck, Phases{true, true, true}, grid, TableManager()
+    };
+
+    const auto& fipuni = fpm.get_int("FIPUNI");
+    auto expect = std::vector<int>(100, 3);
+    {
+        const auto l2 = std::vector<int>(100, 4);
+        expect.insert(expect.end(), l2.begin(), l2.end());
+    };
+
+    // FIPUNI <=> FIPUNIX
+    BOOST_CHECK_EQUAL_COLLECTIONS(fipuni.begin(), fipuni.end(),
+                                  expect.begin(), expect.end());
+}
+
 BOOST_AUTO_TEST_CASE(GET_FIPXYZ) {
     std::string deck_string = R"(
 GRID
