@@ -20,7 +20,7 @@
 #include <stdexcept>
 #include <memory>
 #include <stdlib.h>
-#include <iostream>
+
 #define BOOST_TEST_MODULE ParseContextTests
 #include <boost/test/unit_test.hpp>
 
@@ -72,26 +72,26 @@ BOOST_AUTO_TEST_CASE(TestUnkownKeyword) {
 
 
     parser.addKeyword<ParserKeywords::DIMENS>();
-    parseContext.update(ParseContext::PARSE_UNKNOWN_KEYWORD , InputError::THROW_EXCEPTION );
+    parseContext.update(ParseContext::PARSE_UNKNOWN_KEYWORD , InputErrorAction::THROW_EXCEPTION );
     BOOST_CHECK_THROW( parser.parseString( deck1 , parseContext , errors) , OpmInputError);
 
-    parseContext.update(ParseContext::PARSE_UNKNOWN_KEYWORD , InputError::IGNORE );
+    parseContext.update(ParseContext::PARSE_UNKNOWN_KEYWORD , InputErrorAction::IGNORE );
     BOOST_CHECK_NO_THROW( parser.parseString( deck1 , parseContext , errors) );
 
-    parseContext.update(ParseContext::PARSE_UNKNOWN_KEYWORD , InputError::THROW_EXCEPTION );
-    parseContext.update(ParseContext::PARSE_RANDOM_TEXT , InputError::IGNORE );
+    parseContext.update(ParseContext::PARSE_UNKNOWN_KEYWORD , InputErrorAction::THROW_EXCEPTION );
+    parseContext.update(ParseContext::PARSE_RANDOM_TEXT , InputErrorAction::IGNORE );
     BOOST_CHECK_THROW( parser.parseString( deck2 , parseContext , errors) , OpmInputError);
 
-    parseContext.update(ParseContext::PARSE_UNKNOWN_KEYWORD , InputError::IGNORE );
-    parseContext.update(ParseContext::PARSE_RANDOM_TEXT , InputError::IGNORE );
+    parseContext.update(ParseContext::PARSE_UNKNOWN_KEYWORD , InputErrorAction::IGNORE );
+    parseContext.update(ParseContext::PARSE_RANDOM_TEXT , InputErrorAction::IGNORE );
     BOOST_CHECK_NO_THROW( parser.parseString( deck2 , parseContext , errors) );
 
-    parseContext.update(ParseContext::PARSE_UNKNOWN_KEYWORD , InputError::IGNORE );
-    parseContext.update(ParseContext::PARSE_RANDOM_TEXT , InputError::THROW_EXCEPTION );
+    parseContext.update(ParseContext::PARSE_UNKNOWN_KEYWORD , InputErrorAction::IGNORE );
+    parseContext.update(ParseContext::PARSE_RANDOM_TEXT , InputErrorAction::THROW_EXCEPTION );
     BOOST_CHECK_THROW( parser.parseString( deck2 , parseContext , errors) , OpmInputError);
 
-    parseContext.update(ParseContext::PARSE_UNKNOWN_KEYWORD , InputError::IGNORE );
-    parseContext.update(ParseContext::PARSE_RANDOM_TEXT , InputError::IGNORE );
+    parseContext.update(ParseContext::PARSE_UNKNOWN_KEYWORD , InputErrorAction::IGNORE );
+    parseContext.update(ParseContext::PARSE_RANDOM_TEXT , InputErrorAction::IGNORE );
     BOOST_CHECK_NO_THROW( parser.parseString( deck2 , parseContext , errors) );
 }
 
@@ -110,7 +110,7 @@ BOOST_AUTO_TEST_CASE(TestUnkownKeywordII) {
 
 
     parser.addKeyword<ParserKeywords::DIMENS>();
-    parseContext.update(ParseContext::PARSE_UNKNOWN_KEYWORD , InputError::THROW_EXCEPTION );
+    parseContext.update(ParseContext::PARSE_UNKNOWN_KEYWORD , InputErrorAction::THROW_EXCEPTION );
     BOOST_CHECK_THROW( parser.parseString( deck1 , parseContext, errors ) , OpmInputError);
     parseContext.ignoreKeyword("RUNSPEC");
     BOOST_CHECK_NO_THROW( parser.parseString( deck1 , parseContext, errors ) );
@@ -137,11 +137,11 @@ BOOST_AUTO_TEST_CASE(Handle_extra_records) {
     parser.addKeyword<ParserKeywords::GRID>();
     BOOST_CHECK_THROW( parser.parseString( deck_string , parseContext, errors ) , OpmInputError );
 
-    parseContext.update(ParseContext::PARSE_EXTRA_RECORDS , InputError::IGNORE );
+    parseContext.update(ParseContext::PARSE_EXTRA_RECORDS , InputErrorAction::IGNORE );
     parser.parseString( deck_string , parseContext, errors );
     BOOST_CHECK( parser.hasKeyword( "GRID" ) );
 
-    parseContext.update(ParseContext::PARSE_EXTRA_RECORDS , InputError::THROW_EXCEPTION );
+    parseContext.update(ParseContext::PARSE_EXTRA_RECORDS , InputErrorAction::THROW_EXCEPTION );
     BOOST_CHECK_THROW( parser.parseString( deck_string , parseContext, errors ) , OpmInputError);
 }
 
@@ -168,7 +168,7 @@ BOOST_AUTO_TEST_CASE(Handle_extra_records_2) {
     parser.addKeyword<ParserKeywords::GRID>();
     parser.addKeyword<ParserKeywords::DIMENS>();
 
-    parseContext.update(ParseContext::PARSE_EXTRA_RECORDS , InputError::IGNORE );
+    parseContext.update(ParseContext::PARSE_EXTRA_RECORDS , InputErrorAction::IGNORE );
     BOOST_CHECK_THROW( parser.parseString( deck_string , parseContext, errors ), OpmInputError );
 }
 
@@ -218,8 +218,8 @@ BOOST_AUTO_TEST_CASE(TestUnkownKeyword_DATA) {
 
     parser.addKeyword<ParserKeywords::RUNSPEC>();
     parser.addKeyword<ParserKeywords::DIMENS>();
-    parseContext.update(ParseContext::PARSE_UNKNOWN_KEYWORD , InputError::IGNORE );
-    parseContext.update(ParseContext::PARSE_RANDOM_TEXT , InputError::THROW_EXCEPTION );
+    parseContext.update(ParseContext::PARSE_UNKNOWN_KEYWORD , InputErrorAction::IGNORE );
+    parseContext.update(ParseContext::PARSE_RANDOM_TEXT , InputErrorAction::THROW_EXCEPTION );
     {
         Deck deck = parser.parseString( deck_string1 , parseContext, errors );
         BOOST_CHECK( deck.hasKeyword( "RUNSPEC") );
@@ -246,18 +246,18 @@ BOOST_AUTO_TEST_CASE(TEST_UNKNOWN_OPERATE) {
     ParseContext parseContext;
     Parser parser(false);
 
-    parseContext.update(ParseContext::PARSE_UNKNOWN_KEYWORD , InputError::THROW_EXCEPTION );
+    parseContext.update(ParseContext::PARSE_UNKNOWN_KEYWORD , InputErrorAction::THROW_EXCEPTION );
     BOOST_CHECK_THROW( parser.parseString( deck , parseContext, errors ) , OpmInputError);
 
-    parseContext.update(ParseContext::PARSE_RANDOM_SLASH , InputError::IGNORE );
-    parseContext.update(ParseContext::PARSE_UNKNOWN_KEYWORD , InputError::IGNORE );
+    parseContext.update(ParseContext::PARSE_RANDOM_SLASH , InputErrorAction::IGNORE );
+    parseContext.update(ParseContext::PARSE_UNKNOWN_KEYWORD , InputErrorAction::IGNORE );
     parser.parseString( deck , parseContext, errors );
     BOOST_CHECK_NO_THROW( parser.parseString( deck , parseContext, errors ) );
 
     parser.addKeyword<ParserKeywords::OPERATE>();
     parser.parseString( deck , parseContext, errors );
-    parseContext.update(ParseContext::PARSE_RANDOM_SLASH , InputError::THROW_EXCEPTION );
-    parseContext.update(ParseContext::PARSE_UNKNOWN_KEYWORD , InputError::THROW_EXCEPTION );
+    parseContext.update(ParseContext::PARSE_RANDOM_SLASH , InputErrorAction::THROW_EXCEPTION );
+    parseContext.update(ParseContext::PARSE_UNKNOWN_KEYWORD , InputErrorAction::THROW_EXCEPTION );
     BOOST_CHECK_NO_THROW( parser.parseString( deck , parseContext, errors ) );
 }
 
@@ -279,10 +279,10 @@ BOOST_AUTO_TEST_CASE( CheckMissingSizeKeyword) {
     parser.addKeyword<ParserKeywords::EQLDIMS>();
     parser.addKeyword<ParserKeywords::SOLUTION>();
 
-    parseContext.update( ParseContext::PARSE_MISSING_DIMS_KEYWORD , InputError::THROW_EXCEPTION );
+    parseContext.update( ParseContext::PARSE_MISSING_DIMS_KEYWORD , InputErrorAction::THROW_EXCEPTION );
     BOOST_CHECK_THROW( parser.parseString( deck , parseContext, errors ) , OpmInputError);
 
-    parseContext.update( ParseContext::PARSE_MISSING_DIMS_KEYWORD , InputError::IGNORE );
+    parseContext.update( ParseContext::PARSE_MISSING_DIMS_KEYWORD , InputErrorAction::IGNORE );
     BOOST_CHECK_NO_THROW( parser.parseString( deck , parseContext, errors ) );
 }
 
@@ -310,14 +310,14 @@ BOOST_AUTO_TEST_CASE(TestRandomSlash) {
     parser.addKeyword<ParserKeywords::TSTEP>();
     parser.addKeyword<ParserKeywords::SCHEDULE>();
 
-    parseContext.update(ParseContext::PARSE_RANDOM_SLASH , InputError::THROW_EXCEPTION);
-    parseContext.update(ParseContext::PARSE_RANDOM_TEXT , InputError::IGNORE);
+    parseContext.update(ParseContext::PARSE_RANDOM_SLASH , InputErrorAction::THROW_EXCEPTION);
+    parseContext.update(ParseContext::PARSE_RANDOM_TEXT , InputErrorAction::IGNORE);
     BOOST_CHECK_THROW( parser.parseString( deck1 , parseContext, errors ) , OpmInputError);
     BOOST_CHECK_THROW( parser.parseString( deck2 , parseContext, errors ) , OpmInputError);
 
 
-    parseContext.update(ParseContext::PARSE_RANDOM_SLASH , InputError::IGNORE);
-    parseContext.update(ParseContext::PARSE_RANDOM_TEXT , InputError::THROW_EXCEPTION);
+    parseContext.update(ParseContext::PARSE_RANDOM_SLASH , InputErrorAction::IGNORE);
+    parseContext.update(ParseContext::PARSE_RANDOM_TEXT , InputErrorAction::THROW_EXCEPTION);
     BOOST_CHECK_NO_THROW( parser.parseString( deck1 , parseContext, errors ) );
     BOOST_CHECK_NO_THROW( parser.parseString( deck2 , parseContext, errors ) );
 }
@@ -328,8 +328,8 @@ BOOST_AUTO_TEST_CASE(TestRandomSlash) {
 
 BOOST_AUTO_TEST_CASE(TestInvalidKey) {
     ParseContext parseContext;
-    BOOST_CHECK_THROW( parseContext.addKey("KEY*", InputError::THROW_EXCEPTION) , std::invalid_argument );
-    BOOST_CHECK_THROW( parseContext.addKey("KEY:", InputError::THROW_EXCEPTION) , std::invalid_argument );
+    BOOST_CHECK_THROW( parseContext.addKey("KEY*", InputErrorAction::THROW_EXCEPTION) , std::invalid_argument );
+    BOOST_CHECK_THROW( parseContext.addKey("KEY:", InputErrorAction::THROW_EXCEPTION) , std::invalid_argument );
 }
 
 
@@ -337,47 +337,47 @@ BOOST_AUTO_TEST_CASE(TestNew) {
     ParseContext parseContext;
 
     BOOST_CHECK_EQUAL( false , parseContext.hasKey("NO"));
-    parseContext.addKey("NEW_KEY", InputError::THROW_EXCEPTION);
+    parseContext.addKey("NEW_KEY", InputErrorAction::THROW_EXCEPTION);
     BOOST_CHECK_EQUAL( true , parseContext.hasKey("NEW_KEY"));
     BOOST_CHECK_THROW( parseContext.get("NO") , std::invalid_argument);
-    BOOST_CHECK_EQUAL( parseContext.get("NEW_KEY") , InputError::THROW_EXCEPTION );
-    parseContext.addKey("KEY2", InputError::THROW_EXCEPTION);
-    BOOST_CHECK_EQUAL( parseContext.get("NEW_KEY") , InputError::THROW_EXCEPTION );
+    BOOST_CHECK_EQUAL( parseContext.get("NEW_KEY") , InputErrorAction::THROW_EXCEPTION );
+    parseContext.addKey("KEY2", InputErrorAction::THROW_EXCEPTION);
+    BOOST_CHECK_EQUAL( parseContext.get("NEW_KEY") , InputErrorAction::THROW_EXCEPTION );
 
-    BOOST_CHECK_THROW( parseContext.updateKey("NO" , InputError::IGNORE) , std::invalid_argument);
+    BOOST_CHECK_THROW( parseContext.updateKey("NO" , InputErrorAction::IGNORE) , std::invalid_argument);
 
-    parseContext.updateKey("NEW_KEY" , InputError::WARN);
-    BOOST_CHECK_EQUAL( parseContext.get("NEW_KEY") , InputError::WARN );
+    parseContext.updateKey("NEW_KEY" , InputErrorAction::WARN);
+    BOOST_CHECK_EQUAL( parseContext.get("NEW_KEY") , InputErrorAction::WARN );
 
-    BOOST_CHECK_NO_THROW( parseContext.update("KEY2:NEW_KEY" , InputError::IGNORE));
-    BOOST_CHECK_NO_THROW( parseContext.update("UnknownKey" , InputError::IGNORE));
-    BOOST_CHECK_EQUAL( parseContext.get("NEW_KEY") , InputError::IGNORE );
-    BOOST_CHECK_EQUAL( parseContext.get("KEY2") , InputError::IGNORE );
+    BOOST_CHECK_NO_THROW( parseContext.update("KEY2:NEW_KEY" , InputErrorAction::IGNORE));
+    BOOST_CHECK_NO_THROW( parseContext.update("UnknownKey" , InputErrorAction::IGNORE));
+    BOOST_CHECK_EQUAL( parseContext.get("NEW_KEY") , InputErrorAction::IGNORE );
+    BOOST_CHECK_EQUAL( parseContext.get("KEY2") , InputErrorAction::IGNORE );
 
-    parseContext.addKey("SECRET_KEY", InputError::THROW_EXCEPTION);
-    parseContext.addKey("NEW_KEY2", InputError::THROW_EXCEPTION);
-    parseContext.addKey("NEW_KEY3", InputError::THROW_EXCEPTION);
-    parseContext.update("NEW_KEY*" , InputError::WARN);
-    BOOST_CHECK_EQUAL( parseContext.get("NEW_KEY") , InputError::WARN );
-    BOOST_CHECK_EQUAL( parseContext.get("NEW_KEY2") , InputError::WARN );
-    BOOST_CHECK_EQUAL( parseContext.get("NEW_KEY3") , InputError::WARN );
+    parseContext.addKey("SECRET_KEY", InputErrorAction::THROW_EXCEPTION);
+    parseContext.addKey("NEW_KEY2", InputErrorAction::THROW_EXCEPTION);
+    parseContext.addKey("NEW_KEY3", InputErrorAction::THROW_EXCEPTION);
+    parseContext.update("NEW_KEY*" , InputErrorAction::WARN);
+    BOOST_CHECK_EQUAL( parseContext.get("NEW_KEY") , InputErrorAction::WARN );
+    BOOST_CHECK_EQUAL( parseContext.get("NEW_KEY2") , InputErrorAction::WARN );
+    BOOST_CHECK_EQUAL( parseContext.get("NEW_KEY3") , InputErrorAction::WARN );
 
-    parseContext.update( InputError::IGNORE );
-    BOOST_CHECK_EQUAL( parseContext.get("NEW_KEY3")   , InputError::IGNORE );
-    BOOST_CHECK_EQUAL( parseContext.get("SECRET_KEY") , InputError::IGNORE );
+    parseContext.update( InputErrorAction::IGNORE );
+    BOOST_CHECK_EQUAL( parseContext.get("NEW_KEY3")   , InputErrorAction::IGNORE );
+    BOOST_CHECK_EQUAL( parseContext.get("SECRET_KEY") , InputErrorAction::IGNORE );
 
 
 }
 
 
 BOOST_AUTO_TEST_CASE( test_constructor_with_values) {
-    ParseContext parseContext( {{ParseContext::PARSE_RANDOM_SLASH , InputError::IGNORE},
-                {"UNSUPPORTED_*" , InputError::WARN},
-                    {"UNKNWON-IGNORED" , InputError::WARN}});
+    ParseContext parseContext( {{ParseContext::PARSE_RANDOM_SLASH , InputErrorAction::IGNORE},
+                {"UNSUPPORTED_*" , InputErrorAction::WARN},
+                    {"UNKNWON-IGNORED" , InputErrorAction::WARN}});
 
-    BOOST_CHECK_EQUAL( parseContext.get(ParseContext::PARSE_RANDOM_SLASH) , InputError::IGNORE );
-    BOOST_CHECK_EQUAL( parseContext.get(ParseContext::PARSE_RANDOM_TEXT) , InputError::THROW_EXCEPTION );
-    BOOST_CHECK_EQUAL( parseContext.get(ParseContext::UNSUPPORTED_INITIAL_THPRES) , InputError::WARN );
+    BOOST_CHECK_EQUAL( parseContext.get(ParseContext::PARSE_RANDOM_SLASH) , InputErrorAction::IGNORE );
+    BOOST_CHECK_EQUAL( parseContext.get(ParseContext::PARSE_RANDOM_TEXT) , InputErrorAction::THROW_EXCEPTION );
+    BOOST_CHECK_EQUAL( parseContext.get(ParseContext::UNSUPPORTED_INITIAL_THPRES) , InputErrorAction::WARN );
 }
 
 
@@ -394,10 +394,10 @@ BOOST_AUTO_TEST_CASE( test_too_much_data ) {
     ErrorGuard errors;
 
 
-    parseContext.update(ParseContext::PARSE_EXTRA_DATA , InputError::THROW_EXCEPTION );
+    parseContext.update(ParseContext::PARSE_EXTRA_DATA , InputErrorAction::THROW_EXCEPTION );
     BOOST_CHECK_THROW( parser.parseString( deckString , parseContext, errors ) , OpmInputError);
 
-    parseContext.update(ParseContext::PARSE_EXTRA_DATA , InputError::IGNORE );
+    parseContext.update(ParseContext::PARSE_EXTRA_DATA , InputErrorAction::IGNORE );
     auto deck = parser.parseString( deckString , parseContext, errors );
 }
 
@@ -405,8 +405,8 @@ BOOST_AUTO_TEST_CASE( test_too_much_data ) {
 BOOST_AUTO_TEST_CASE(test_1arg_constructor) {
     setenv("OPM_ERRORS_IGNORE", "PARSE_RANDOM_SLASH", 1);
     {
-        ParseContext ctx(InputError::WARN);
-        BOOST_CHECK_EQUAL(ctx.get(ParseContext::PARSE_RANDOM_SLASH), InputError::IGNORE);
+        ParseContext ctx(InputErrorAction::WARN);
+        BOOST_CHECK_EQUAL(ctx.get(ParseContext::PARSE_RANDOM_SLASH), InputErrorAction::IGNORE);
     }
 }
 
@@ -423,10 +423,10 @@ AQUCT
 
     BOOST_CHECK_THROW(parser.parseString(deckString, parseContext, errors), OpmInputError);
 
-    parseContext.update(ParseContext::PARSE_INVALID_KEYWORD_COMBINATION , InputError::IGNORE );
+    parseContext.update(ParseContext::PARSE_INVALID_KEYWORD_COMBINATION , InputErrorAction::IGNORE );
     BOOST_CHECK_NO_THROW(parser.parseString(deckString, parseContext, errors));
 
-    parseContext.update(ParseContext::PARSE_INVALID_KEYWORD_COMBINATION , InputError::THROW_EXCEPTION );
+    parseContext.update(ParseContext::PARSE_INVALID_KEYWORD_COMBINATION , InputErrorAction::THROW_EXCEPTION );
     BOOST_CHECK_THROW(parser.parseString(deckString, parseContext, errors), OpmInputError);
 }
 
@@ -448,10 +448,10 @@ RTEMPVD
 
     BOOST_CHECK_THROW(parser.parseString(deckString, parseContext, errors), OpmInputError);
 
-    parseContext.update(ParseContext::PARSE_INVALID_KEYWORD_COMBINATION , InputError::IGNORE );
+    parseContext.update(ParseContext::PARSE_INVALID_KEYWORD_COMBINATION , InputErrorAction::IGNORE );
     BOOST_CHECK_NO_THROW(parser.parseString(deckString, parseContext, errors));
 
-    parseContext.update(ParseContext::PARSE_INVALID_KEYWORD_COMBINATION , InputError::THROW_EXCEPTION );
+    parseContext.update(ParseContext::PARSE_INVALID_KEYWORD_COMBINATION , InputErrorAction::THROW_EXCEPTION );
     BOOST_CHECK_THROW(parser.parseString(deckString, parseContext, errors), OpmInputError);
 }
 
@@ -491,7 +491,7 @@ BOOST_AUTO_TEST_CASE( test_invalid_wtemplate_config ) {
     Parser parser;
     ErrorGuard errors;
 
-    parseContext.update(ParseContext::SCHEDULE_INVALID_NAME , InputError::THROW_EXCEPTION );
+    parseContext.update(ParseContext::SCHEDULE_INVALID_NAME , InputErrorAction::THROW_EXCEPTION );
 
     std::vector < std::string > testSamples;
     std::string testSample;
@@ -755,11 +755,11 @@ RPTRUNSPEC
     ParseContext context;
     ErrorGuard error;
 
-    context.update(ParseContext::PARSE_LONG_KEYWORD, InputError::IGNORE);
+    context.update(ParseContext::PARSE_LONG_KEYWORD, InputErrorAction::IGNORE);
     auto deck = parser.parseString(deck_string, context, error);
     BOOST_CHECK( deck.hasKeyword("RPTRUNSP") );
 
-    context.update(ParseContext::PARSE_LONG_KEYWORD, InputError::THROW_EXCEPTION);
+    context.update(ParseContext::PARSE_LONG_KEYWORD, InputErrorAction::THROW_EXCEPTION);
     BOOST_CHECK_THROW( parser.parseString(deck_string, context, error), OpmInputError);
 }
 
