@@ -17,19 +17,18 @@
   along with OPM.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-
-
 #define BOOST_TEST_MODULE SimulationConfigTests
 
 #include <boost/test/unit_test.hpp>
 
+#include <opm/input/eclipse/EclipseState/SimulationConfig/RockConfig.hpp>
+
 #include <opm/common/utility/OpmInputError.hpp>
 
 #include <opm/input/eclipse/EclipseState/Runspec.hpp>
 #include <opm/input/eclipse/EclipseState/Grid/FieldPropsManager.hpp>
 #include <opm/input/eclipse/EclipseState/Grid/EclipseGrid.hpp>
 #include <opm/input/eclipse/EclipseState/SimulationConfig/SimulationConfig.hpp>
-#include <opm/input/eclipse/EclipseState/SimulationConfig/RockConfig.hpp>
 #include <opm/input/eclipse/EclipseState/Tables/TableManager.hpp>
 
 #include <opm/input/eclipse/Parser/Parser.hpp>
@@ -38,6 +37,9 @@
 #include <opm/input/eclipse/Deck/Deck.hpp>
 #include <opm/input/eclipse/Deck/DeckSection.hpp>
 
+#include <cstddef>
+#include <string>
+
 using namespace Opm;
 
 const std::string& inputStr = "RUNSPEC\n"
@@ -117,6 +119,30 @@ GRID
 REGIONS
 )";
 
+const std::string& inputStr_bc = R"(
+RUNSPEC
+DIMENS
+ 10 3 4 /
+GRID
+SOLUTION
+BC
+1 1 1 1 1 10 X- FREE GAS/
+/
+REGIONS
+)";
+
+const std::string& inputStr_bccon = R"(
+RUNSPEC
+DIMENS
+ 10 3 4 /
+GRID
+BCCON
+1 1 1 1 1 1 1 X- /
+2 10 10 4* X /
+/
+REGIONS
+)";
+
 namespace {
     std::string simDeckStringTEMP()
     {
@@ -148,6 +174,23 @@ static Deck createDeck(const std::string& input) {
     return parser.parseString(input);
 }
 
+BOOST_AUTO_TEST_CASE(SimulationConfigBCCON) {
+    auto deck = createDeck(inputStr_bccon);
+    BCConfig bcconfig(deck);
+    std::vector<int> i1s = { 1, 10};
+    std::vector<int> k2s = { 1, 4};
+    std::vector<Opm::FaceDir::DirEnum> dirs = { Opm::FaceDir::XMinus , Opm::FaceDir::XPlus};
+    for (const auto bc : bcconfig) {
+        BOOST_CHECK(bc.i1 == (i1s[bc.index - 1 ]) - 1);
+        BOOST_CHECK(bc.k2 == (k2s[bc.index - 1 ]) - 1);
+        BOOST_CHECK(bc.dir == dirs[bc.index -1 ]);
+    }
+
+}
+BOOST_AUTO_TEST_CASE(SimulationConfigBC) {
+    BOOST_CHECK_THROW( BCConfig(createDeck(inputStr_bc)), OpmInputError );
+}
+
 
 BOOST_AUTO_TEST_CASE(SimulationConfigGetThresholdPressureTableTest) {
     auto deck = createDeck(inputStr);
@@ -280,7 +323,8 @@ BOOST_AUTO_TEST_CASE(SimulationConfig_TEMP_THERMAL)
 }
 
 
-BOOST_AUTO_TEST_CASE(TESTRockConfig) {
+BOOST_AUTO_TEST_CASE(TESTRockConfig_Standard)
+{
     const std::string deck_string = R"(
 RUNSPEC
 
@@ -310,3 +354,45 @@ ROCKOPTS
     const auto& comp = rc.comp();
     BOOST_CHECK_EQUAL(comp.size(), 3U);
 }
+
+BOOST_AUTO_TEST_CASE(TESTRockConfig_Default)
+{
+    const auto deck = Parser{}.parseString(R"(
+RUNSPEC
+
+TABDIMS
+  3  / -- NTSFUN = 3
+
+PROPS
+
+ROCKOPTS
+  1* 1* SATNUM /
+
+ROCK
+123.4 0.40E-05 /
+/
+271.8 1.61e-05 /
+
+)");
+
+    const auto grid = EclipseGrid { 10, 10, 10 };
+    const auto fp = FieldPropsManager {
+        deck, Phases{true, true, true}, grid, TableManager()
+    };
+
+    const auto rc = RockConfig { deck, fp };
+
+    BOOST_CHECK_EQUAL(rc.rocknum_property(), "SATNUM");
+
+    const auto& comp = rc.comp();
+    BOOST_REQUIRE_EQUAL(comp.size(), std::size_t{3});
+
+    BOOST_CHECK_CLOSE(comp[0].pref, 123.4*1.0e5, 1.0e-8);
+    BOOST_CHECK_CLOSE(comp[0].compressibility, 0.4e-5/1.0e5, 1.0e-8);
+
+    BOOST_CHECK_CLOSE(comp[1].pref, 123.4*1.0e5, 1.0e-8);
+    BOOST_CHECK_CLOSE(comp[1].compressibility, 0.4e-5/1.0e5, 1.0e-8);
+
+    BOOST_CHECK_CLOSE(comp[2].pref, 271.8*1.0e5, 1.0e-8);
+    BOOST_CHECK_CLOSE(comp[2].compressibility, 1.61e-5/1.0e5, 1.0e-8);
+}
