@@ -18,41 +18,50 @@
 */
 
 #define BOOST_TEST_MODULE Aggregate_MSW_Data
+
 #include <opm/output/eclipse/AggregateMSWData.hpp>
-#include <opm/output/eclipse/WriteRestartHelpers.hpp>
 
 #include <boost/test/unit_test.hpp>
 
-#include <opm/output/eclipse/AggregateWellData.hpp>
-#include <opm/input/eclipse/Python/Python.hpp>
-
 #include <opm/output/eclipse/VectorItems/intehead.hpp>
-#include <opm/output/eclipse/VectorItems/well.hpp>
 #include <opm/output/eclipse/VectorItems/msw.hpp>
+#include <opm/output/eclipse/VectorItems/well.hpp>
+
+#include <opm/output/eclipse/WriteRestartHelpers.hpp>
 
 #include <opm/output/data/Wells.hpp>
 
 #include <opm/io/eclipse/rst/segment.hpp>
-#include <opm/input/eclipse/Deck/Deck.hpp>
-#include <opm/input/eclipse/Parser/Parser.hpp>
+
 #include <opm/input/eclipse/EclipseState/EclipseState.hpp>
+#include <opm/input/eclipse/EclipseState/Grid/EclipseGrid.hpp>
+
 #include <opm/input/eclipse/Schedule/Schedule.hpp>
 #include <opm/input/eclipse/Schedule/SummaryState.hpp>
+
 #include <opm/common/utility/TimeService.hpp>
 
-#include <exception>
+#include <opm/input/eclipse/Python/Python.hpp>
+
+#include <opm/input/eclipse/Deck/Deck.hpp>
+
+#include <opm/input/eclipse/Parser/Parser.hpp>
+
+#include <cmath>
+#include <cstddef>
+#include <memory>
 #include <stdexcept>
+#include <string>
 #include <utility>
 #include <vector>
-#include <iostream>
-#include <cstddef>
 
 namespace {
 
 namespace VI = ::Opm::RestartIO::Helpers::VectorItems;
 
-Opm::Deck first_sim(std::string fname) {
-    return Opm::Parser {} .parseFile(fname);
+Opm::Deck first_sim(const std::string& fname)
+{
+    return Opm::Parser {}.parseFile(fname);
 }
 
 Opm::SummaryState sim_state()
@@ -92,6 +101,7 @@ Opm::SummaryState sim_state()
     state.update_well_var("WINJ", "WBHP",  234.);
     return state;
 }
+
 Opm::data::Wells wr()
 {
     using o = ::Opm::data::Rates::opt;
@@ -142,21 +152,110 @@ Opm::data::Wells wr()
     }
     return xw;
 }
+
+//------------------------------------------------------------------+
+// Models a multi-lateral well with the following segment structure |
+//------------------------------------------------------------------+
+//                                                                  |
+//                     12   13    14     15     16                  |
+//                   o----o----o-----o------o-----o (2)             |
+//              11  /              20 \   21 \                      |
+//                 /                   o      o (6)                 |
+//                /                     \                           |
+//               /                    22 \  23   24                 |
+//   1   2   3  /  4   5   6              o----o----o (5)           |
+//  ---o---o---o-----o---o---o (1)                                  |
+//                        \                                         |
+//                       7 \  8   9    10                           |
+//                          o---o---o-----o (3)                     |
+//                                         \                        |
+//                                       17 \   18   19             |
+//                                           o----o----o (4)        |
+//------------------------------------------------------------------+
+//  Branch (1):  1,  2,  3,  4,  5,  6                              |
+//  Branch (2): 11, 12, 13, 14, 15, 16                              |
+//  Branch (3):  7,  8,  9, 10                                      |
+//  Branch (4): 17, 18, 19                                          |
+//  Branch (5): 20, 22, 23, 24                                      |
+//  Branch (6): 21                                                  |
+//------------------------------------------------------------------+
+Opm::Deck multilaterals()
+{
+    return Opm::Parser{}.parseString(R"(RUNSPEC
+START
+29 'SEP' 2023 /
+DIMENS
+10 10 3 /
+OIL
+GAS
+WATER
+DISGAS
+VAPOIL
+GRID
+DXV
+10*100.0 /
+DYV
+10*100.0 /
+DZV
+3*5.0 /
+PERMX
+300*100.0 /
+COPY
+PERMX PERMY /
+PERMX PERMZ /
+/
+MULTIPLY
+PERMZ 0.1 /
+/
+PORO
+300*0.3 /
+DEPTHZ
+121*2000.0 /
+SCHEDULE
+WELSPECS
+ 'MLP' 'G' 10 10 2002.5 'OIL' /
+/
+COMPDAT
+ 'MLP' 10 10 3 3 'OPEN' 1* 123.4 /
+/
+WELSEGS
+ 'MLP' 2002.5 0.0 1* 'INC' 'H--' /
+--
+  2  6 1  1 0.1 0.1 0.2 0.01 /
+  7 10 3  5 0.1 0.1 0.2 0.01 /
+ 11 16 2  3 0.1 0.1 0.2 0.01 /
+ 17 19 4 10 0.1 0.1 0.2 0.01 /
+ 20 20 5 14 0.1 0.1 0.2 0.01 /
+ 21 21 6 15 0.1 0.1 0.2 0.01 /
+ 22 24 5 20 0.1 0.1 0.2 0.01 /
+/
+COMPSEGS
+ 'MLP' /
+--
+ 10 10 3 5 0.0 1.0 'Z' /
+/
+WCONPROD
+ 'MLP' 'OPEN' 'ORAT' 321.0 4* 10.0 /
+/
+TSTEP
+5*30 /
+END
+)");
 }
 
+} // Anonymous namespace
+
 struct SimulationCase
 {
     explicit SimulationCase(const Opm::Deck& deck)
-        : es   ( deck )
-        , grid ( deck )
-        , python( std::make_shared<Opm::Python>() )
-        , sched( deck, es, python )
+        : es   (deck)
+        , grid (deck)
+        , sched(deck, es, std::make_shared<Opm::Python>())
     {}
 
     // Order requirement: 'es' must be declared/initialised before 'sched'.
     Opm::EclipseState es;
     Opm::EclipseGrid  grid;
-    std::shared_ptr<Opm::Python>  python;
     Opm::Schedule     sched;
 };
 
@@ -164,7 +263,6 @@ struct SimulationCase
 
 BOOST_AUTO_TEST_SUITE(Aggregate_MSW)
 
-
 // test dimensions of multisegment data
 BOOST_AUTO_TEST_CASE (Constructor)
 {
@@ -191,46 +289,36 @@ BOOST_AUTO_TEST_CASE (Constructor)
     const auto nrsegz = VI::intehead::NRSEGZ;
     const auto nlbrmx = VI::intehead::NLBRMX;
     const auto nilbrz = VI::intehead::NILBRZ;
+
     BOOST_CHECK_EQUAL(static_cast<int>(amswd.getISeg().size()), ih[nswlmx] * ih[nsegmx] * ih[nisegz]);
     BOOST_CHECK_EQUAL(static_cast<int>(amswd.getRSeg().size()), ih[nswlmx] * ih[nsegmx] * ih[nrsegz]);
     BOOST_CHECK_EQUAL(static_cast<int>(amswd.getILBs().size()), ih[nswlmx] * ih[nlbrmx]);
     BOOST_CHECK_EQUAL(static_cast<int>(amswd.getILBr().size()), ih[nswlmx] * ih[nlbrmx] * ih[nilbrz]);
 }
 
-
 BOOST_AUTO_TEST_CASE (Declared_MSW_Data)
 {
-
     const auto simCase = SimulationCase {first_sim("TEST_AGGREGATE_MSW.DATA")};
 
-    Opm::EclipseState es = simCase.es;
-    Opm::Runspec rspec   = es.runspec();
-    Opm::SummaryState smry = sim_state();
-    Opm::Schedule     sched = simCase.sched;
-    Opm::EclipseGrid  grid = simCase.grid;
-    const auto& units    = es.getUnits();
-
+    const auto& es    = simCase.es;
+    const auto& grid  = simCase.grid;
+    const auto& sched = simCase.sched;
+    const auto& units = es.getUnits();
+    const auto  smry  = sim_state();
 
     // Report Step 1: 2008-10-10 --> 2011-01-20
     const auto rptStep = std::size_t {1};
 
-    double secs_elapsed = 3.1536E07;
+    const double secs_elapsed = 3.1536E07;
     const auto ih = Opm::RestartIO::Helpers::
-                    createInteHead(es, grid, sched, secs_elapsed,
-                                   rptStep, rptStep+1, rptStep);
-
-    //BOOST_CHECK_EQUAL(ih.nwells, MockIH::Sz{2});
+        createInteHead(es, grid, sched, secs_elapsed,
+                       rptStep, rptStep + 1, rptStep);
 
     const Opm::data::Wells wrc = wr();
+
     auto amswd = Opm::RestartIO::Helpers::AggregateMSWData {ih};
-    amswd.captureDeclaredMSWData(simCase.sched,
-                                 rptStep,
-                                 units,
-                                 ih,
-                                 grid,
-                                 smry,
-                                 wrc
-                                );
+    amswd.captureDeclaredMSWData(sched, rptStep, units,
+                                 ih, grid, smry, wrc);
 
     // ISEG (PROD)
     {
@@ -409,39 +497,214 @@ BOOST_AUTO_TEST_CASE (Declared_MSW_Data)
     }
 }
 
+// The segments and branches must appear in the following order in the
+// ILBS/ILBR output arrays.
+//
+//      1,  2,  3,  4,  5,  6 -- Branch (1)
+//     11, 12, 13, 14, 15, 16 -- Branch (2)
+//      7,  8,  9, 10         -- Branch (3)
+//     20, 22, 23, 24         -- Branch (5)
+//     21,                    -- Branch (6)
+//     17, 18, 19             -- Branch (4)
+//
+BOOST_AUTO_TEST_CASE(Multilateral_Branches)
+{
+    const auto cse = SimulationCase { multilaterals() };
 
-BOOST_AUTO_TEST_CASE(MSW_AICD) {
+    const auto& es    = cse.es;
+    const auto& grid  = cse.grid;
+    const auto& sched = cse.sched;
+    const auto& units = es.getUnits();
+    const auto  smry  = Opm::SummaryState { Opm::TimeService::now() };
+
+    // Report Step 1: 2023-09-29 --> 2023-10-23
+    const auto rptStep = std::size_t {1};
+
+    const double secs_elapsed = 30 * 86'400.0;
+    const auto ih = Opm::RestartIO::Helpers::
+        createInteHead(es, grid, sched, secs_elapsed,
+                       rptStep, rptStep + 1, rptStep);
+
+    const auto xw = Opm::data::Wells {};
+
+    auto amswd = Opm::RestartIO::Helpers::AggregateMSWData {ih};
+    amswd.captureDeclaredMSWData(sched, rptStep, units,
+                                 ih, grid, smry, xw);
+
+    // ILBS--First segment on each branch other than branch 1.  Ordered by
+    // discovery.
+    {
+        const auto& ilbs = amswd.getILBs();
+
+        // No WSEGDIMS => size = maximum branch number
+        BOOST_CHECK_EQUAL(ilbs.size(), std::vector<int>::size_type{6});
+
+        const auto expect = std::vector {
+            11, 7, 20, 21, 17, 0,
+        };
+
+        BOOST_CHECK_EQUAL_COLLECTIONS(ilbs  .begin(), ilbs  .end(),
+                                      expect.begin(), expect.end());
+    }
+
+    auto ilbrOffset = [&ih](const int branch)
+    {
+        return ih[VI::intehead::NILBRZ] * (branch - 1);
+    };
+
+    // ILBR, branch 1
+    {
+        const auto* ilbr = &amswd.getILBr()[ilbrOffset(1)];
+
+        BOOST_CHECK_EQUAL(ilbr[VI::ILbr::OutletSegment],          0);
+        BOOST_CHECK_EQUAL(ilbr[VI::ILbr::NumBranchSegments],      6);
+        BOOST_CHECK_EQUAL(ilbr[VI::ILbr::FirstSegment],           1);
+        BOOST_CHECK_EQUAL(ilbr[VI::ILbr::LastSegment],            6);
+        BOOST_CHECK_EQUAL(ilbr[VI::ILbr::KickOffDiscoveryOffset], 0);
+    }
+
+    // ILBR, branch 2
+    {
+        const auto* ilbr = &amswd.getILBr()[ilbrOffset(2)];
+
+        BOOST_CHECK_EQUAL(ilbr[VI::ILbr::OutletSegment],           3);
+        BOOST_CHECK_EQUAL(ilbr[VI::ILbr::NumBranchSegments],       6);
+        BOOST_CHECK_EQUAL(ilbr[VI::ILbr::FirstSegment],           11);
+        BOOST_CHECK_EQUAL(ilbr[VI::ILbr::LastSegment],            16);
+        BOOST_CHECK_EQUAL(ilbr[VI::ILbr::KickOffDiscoveryOffset],  1);
+    }
+
+    // ILBR, branch 3
+    {
+        const auto* ilbr = &amswd.getILBr()[ilbrOffset(3)];
+
+        BOOST_CHECK_EQUAL(ilbr[VI::ILbr::OutletSegment],           5);
+        BOOST_CHECK_EQUAL(ilbr[VI::ILbr::NumBranchSegments],       4);
+        BOOST_CHECK_EQUAL(ilbr[VI::ILbr::FirstSegment],            7);
+        BOOST_CHECK_EQUAL(ilbr[VI::ILbr::LastSegment],            10);
+        BOOST_CHECK_EQUAL(ilbr[VI::ILbr::KickOffDiscoveryOffset],  2);
+    }
+
+    // ILBR, branch 4
+    {
+        const auto* ilbr = &amswd.getILBr()[ilbrOffset(4)];
+
+        BOOST_CHECK_EQUAL(ilbr[VI::ILbr::OutletSegment],          10);
+        BOOST_CHECK_EQUAL(ilbr[VI::ILbr::NumBranchSegments],       3);
+        BOOST_CHECK_EQUAL(ilbr[VI::ILbr::FirstSegment],           17);
+        BOOST_CHECK_EQUAL(ilbr[VI::ILbr::LastSegment],            19);
+        BOOST_CHECK_EQUAL(ilbr[VI::ILbr::KickOffDiscoveryOffset],  5);
+    }
+
+    // ILBR, branch 5
+    {
+        const auto* ilbr = &amswd.getILBr()[ilbrOffset(5)];
+
+        BOOST_CHECK_EQUAL(ilbr[VI::ILbr::OutletSegment],          14);
+        BOOST_CHECK_EQUAL(ilbr[VI::ILbr::NumBranchSegments],       4);
+        BOOST_CHECK_EQUAL(ilbr[VI::ILbr::FirstSegment],           20);
+        BOOST_CHECK_EQUAL(ilbr[VI::ILbr::LastSegment],            24);
+        BOOST_CHECK_EQUAL(ilbr[VI::ILbr::KickOffDiscoveryOffset],  3);
+    }
+
+    // ILBR, branch 6
+    {
+        const auto* ilbr = &amswd.getILBr()[ilbrOffset(6)];
+
+        BOOST_CHECK_EQUAL(ilbr[VI::ILbr::OutletSegment],          15);
+        BOOST_CHECK_EQUAL(ilbr[VI::ILbr::NumBranchSegments],       1);
+        BOOST_CHECK_EQUAL(ilbr[VI::ILbr::FirstSegment],           21);
+        BOOST_CHECK_EQUAL(ilbr[VI::ILbr::LastSegment],            21);
+        BOOST_CHECK_EQUAL(ilbr[VI::ILbr::KickOffDiscoveryOffset],  4);
+    }
+}
+
+// The segments must appear in the following depth first search toe-to-heel
+// order in ISEG[0].  We furthermore, go along kick-off branches before
+// searching the main branch.  Note that this order is *different* from
+// ILBS/ILBR.
+//
+//     24, 23, 22, 20,         -- Branch (5)
+//     21,                     -- Branch (6)
+//     16, 15, 14, 13, 12, 11, -- Branch (2)
+//     19, 18, 17,             -- Branch (4)
+//     10,  9,  8,  7,         -- Branch (3)
+//      6,  5,  4,  3,  2,  1, -- Branch (1)
+//
+BOOST_AUTO_TEST_CASE(Multilateral_Segments_ISEG_0)
+{
+    const auto cse = SimulationCase { multilaterals() };
+
+    const auto& es    = cse.es;
+    const auto& grid  = cse.grid;
+    const auto& sched = cse.sched;
+    const auto& units = es.getUnits();
+    const auto  smry  = Opm::SummaryState { Opm::TimeService::now() };
+
+    // Report Step 1: 2023-09-29 --> 2023-10-23
+    const auto rptStep = std::size_t {1};
+
+    const double secs_elapsed = 30 * 86'400.0;
+    const auto ih = Opm::RestartIO::Helpers::
+        createInteHead(es, grid, sched, secs_elapsed,
+                       rptStep, rptStep + 1, rptStep);
+
+    const auto xw = Opm::data::Wells {};
+
+    auto amswd = Opm::RestartIO::Helpers::AggregateMSWData {ih};
+    amswd.captureDeclaredMSWData(sched, rptStep, units,
+                                 ih, grid, smry, xw);
+
+    auto isegOffset = [&ih](const int ix)
+    {
+        return ih[VI::intehead::NISEGZ] * ix;
+    };
+
+    const auto expect = std::vector {
+        24, 23, 22, 20,         // Branch (5)
+        21,                     // Branch (6)
+        16, 15, 14, 13, 12, 11, // Branch (2)
+        19, 18, 17,             // Branch (4)
+        10,  9,  8,  7,         // Branch (3)
+         6,  5,  4,  3,  2,  1, // Branch (1)
+    };
+
+    const auto& iseg = amswd.getISeg();
+
+    for (auto i = 0*expect.size(); i < expect.size(); ++i) {
+        BOOST_CHECK_MESSAGE(iseg[isegOffset(i)] == expect[i],
+                            "ISEG[0](" << i << ") == "
+                            << iseg[isegOffset(i)]
+                            << " differs from expected value "
+                            << expect[i]);
+    }
+}
+
+BOOST_AUTO_TEST_CASE(MSW_AICD)
+{
     const auto simCase = SimulationCase {first_sim("TEST_AGGREGATE_MSW.DATA")};
 
-    Opm::EclipseState es = simCase.es;
-    Opm::Runspec rspec   = es.runspec();
-    Opm::SummaryState smry = sim_state();
-    Opm::Schedule     sched = simCase.sched;
-    Opm::EclipseGrid  grid = simCase.grid;
-    const auto& units    = es.getUnits();
-
+    const auto& es    = simCase.es;
+    const auto& grid  = simCase.grid;
+    const auto& sched = simCase.sched;
+    const auto& units = es.getUnits();
+    const auto  smry  = sim_state();
 
     // Report Step 1: 2008-10-10 --> 2011-01-20
     const auto rptStep = std::size_t {1};
 
-    double secs_elapsed = 3.1536E07;
+    const double secs_elapsed = 3.1536E07;
     const auto ih = Opm::RestartIO::Helpers::
-                    createInteHead(es, grid, sched, secs_elapsed,
-                                   rptStep, rptStep+1, rptStep);
-
+        createInteHead(es, grid, sched, secs_elapsed,
+                       rptStep, rptStep + 1, rptStep);
 
     const Opm::data::Wells wrc = wr();
-    auto amswd = Opm::RestartIO::Helpers::AggregateMSWData {ih};
-    amswd.captureDeclaredMSWData(simCase.sched,
-                                 rptStep,
-                                 units,
-                                 ih,
-                                 grid,
-                                 smry,
-                                 wrc
-                                );
 
-// ISEG (PROD)
+    auto amswd = Opm::RestartIO::Helpers::AggregateMSWData {ih};
+    amswd.captureDeclaredMSWData(sched, rptStep, units,
+                                 ih, grid, smry, wrc);
+
+    // ISEG (PROD)
     {
         const auto& iSeg = amswd.getISeg();
         auto start = 7*ih[VI::intehead::NISEGZ];
@@ -507,48 +770,38 @@ BOOST_AUTO_TEST_CASE(MSW_AICD) {
         BOOST_CHECK_CLOSE(rseg[i0 + VI::RSeg::index::flowFractionOilViscosityExponent], 1.01 , 1.0e-10);
         BOOST_CHECK_CLOSE(rseg[i0 + VI::RSeg::index::flowFractionWaterViscosityExponent], 1.02 , 1.0e-10);
         BOOST_CHECK_CLOSE(rseg[i0 + VI::RSeg::index::flowFractionGasViscosityExponent], 1.03  , 1.0e-10);
-
     }
-
 }
 
-
-BOOST_AUTO_TEST_CASE(MSW_RST) {
+BOOST_AUTO_TEST_CASE(MSW_RST)
+{
     const auto simCase = SimulationCase {first_sim("TEST_AGGREGATE_MSW.DATA")};
 
-    Opm::EclipseState es = simCase.es;
-    Opm::Runspec rspec   = es.runspec();
-    Opm::SummaryState smry = sim_state();
-    Opm::Schedule     sched = simCase.sched;
-    Opm::EclipseGrid  grid = simCase.grid;
-    const auto& units    = es.getUnits();
-
+    const auto& es    = simCase.es;
+    const auto& grid  = simCase.grid;
+    const auto& sched = simCase.sched;
+    const auto& units = es.getUnits();
+    const auto  smry  = sim_state();
 
     // Report Step 1: 2008-10-10 --> 2011-01-20
     const auto rptStep = std::size_t {1};
 
-    double secs_elapsed = 3.1536E07;
+    const double secs_elapsed = 3.1536E07;
     const auto ih = Opm::RestartIO::Helpers::
-                    createInteHead(es, grid, sched, secs_elapsed,
-                                   rptStep, rptStep+1, rptStep);
-
+        createInteHead(es, grid, sched, secs_elapsed,
+                       rptStep, rptStep + 1, rptStep);
 
     const Opm::data::Wells wrc = wr();
+
     auto amswd = Opm::RestartIO::Helpers::AggregateMSWData {ih};
-    amswd.captureDeclaredMSWData(simCase.sched,
-                                 rptStep,
-                                 units,
-                                 ih,
-                                 grid,
-                                 smry,
-                                 wrc
-                                );
+    amswd.captureDeclaredMSWData(sched, rptStep, units,
+                                 ih, grid, smry, wrc);
 
     const auto& iseg = amswd.getISeg();
     const auto& rseg = amswd.getRSeg();
-    auto segment = Opm::RestartIO::RstSegment(simCase.es.getUnits(), 1, iseg.data(), rseg.data());
+
+    auto segment = Opm::RestartIO::RstSegment(simCase.es.getUnits(), 1,
+                                              iseg.data(), rseg.data());
 }
 
-
-
-BOOST_AUTO_TEST_SUITE_END()
+BOOST_AUTO_TEST_SUITE_END()     // Aggregate_MSW
