@@ -64,6 +64,7 @@
 #include <opm/input/eclipse/Schedule/Events.hpp>
 #include <opm/input/eclipse/Schedule/Group/GConSale.hpp>
 #include <opm/input/eclipse/Schedule/Group/GConSump.hpp>
+#include <opm/input/eclipse/Schedule/Group/GroupEconProductionLimits.hpp>
 #include <opm/input/eclipse/Schedule/Group/Group.hpp>
 #include <opm/input/eclipse/Schedule/Group/GuideRateConfig.hpp>
 #include <opm/input/eclipse/Schedule/Group/GuideRateModel.hpp>
@@ -75,6 +76,7 @@
 #include <opm/input/eclipse/Schedule/MSW/WellSegments.hpp>
 #include <opm/input/eclipse/Schedule/Network/Balance.hpp>
 #include <opm/input/eclipse/Schedule/Network/ExtNetwork.hpp>
+#include <opm/input/eclipse/Schedule/Well/FilterCake.hpp>
 #include <opm/input/eclipse/Schedule/Network/Node.hpp>
 #include <opm/input/eclipse/Schedule/OilVaporizationProperties.hpp>
 #include <opm/input/eclipse/Schedule/RFTConfig.hpp>
@@ -108,6 +110,7 @@
 #include <opm/input/eclipse/Schedule/Well/WellTestState.hpp>
 #include <opm/input/eclipse/Schedule/Well/WList.hpp>
 #include <opm/input/eclipse/Schedule/Well/WListManager.hpp>
+#include <opm/input/eclipse/Schedule/Well/WVFPDP.hpp>
 #include <opm/input/eclipse/Schedule/Well/WVFPEXP.hpp>
 #include <opm/input/eclipse/Schedule/WriteRestartFileEvents.hpp>
 #include <opm/input/eclipse/EclipseState/SimulationConfig/BCConfig.hpp>
@@ -211,6 +214,8 @@ TEST_FOR_TYPE_NAMED(data::SegmentPhaseQuantity, SegmentPhaseQuantity)
 TEST_FOR_TYPE_NAMED(data::Solution, Solution)
 TEST_FOR_TYPE_NAMED(data::Well, dataWell)
 TEST_FOR_TYPE_NAMED(data::Wells, Wells)
+TEST_FOR_TYPE_NAMED(data::WellBlockAvgPress, dataWBPObject)
+TEST_FOR_TYPE_NAMED(data::WellBlockAveragePressures, dataWBPCollection)
 TEST_FOR_TYPE(Deck)
 TEST_FOR_TYPE(DeckItem)
 TEST_FOR_TYPE(DeckKeyword)
@@ -224,7 +229,9 @@ TEST_FOR_TYPE(EndpointScaling)
 TEST_FOR_TYPE(Eqldims)
 TEST_FOR_TYPE(Equil)
 TEST_FOR_TYPE(TLMixpar)
+TEST_FOR_TYPE(Ppcwmax)
 TEST_FOR_TYPE(Events)
+TEST_FOR_TYPE(FilterCake)
 TEST_FOR_TYPE(Fault)
 TEST_FOR_TYPE(FaultCollection)
 TEST_FOR_TYPE(FaultFace)
@@ -233,6 +240,7 @@ TEST_FOR_TYPE(FoamConfig)
 TEST_FOR_TYPE(FoamData)
 TEST_FOR_TYPE(GConSale)
 TEST_FOR_TYPE(GConSump)
+TEST_FOR_TYPE(GroupEconProductionLimits)
 TEST_FOR_TYPE(GridDims)
 TEST_FOR_TYPE(Group)
 TEST_FOR_TYPE_NAMED(Group::GroupInjectionProperties, GroupInjectionProperties)
@@ -246,6 +254,7 @@ TEST_FOR_TYPE(KeywordLocation)
 TEST_FOR_TYPE(MessageLimits)
 TEST_FOR_TYPE(MULTREGTScanner)
 TEST_FOR_TYPE(NNC)
+TEST_FOR_TYPE_NAMED(Network::ExtNetwork, ExtNetwork)
 TEST_FOR_TYPE_NAMED(Network::Node, NetworkNode)
 TEST_FOR_TYPE(OilVaporizationProperties)
 TEST_FOR_TYPE(PAvg)
