@@ -46,7 +46,6 @@
 #include <opm/input/eclipse/EclipseState/Grid/EclipseGrid.hpp>
 #include <opm/input/eclipse/EclipseState/EclipseState.hpp>
 #include <opm/input/eclipse/Schedule/Schedule.hpp>
-#include <opm/input/eclipse/Schedule/Well/PAvgCalculatorCollection.hpp>
 #include <opm/input/eclipse/Schedule/Well/Well.hpp>
 #include <opm/input/eclipse/EclipseState/SummaryConfig/SummaryConfig.hpp>
 #include <opm/input/eclipse/Parser/Parser.hpp>
@@ -172,13 +171,16 @@ static data::Wells result_wells() {
       syncronized with the global index in the COMPDAT keyword in the
       input deck.
     */
-    data::Connection well1_comp1 { 0  , crates1, 1.9 , 123.4, 314.15, 0.35, 0.25, 2.718e2, 0.12345};
+    data::ConnectionFiltrate con_filtrate {0.1, 1, 3, 0.4, 1.e-9, 0.2, 0.05, 10.}; // values are not tested in this test
+    data::Connection well1_comp1 { 0  , crates1, 1.9 , 123.4, 314.15, 0.35, 0.25, 2.718e2, 0.12345, con_filtrate };
 
     /*
       The completions
     */
+    data::WellFiltrate well_filtrate {0.1, 1., 0.3}; // values are not tested in this test
     data::Well well1 {
         rates1, 0.1 * ps, 0.2 * ps, 0.3 * ps, 1,
+        well_filtrate,
         ::Opm::Well::Status::OPEN,
         { {well1_comp1} },
         { { segment.segNumber, segment } },
@@ -214,34 +216,35 @@ static data::GroupAndNetworkValues result_group_network() {
     return grp_nwrk;
 }
 
-
-struct setup {
+struct setup
+{
     Deck deck;
     EclipseState es;
     const EclipseGrid& grid;
-    std::shared_ptr<Python> python;
     Schedule schedule;
     SummaryConfig config;
     data::Wells wells;
+    data::WellBlockAveragePressures wbp;
     data::GroupAndNetworkValues grp_nwrk;
     std::string name;
     WorkArea ta;
 
-    /*-----------------------------------------------------------------*/
+    // ------------------------------------------------------------------------
 
-    setup(std::string fname, const std::string& path = "UDQ_ACTIONX_TEST1_U.DATA") :
-        deck( Parser().parseFile( path) ),
-        es( deck ),
-        grid( es.getInputGrid() ),
-        python( std::make_shared<Python>() ),
-        schedule( deck, es, python),
-        config( deck, schedule, es.fieldProps(), es.aquifer() ),
-        wells( result_wells() ),
-        grp_nwrk( result_group_network() ),
-        name( toupper(std::move(fname)) ),
-        ta( "test_summary_group_constraints" )
+    explicit setup(std::string        case_name,
+                   const std::string& path = "UDQ_ACTIONX_TEST1_U.DATA")
+        : deck     { Parser{}.parseFile(path) }
+        , es       { deck }
+        , grid     { es.getInputGrid() }
+        , schedule { deck, es, std::make_shared<Python>() }
+        , config   { deck, schedule, es.fieldProps(), es.aquifer() }
+        , wells    { result_wells() }
+        , wbp      {}
+        , grp_nwrk { result_group_network() }
+        , name     { toupper(std::move(case_name)) }
+        , ta       { "test_summary_group_constraints" }
     {}
-    };
+};
 } // Anonymous namespace
 
 // =====================================================================
@@ -262,10 +265,10 @@ BOOST_AUTO_TEST_CASE(group_keywords) {
     SummaryState st(TimeService::now());
 
     out::Summary writer( cfg.es, cfg.config, cfg.grid, cfg.schedule , cfg.name );
-    writer.eval(st, 0, 0*day, cfg.wells, cfg.grp_nwrk, {}, {}, {}, {});
+    writer.eval(st, 0, 0*day, cfg.wells, cfg.wbp, cfg.grp_nwrk, {}, {}, {}, {});
     writer.add_timestep( st, 0, false);
 
-    writer.eval(st, 1, 1*day, cfg.wells, cfg.grp_nwrk, {}, {}, {}, {});
+    writer.eval(st, 1, 1*day, cfg.wells, cfg.wbp, cfg.grp_nwrk, {}, {}, {}, {});
     writer.add_timestep( st, 1, false);
 
     writer.write();
