@@ -45,6 +45,7 @@
 #include <opm/input/eclipse/Schedule/Action/State.hpp>
 #include <opm/input/eclipse/Schedule/Schedule.hpp>
 #include <opm/input/eclipse/Schedule/SummaryState.hpp>
+#include <opm/input/eclipse/Schedule/Well/Well.hpp>
 #include <opm/input/eclipse/Schedule/Well/WellEconProductionLimits.hpp>
 #include <opm/input/eclipse/Schedule/Well/WellTestState.hpp>
 #include <opm/input/eclipse/Schedule/Well/WVFPEXP.hpp>
@@ -661,7 +662,7 @@ BOOST_AUTO_TEST_CASE(Construct_Well_Guide_Rates_Group_Control_Object)
     const auto state =
         makeRestartState(simCase, baseName, rptStep, "test_rst_wgrupcon");
 
-    auto makeRestartWell = [&state, rptStep](const std::string& well_name)
+    auto makeRestartWell = [&state](const std::string& well_name)
     {
         return Opm::Well {
             state.get_well(well_name),
@@ -745,7 +746,7 @@ BOOST_AUTO_TEST_CASE(Construct_Well_Explicit_THP_Control_Options_Object)
     const auto state =
         makeRestartState(simCase, baseName, rptStep, "test_rst_wvfpexp");
 
-    auto makeTHPOptions = [&state, rptStep](const std::string& well_name)
+    auto makeTHPOptions = [&state](const std::string& well_name)
     {
         return Opm::Well {
             state.get_well(well_name),
